@@ -1,5 +1,5 @@
 /*  AlsaNode.h
- *  Copyright (C) 1998-2002 Andy Lo A Foe <andy@alsaplayer.org>
+ *  Copyright (C) 1998-2002 Andy Lo A Foe <andy@loafoe.com>
  *
  *  This file is part of AlsaPlayer
  *
@@ -16,10 +16,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
- *
- *  $Id: AlsaNode.h 1250 2007-07-08 14:17:12Z dominique_libre $
- * 
-*/ 
+*/
 
 #ifndef __AlsaNode_h__
 #define __AlsaNode_h__
@@ -33,9 +30,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <pthread.h>
-#ifdef USE_ESD
-#include <esd.h>
-#endif
 
 #include "output_plugin.h"
 
@@ -62,18 +56,18 @@ class AlsaNode
 	output_plugin plugins[MAX_OUTPUT_PLUGINS];
 	output_plugin *plugin;
 	int plugin_count;
-	subscriber subs[MAX_SUB]; 
-	pthread_mutex_t queue_mutex; 
+	subscriber subs[MAX_SUB];
+	pthread_mutex_t queue_mutex;
 	pthread_mutex_t thread_mutex;
 	void *handle;
 	int count;
-	int follow_id; 
+	int follow_id;
 	int fragment_size;
 	int nr_fragments;
 	int sample_freq;
 	int external_latency;
-	char *driver_name;
-	char *driver_args;
+	char driver_name [512];
+	char driver_args [1024];
 	char client_name[32];
 	bool realtime_sched;
 	bool thread_running;
@@ -81,7 +75,7 @@ class AlsaNode
 	bool looping;
 	static void looper(void *);
 	pthread_t looper_thread;
- public:		
+ public:
 	AlsaNode(const char *name, const char *args, int realtime=0);
 	~AlsaNode();
 	int SetSamplingRate(int freq);
