@@ -1,5 +1,5 @@
 /*  AlsaPlayer.h
- *  Copyright (C) 2002 Andy Lo A Foe <andy@alsaplayer.org>
+ *  Copyright (C) 2002 Andy Lo A Foe <andy@loafoe.com>
  *
  *  This file is part of AlsaPlayer.
  *
@@ -24,7 +24,7 @@
 extern char *global_session_name;
 extern char *global_plugin_dir;
 extern char *global_interface_script;
-extern char *global_pluginroot;
+extern const char *global_pluginroot;
 extern int global_verbose;
 extern int global_session_id;
 extern int global_quiet;
