@@ -1,24 +1,22 @@
 /*  SampleBuffer.h
- *  Copyright (C) 1998-2002 Andy Lo A Foe <andy@alsaplayer.org>
+ *  Copyright (C) 1998-2002 Andy Lo A Foe <andy@loafoe.com>
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This file is part of AlsaPlayer.
+ *
+ *  AlsaPlayer is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation; either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  AlsaPlayer is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
- *
- *  $Id: SampleBuffer.h 375 2002-03-30 01:52:03Z adnans $
- *
-*/ 
+*/
 
 #ifndef __SampleBuffer_h__
 #define __SampleBuffer_h__
@@ -49,23 +47,22 @@ class SampleBuffer {
         int read_direction;
         //int read_index;
         //int write_index;
-        //char *buffer_data;
-	pthread_mutex_t	lock;	
+	pthread_mutex_t	lock;
  public:
 
 	int read_index;
 	int write_index;
-	char *buffer_data;
-	
+	short *buffer_data;
+
 	SampleBuffer(int mode, int size);
 	~SampleBuffer();
-	
+
 	int WriteSamples(void *data, int length);
         int ReadSamples(void *data, int length);
 	int Seek(int index);
 	void SetReadDirection(int rd);
 	void ResetRead();
-	int GetBufferSizeBytes(int frame_size = -1);
+	int GetBufferSizeBytes(int block_size = -1);
 	int GetBufferSize();
 	int GetSamplesInBuffer();
 	int GetReadIndex();
@@ -75,7 +72,7 @@ class SampleBuffer {
 	int GetFreeSamples();
 	int GetAvailableSamples();
 	int GetSampleSize() { return sample_size; }
-	void Clear(); 
+	void Clear();
 };
 
 #endif
