@@ -1,5 +1,5 @@
 /*  interface_plugin.h
- *  Copyright (C) 2001-2002 Andy Lo A Foe <andy@alsaplayer.org>
+ *  Copyright (C) 2001-2002 Andy Lo A Foe <andy@loafoe.com>
  *
  *  This file is part of AlsaPlayer.
  *
@@ -16,9 +16,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
- * $Id: interface_plugin.h 1250 2007-07-08 14:17:12Z dominique_libre $
- * 
- */ 
+ */
 
 #ifndef __interface_plugin_h__
 #define __interface_plugin_h__
@@ -45,8 +43,8 @@ typedef void(*interface_close_type)();
 typedef struct _interface_plugin
 {
 	interface_version_type version;		
-	char *name;
-	char *author;
+	const char *name;
+	const char *author;
 	void *handle;
 	interface_init_type init;
 	interface_start_type start;
