@@ -1,5 +1,5 @@
 /*  output_plugin.h
- *  Copyright (C) 1999-2002 Andy Lo A Foe <andy@alsaplayer.org>
+ *  Copyright (C) 1999-2002 Andy Lo A Foe <andy@loafoe.com>
  *
  *  This file is part of AlsaPlayer.
  *
@@ -16,9 +16,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
- *  $Id: output_plugin.h 1250 2007-07-08 14:17:12Z dominique_libre $
- *  
- */ 
+ */
 
 #ifndef __output_plugin_h__
 #define __output_plugin_h__
@@ -30,7 +28,7 @@ typedef int output_version_type;
 typedef int(*output_init_type)(void);
 typedef int(*output_open_type)(const char *path);
 typedef void(*output_close_type)(void);
-typedef int(*output_write_type)(void *data, int byte_count);
+typedef int(*output_write_type)(short *data, int short_count);
 typedef int(*output_start_callbacks_type)(void *data);
 typedef int(*output_set_buffer_type)(int *frag_size, int *frag_count, int *channels);
 typedef unsigned int(*output_set_sample_rate_type)(unsigned int rate);
@@ -41,21 +39,21 @@ typedef struct _output_plugin
 {
 	/**
 	 * Version of output plugin. Must be OUTPUT_PLUGIN_VERSION
-	 */ 
+	 */
 	output_version_type version;
 
 	/**
 	 * Name of output plugin
-	 */ 
-	char *name;
+	 */
+	const char *name;
 
 	/**
 	 * Author of the plugin
-	 */ 
-	char *author;
+	 */
+	const char *author;
 
 	/**
-	 * Initialize output plugin. Called before the plugin is 
+	 * Initialize output plugin. Called before the plugin is
 	 * opened for use
 	 */
 	output_init_type init;
@@ -63,24 +61,24 @@ typedef struct _output_plugin
 	/**
 	 * @param path The path or device designation that should be used
 	 *
-	 * Opens the output plugin. A value of 1 should be returned on 
+	 * Opens the output plugin. A value of 1 should be returned on
 	 * success, 0 on failure.
 	 */
 	output_open_type open;
 
 	/**
 	 * Close the output plugin
-	 */ 
+	 */
 	output_close_type close;
 
 	/**
 	 * @param data Buffer that contains the data
-	 * @byte_count Number of bytes that should be read from the buffer
+	 * @param byte_count Number of bytes that should be read from the buffer
 	 *
 	 * Write out data to the output device. This is a byte count and
 	 * will typically be the same size as a fragment. A value of 1 should
 	 * be returned on success, 0 on failure.
-	 */ 
+	 */
 	output_write_type write;
 
 	/**
@@ -89,7 +87,7 @@ typedef struct _output_plugin
 	 * This function is used for callback based plugins like JACK
 	 */
 	output_start_callbacks_type start_callbacks;
-	
+
 	/**
 	 * @param frag_size Fragment size to use (in bytes)
 	 * @param frag_count Fragment count to use (in bytes)
@@ -102,7 +100,7 @@ typedef struct _output_plugin
 	 * 0 on failure.
 	 */
 	output_set_buffer_type set_buffer;
-	
+
 	/**
 	 * @param rate Sample rate to use
 	 *
@@ -116,14 +114,19 @@ typedef struct _output_plugin
 	 * output device. This function is optional.
 	 */
 	output_get_queue_count_type get_queue_count;
-	
+
 	/**
 	 * Returns the latency of the output device in bytes. This function
 	 * is optional.
-	 */ 
+	 */
 	output_get_latency_type get_latency;
 } output_plugin;
 
 typedef output_plugin*(*output_plugin_info_type)(void);
 
+#ifdef __cplusplus
+extern "C"
+#endif
+output_plugin* output_plugin_info (void);
+
 #endif
