@@ -1,5 +1,5 @@
 /*  prefs.h - Preferences system header
- *  Copyright (C) 2002 Andy Lo A Foe <andy@alsaplayer.org>
+ *  Copyright (C) 2002 Andy Lo A Foe <andy@loafoe.com>
  *
  *  This file is part of AlsaPlayer.
  *
@@ -18,11 +18,7 @@
  *
  *  Inspired by xine's prefs 
  *
- *
- *
- *  $Id: prefs.h 1250 2007-07-08 14:17:12Z dominique_libre $
- *
-*/ 
+*/
 #ifndef __prefs_h__
 #define __prefs_h__
 
@@ -52,15 +48,15 @@ typedef struct _prefs_handle prefs_handle_t;
 prefs_handle_t *prefs_load(const char *filename);
 void prefs_free(prefs_handle_t *prefs);
 
-void prefs_set_int(prefs_handle_t *prefs, const char *section, char *key, int val);
-void prefs_set_string(prefs_handle_t *prefs, const char *section, char *key, char *val);
-void prefs_set_float(prefs_handle_t *prefs, const char *section, char *key, float val);
-void prefs_set_bool(prefs_handle_t *prefs, const char *section, char *key, int val);
+void prefs_set_int(prefs_handle_t *prefs, const char *section, const char *key, int val);
+void prefs_set_string(prefs_handle_t *prefs, const char *section, const char *key, const char *val);
+void prefs_set_float(prefs_handle_t *prefs, const char *section, const char *key, float val);
+void prefs_set_bool(prefs_handle_t *prefs, const char *section, const char *key, int val);
 
-int prefs_get_int(prefs_handle_t *prefs, const char *section, char *key, int default_val);
-char *prefs_get_string(prefs_handle_t *prefs, const char *section, char *key, char *default_val);
-float prefs_get_float(prefs_handle_t *prefs, const char *section, char *key, float default_val);
-int prefs_get_bool(prefs_handle_t *prefs, const char *section, char *key, int default_val);
+int prefs_get_int(prefs_handle_t *prefs, const char *section, const char *key, int default_val);
+const char *prefs_get_string(prefs_handle_t *prefs, const char *section, const char *key, const char *default_val);
+float prefs_get_float(prefs_handle_t *prefs, const char *section, const char *key, float default_val);
+int prefs_get_bool(prefs_handle_t *prefs, const char *section, const char *key, int default_val);
 
 prefs_key_t* prefs_sort(prefs_handle_t *prefs);
 
