@@ -1,5 +1,5 @@
 /*  ControlSocket.cpp
- *  Copyright (C) 2002 Andy Lo A Foe <andy@alsaplayer.org>
+ *  Copyright (C) 2002 Andy Lo A Foe <andy@loafoe.com>
  *
  *  This file is part of AlsaPlayer.
  *
@@ -16,8 +16,9 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
- */ 
+ */
 #include "AlsaPlayer.h"
+#include <climits>
 #include <cstdio>
 #include <cstdlib>
 #include <unistd.h>
@@ -28,6 +29,7 @@
 #include <sys/un.h>
 #include <csignal>
 #include <pwd.h>
+#include <limits.h>
 #include "control.h"
 #include "message.h"
 #include "Playlist.h"
@@ -61,7 +63,6 @@ static void socket_looper(void *arg)
 	char strnum[64];
 	char tmp[512];
 	long total_time;
-	void *data;
 	float *float_val;
 	char *path;
 	int32_t *int_val;
@@ -92,15 +93,15 @@ static void socket_looper(void *arg)
 	saddr.sun_family = AF_UNIX;
 	while (session_id < MAX_AP_SESSIONS && !session_ok) {
 		if (!ap_ping(session_id)) {
-			sprintf(saddr.sun_path, "/tmp/alsaplayer_%s_%d", pwd == NULL ?
+			snprintf(saddr.sun_path, sizeof (saddr.sun_path), "/tmp/alsaplayer_%s_%d", pwd == NULL ?
 					"anonymous" : pwd->pw_name, session_id);
 			unlink(saddr.sun_path); // Clean up a bit
 		} else {
 			// Live session so skip it immediately
 			session_id++;
 			continue;
-		}	
-		sprintf(saddr.sun_path, "/tmp/alsaplayer_%s_%d", pwd == NULL ? 
+		}
+		snprintf(saddr.sun_path, sizeof (saddr.sun_path), "/tmp/alsaplayer_%s_%d", pwd == NULL ?
 				"anonymous" : pwd->pw_name, session_id);
 		if (bind(socket_fd, (struct sockaddr *) &saddr, sizeof (saddr)) != -1) {
 			chmod(saddr.sun_path, 00600); // Force permission
@@ -113,14 +114,13 @@ static void socket_looper(void *arg)
 	if (!session_ok) {
 		alsaplayer_error("Out of alsaplayer sockets (MAX = %d)", MAX_AP_SESSIONS);
 		return;
-	}	
+	}
 	global_session_id = session_id;
 
 	while (socket_thread_running) {
 		FD_ZERO(&set);
 		FD_SET(socket_fd, &set);
 		len = sizeof (saddr);
-		data = NULL;
 
 		if ((select(socket_fd + 1, &set, NULL, NULL, NULL) <= 0) ||
 				((fd = accept(socket_fd, (struct sockaddr *) &saddr, &len)) == -1))
@@ -128,18 +128,24 @@ static void socket_looper(void *arg)
 		// So we have a connection
 		msg = ap_message_receive(fd);
 
+		// Check for broken connection
+		if (!msg) {
+			close(fd);
+			continue;
+		}
+
 		// Check version
 		if (msg->header.version != AP_CONTROL_VERSION) {
 			alsaplayer_error("protocol version mismatch (server): %x != %x",
 				msg->header.version, AP_CONTROL_VERSION);
 			close(fd);
 			continue;
-		}	
+		}
 
 		ap_message_t *reply = ap_message_new();
 
 		//alsaplayer_error("server: got something (%x)", msg->header.cmd);
-	
+
 		player = playlist->GetCorePlayer();
 
 		nr_requests++;
@@ -166,16 +172,16 @@ static void socket_looper(void *arg)
 			case AP_ADD_PATH:
 				if ((path = ap_message_find_string(msg, "path1"))) {
 					playlist->Insert(path, playlist->Length());
-				}	
+				}
 				ap_message_add_int32(reply, "ack", 1);
 				break;
 			case AP_ADD_PLAYLIST:
 				if ((path = ap_message_find_string(msg, "path1"))) {
 					playlist->Load(path, playlist->Length(), 0);
-				}	
+				}
 				ap_message_add_int32(reply, "ack", 1);
 				break;
-			case AP_PLAY: 
+			case AP_PLAY:
 				if (player) {
 					playlist->UnPause();
 					if (player->IsPlaying()) {
@@ -195,7 +201,7 @@ static void socket_looper(void *arg)
 				}
 				ap_message_add_int32(reply, "ack", 0);
 				break;
-			case AP_NEXT: 
+			case AP_NEXT:
 				playlist->Next();
 				playlist->UnPause();
 				ap_message_add_int32(reply, "ack", 1);
@@ -212,7 +218,7 @@ static void socket_looper(void *arg)
 					ap_message_add_int32(reply, "ack", 1);
 				} else {
 					ap_message_add_int32(reply, "ack", 0);
-				}	
+				}
 				break;
 			case AP_SHUFFLE_PLAYLIST:
 				playlist->Shuffle();
@@ -230,7 +236,7 @@ static void socket_looper(void *arg)
 				break;
 			case AP_UNPAUSE:
 				playlist->UnPause();
-				if (player) 
+				if (player)
 					player->UnPause();
 				ap_message_add_int32(reply, "ack", 1);
 				break;
@@ -258,15 +264,15 @@ static void socket_looper(void *arg)
 					player = playlist->GetCorePlayer();
 					if (player) {
 						player->SetSpeed(*float_val);
-						ap_message_add_int32(reply, "ack", 1);	
-					}	
+						ap_message_add_int32(reply, "ack", 1);
+					}
 				}
 				break;
 			case AP_GET_SPEED:
 				if (player) {
 					ap_message_add_float(reply, "speed", player->GetSpeed());
 					ap_message_add_int32(reply, "ack", 1);
-				}	
+				}
 				break;
 			case AP_IS_PAUSED:
                                 ap_message_add_int32(reply, "int", playlist->IsPaused());
@@ -276,7 +282,7 @@ static void socket_looper(void *arg)
 				if (player) {
 					ap_message_add_int32(reply, "int", player->IsPlaying());
 					ap_message_add_int32(reply, "ack", 1);
-				}	
+				}
 				break;
 			case AP_IS_LOOPING:
 				ap_message_add_int32(reply, "int", playlist->LoopingSong());
@@ -292,28 +298,42 @@ static void socket_looper(void *arg)
 					ap_message_add_int32(reply, "ack", 1);
 				}
 				break;
+			case AP_IS_ONEBYONE:
+				ap_message_add_int32(reply, "int", playlist->IsOneByOne());
+                                ap_message_add_int32(reply, "ack", 1);
+                                break;
+			case AP_SET_ONEBYONE:
+                                if ((int_val = ap_message_find_int32(msg, "int"))) {
+                                        if (*int_val) {
+                                                playlist->SetOneByOne();
+                                        } else {
+                                                playlist->UnSetOneByOne();
+                                        }
+                                        ap_message_add_int32(reply, "ack", 1);
+                                }
+                                break;
 			case AP_GET_PLAYLIST:
 				playlist->Lock();
 				playlist_length = playlist->Length();
 				ap_message_add_int32(reply, "items", playlist->Length());
 				for (pc=1; pc <= playlist_length; pc++) {
-					sprintf(strnum, "%d", pc);
+					snprintf(strnum, sizeof (strnum), "%d", pc);
 					item = playlist->GetItem(pc);
 					if (item->title.size()) {
-						sprintf(tmp, "%s %s", item->title.c_str(), 
-			                        item->artist.size() ? (std::string("- ") + 
+						snprintf(tmp, sizeof (tmp), "%s %s", item->title.c_str(),
+			                        item->artist.size() ? (std::string("- ") +
 						item->artist).c_str() : "");
 						ap_message_add_string(reply, strnum, tmp);
 					} else {
-						sprintf(tmp, "%s", item->filename.c_str());
+						snprintf(tmp, sizeof (tmp), "%s", item->filename.c_str());
 						path = strrchr(tmp, '/');
 						if (path) {
 							path++;
 						} else {
 							path = tmp;
-						}	
+						}
 						ap_message_add_string(reply, strnum, path);
-					}	
+					}
 				}
 				playlist->Unlock();
 				ap_message_add_int32(reply, "ack", 1);
@@ -344,7 +364,7 @@ static void socket_looper(void *arg)
 				if (player) {
 					ap_message_add_float(reply, "float",  player->GetVolume());
 					ap_message_add_int32(reply, "ack", 1);
-				}	
+				}
 				break;
 			case AP_SET_PAN:
 				if (player) {
@@ -358,7 +378,7 @@ static void socket_looper(void *arg)
 				if (player) {
 					ap_message_add_float(reply, "float",  player->GetPan());
 					ap_message_add_int32(reply, "ack", 1);
-				}	
+				}
 				break;
 			case AP_GET_TITLE:
 				if (player) {
@@ -440,32 +460,32 @@ static void socket_looper(void *arg)
 				if (global_session_name) {
 					ap_message_add_string(reply, "string", global_session_name);
 				} else {
-					sprintf(session_name, "alsaplayer-%d", global_session_id);
+					snprintf(session_name, sizeof (session_name), "alsaplayer-%d", global_session_id);
 					ap_message_add_string(reply, "string", session_name);
 				}
 				ap_message_add_int32(reply, "ack", 1);
 				break;
-			case AP_GET_POS_SECOND:	
+			case AP_GET_POS_SECOND:
 				if (player) {
-					ap_message_add_int32(reply, "int", 
+					ap_message_add_int32(reply, "int",
 						player->GetCurrentTime() / 100);
 					ap_message_add_int32(reply, "ack", 1);
-				}	
+				}
 				break;
 			case AP_SET_POS_SECOND_RELATIVE:
 				if (player) {
 					if ((int_val = ap_message_find_int32(msg, "int"))) {
-						fsize = player->GetFrameSize();
+						fsize = player->GetBlockSize();
 						if (fsize) {
 							*int_val += ( player->GetCurrentTime() / 100);
 							*int_val *= player->GetSampleRate();
 							*int_val /= fsize;
 							*int_val *= player->GetChannels();
-							*int_val *= 2; // 16-bit ("2" x 8-bit)a
+							// *int_val *= 2; // 16-bit ("2" x 8-bit)a
 							if (*int_val < 0)
 								*int_val = 0;
 							player->Seek(*int_val);
-						}	
+						}
 					}
 				}
 				ap_message_add_int32(reply, "ack", 1);
@@ -473,14 +493,14 @@ static void socket_looper(void *arg)
 			case AP_SET_POS_SECOND:
 				if (player) {
 					if ((int_val = ap_message_find_int32(msg, "int"))) {
-						fsize = player->GetFrameSize();
+						fsize = player->GetBlockSize();
 						if (fsize) {
 							*int_val *= player->GetSampleRate();
 							*int_val /= fsize;
 							*int_val *= player->GetChannels();
-							*int_val *= 2; // 16-bit ("2" x 8-bit)
+							// *int_val *= 2; // 16-bit ("2" x 8-bit)
 							player->Seek(*int_val);
-						}	
+						}
 					}
 				}
 				ap_message_add_int32(reply, "ack", 1);
@@ -491,24 +511,24 @@ static void socket_looper(void *arg)
 				break;
 			case AP_GET_SONG_LENGTH_SECOND:
 				if (player) {
-					total_time = player->GetCurrentTime(player->GetFrames());
+					total_time = player->GetCurrentTime(player->GetBlocks());
 					ap_message_add_int32(reply, "int", (int32_t)(total_time / 100));
 					ap_message_add_int32(reply, "ack", 1);
 				}
 				break;
-			case AP_GET_SONG_LENGTH_FRAME:
+			case AP_GET_SONG_LENGTH_BLOCK:
 				if (player) {
-					ap_message_add_int32(reply, "int", player->GetFrames());
+					ap_message_add_int32(reply, "int", player->GetBlocks());
 					ap_message_add_int32(reply, "ack", 1);
 				}
 				break;
-			case AP_GET_POS_FRAME:
+			case AP_GET_POS_BLOCK:
 				if (player) {
 					ap_message_add_int32(reply, "int", player->GetPosition());
 					ap_message_add_int32(reply, "ack", 1);
 				}
 				break;
-			case AP_SET_POS_FRAME:
+			case AP_SET_POS_BLOCK:
 				if (player) {
 					if ((int_val = ap_message_find_int32(msg, "int"))) {
 						player->Seek(*int_val);
@@ -565,7 +585,7 @@ static void socket_looper(void *arg)
 					}
 				}
 				break;
-		    
+
 			default: alsaplayer_error("CMD unknown or not implemented= %x",
 					msg->header.cmd);
 				 break;
@@ -574,11 +594,11 @@ static void socket_looper(void *arg)
 		ap_message_send(fd, reply);
 		ap_message_delete(reply);
 		ap_message_delete(msg);
-		close(fd);	
+		close(fd);
 	}
 	if (global_verbose) {
 		alsaplayer_error("control: received %ld requests", nr_requests);
-	}		
+	}
 	unlink(saddr.sun_path);
 }
 
