@@ -1,5 +1,5 @@
-/*  CorePlayer.cpp - Core player object, most of the hacking is done here!  
- *	Copyright (C) 1998-2003 Andy Lo A Foe <andy@alsaplayer.org>
+/*  CorePlayer.cpp - Core player object, most of the hacking is done here!
+ *	Copyright (C) 1998-2003 Andy Lo A Foe <andy@loafoe.com>
  *
  *  This file is part of AlsaPlayer.
  *
@@ -16,9 +16,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
- *  $Id: CorePlayer.cpp 1267 2007-07-09 20:14:22Z dominique_libre $
- *
-*/ 
+*/
 
 
 /*
@@ -50,26 +48,17 @@
 #include "Effects.h"
 #include "utilities.h"
 #include "alsaplayer_error.h"
+#include "ap_string.h"
 
-extern void exit_sighandler(int);
-static char addon_dir[1024];
+//#include "MetadataReader.h" //TODO
 
-
-#if 0	// unused; Frank
-bool surround_func(void *arg, void *data, int size)
-{
-	int amount = 50;
-	int16_t *d = (int16_t *)data;
-	for (int i=0; i < size; i+=2) {
-		int16_t l =  d[i]; // Left
-		int16_t r =  d[i+1]; // Right
-		d[i] =  ((l*(256-amount))-(r*amount))/256;
-		d[i+1] = ((r*(256-amount))-(l*amount))/256;
-	}
-	return true;
-}
+// some arch don't define PATH_MAX
+#ifndef PATH_MAX
+#define PATH_MAX 1024
 #endif
 
+extern void exit_sighandler(int);
+static char addon_dir[PATH_MAX];
 
 int CorePlayer::plugin_count = 0;
 int CorePlayer::plugins_loaded = 0;
@@ -106,7 +95,7 @@ void CorePlayer::RegisterNotifier(coreplayer_notifier *core_notif, void *data)
 	if (!core_notif) {
 		alsaplayer_error("Notifier is NULL");
 		return;
-	}	
+	}
 	if (data)
 		core_notif->data = data;
 	if (core_notif->speed_changed)
@@ -132,22 +121,17 @@ void CorePlayer::UnRegisterNotifier(coreplayer_notifier *core_notif)
 	UnlockNotifiers();
 }
 
-// some arch don't define PATH_MAX
-#ifndef PATH_MAX
-#define PATH_MAX 1024
-#endif
-
 void CorePlayer::load_input_addons()
 {
-        char path[PATH_MAX];
-	
+	char path[PATH_MAX];
+
 	DIR *dir;
 	struct stat buf;
 	dirent *entry;
-	
+
 	input_plugin_info_type input_plugin_info;
 
-	sprintf(path, "%s/input", addon_dir);
+	snprintf(path, sizeof (path), "%s/input", addon_dir);
 
 	memset(plugins, 0, sizeof(plugins));
 
@@ -159,7 +143,7 @@ void CorePlayer::load_input_addons()
 		    strcmp(entry->d_name, "..") == 0) {
 				continue;
 		}
-		sprintf(path, "%s/input/%s", addon_dir, entry->d_name);
+		snprintf(path, sizeof (path), "%s/input/%s", addon_dir, entry->d_name);
 		if (stat(path, &buf)) continue;
 		if (!S_ISREG(buf.st_mode)) continue;
 
@@ -168,7 +152,7 @@ void CorePlayer::load_input_addons()
 		char *ext = strrchr(path, '.');
 		if (!ext)
 			continue;
-				
+
 		ext++;
 
 		if (strcasecmp(ext, "so"))
@@ -208,20 +192,20 @@ CorePlayer::CorePlayer(AlsaNode *the_node)
 	int i;
 
 	/* Init mutexes */
-	pthread_mutex_init(&counter_mutex, NULL); 
+	pthread_mutex_init(&counter_mutex, NULL);
 	pthread_mutex_init(&player_mutex, NULL);
 	pthread_mutex_init(&notifier_mutex, NULL);
 	pthread_cond_init(&producer_ready, NULL);
-	
-	producer_thread = 0; 
-	total_frames = 0;
+
+	producer_thread = 0;
+	total_blocks = 0;
 	streaming = false;
 	producing = false;
 	jumped = false;
 	//plugin_count = 0;
 	plugin = NULL;
-	jump_point = repitched = write_buf_changed = 0; 
-	new_frame_number = 0;
+	jump_point = repitched = write_buf_changed = 0;
+	new_block_number = 0;
 	read_direction = DIR_FORWARD;
 	node = the_node;
 	pitch = 1.0;
@@ -268,7 +252,7 @@ CorePlayer::CorePlayer(AlsaNode *the_node)
 	the_object->ready = 0;
 
 	pthread_mutex_init(&the_object->object_mutex, NULL);
-	strcpy(addon_dir, ADDON_DIR);
+	ap_strlcpy(addon_dir, ADDON_DIR, sizeof (addon_dir));
 
 	// Load the input addons
 	pthread_mutex_lock(&plugins_mutex);
@@ -280,7 +264,7 @@ CorePlayer::CorePlayer(AlsaNode *the_node)
 	if ((input_buffer = new char[128 * 1024]) == NULL) {
 		alsaplayer_error("cannot allocate mix buffer");
 		exit(1);
-	}	
+	}
 #endif
 	pthread_mutex_unlock(&plugins_mutex);
 }
@@ -322,18 +306,18 @@ void CorePlayer::UnregisterPlugins()
 
 	Stop();
 	Close();
-	
+
 	Lock();
 	for (int i = 0; i < plugin_count; i++) {
 		tmp = &plugins[i];
-		//alsaplayer_error("Unloading Input plugin: %s", tmp->name); 
+		//alsaplayer_error("Unloading Input plugin: %s", tmp->name);
 		if (tmp->handle) {
 			//tmp->shutdown(); // Shutdown plugin
 			dlclose(tmp->handle);
 			tmp->handle = NULL;
 			tmp = NULL;
-		}	
-	}		
+		}
+	}
 	Unlock();
 }
 
@@ -350,7 +334,7 @@ int CorePlayer::RegisterPlugin(input_plugin *the_plugin)
 	if (the_plugin->author == NULL) {
 		alsaplayer_error("No author");
 		error_count++;
-	}	
+	}
 	if (the_plugin->init == NULL) {
 		alsaplayer_error("No init function");
 		error_count++;
@@ -367,30 +351,34 @@ int CorePlayer::RegisterPlugin(input_plugin *the_plugin)
 		alsaplayer_error("No close function");
 		error_count++;
 	}
-	if (the_plugin->play_frame == NULL) {
-		alsaplayer_error("No play_frame function");
+	if (the_plugin->play_block == NULL) {
+		alsaplayer_error("No play_block function");
 		error_count++;
-	}	
-	if (the_plugin->frame_seek == NULL) {
-		alsaplayer_error("No frame_seek function");
+	}
+	if (the_plugin->block_seek == NULL) {
+		alsaplayer_error("No block_seek function");
 		error_count++;
 	}
 
-	if (the_plugin->nr_frames == NULL) {
-		alsaplayer_error("No nr_frames function");
+	if (the_plugin->nr_blocks == NULL) {
+		alsaplayer_error("No nr_blocks function");
 		error_count++;
 	}
-	if (the_plugin->frame_size == NULL) {
-		alsaplayer_error("No frame_size function");
+	if (the_plugin->frame_count == NULL) {
+		alsaplayer_error("No frame_count function");
+		error_count++;
+	}
+	if (the_plugin->block_size == NULL) {
+		alsaplayer_error("No block_size function");
 		error_count++;
 	}
 	if (the_plugin->channels == NULL) {
 		alsaplayer_error("No channels function");
 		error_count++;
-	}	
+	}
 	if (the_plugin->stream_info == NULL) {
 		alsaplayer_error("No stream_info function");
-		error_count++;
+//		error_count++; // we don't require this function since generic metadata parsing
 	}
 	if (the_plugin->shutdown == NULL) {
 		alsaplayer_error("No shutdown function");
@@ -400,15 +388,15 @@ int CorePlayer::RegisterPlugin(input_plugin *the_plugin)
 	    	alsaplayer_error("At least %d error(s) were detected", error_count);
 		return 0;
 	}
-	
+
 	tmp = &plugins[plugin_count];
 	tmp->version = the_plugin->version;
 	if (tmp->version) {
 		if ((version = tmp->version) != INPUT_PLUGIN_VERSION) {
 			alsaplayer_error("Wrong version number on plugin (v%d, wanted v%d)",
 					version - INPUT_PLUGIN_BASE_VERSION,
-					INPUT_PLUGIN_VERSION - INPUT_PLUGIN_BASE_VERSION);      
-			// this is a minor error actually 
+					INPUT_PLUGIN_VERSION - INPUT_PLUGIN_BASE_VERSION);
+			// this is a minor error actually
 			// no need to stop there
 //			return 0;
 		}
@@ -420,24 +408,24 @@ int CorePlayer::RegisterPlugin(input_plugin *the_plugin)
 	tmp->can_handle = the_plugin->can_handle;
 	tmp->open = the_plugin->open;
 	tmp->close = the_plugin->close;
-	tmp->play_frame = the_plugin->play_frame;
-	tmp->frame_seek = the_plugin->frame_seek;
-	tmp->frame_size = the_plugin->frame_size;
-	tmp->nr_frames = the_plugin->nr_frames;
-	tmp->frame_to_sec = the_plugin->frame_to_sec;
+	tmp->play_block = the_plugin->play_block;
+	tmp->block_seek = the_plugin->block_seek;
+	tmp->block_size = the_plugin->block_size;
+	tmp->nr_blocks = the_plugin->nr_blocks;
+	tmp->block_to_sec = the_plugin->block_to_sec;
 	tmp->sample_rate = the_plugin->sample_rate;
 	tmp->channels = the_plugin->channels;
 	tmp->stream_info = the_plugin->stream_info;
 	tmp->shutdown = the_plugin->shutdown;
 	tmp->nr_tracks = the_plugin->nr_tracks;
 	tmp->track_seek = the_plugin->track_seek;
-	
+
 	if (!tmp->init()) {
 		alsaplayer_error("Plugin failed to initialize (\"%s\")", tmp->name);
 		return 0;
-	}	
+	}
 	plugin_count++;
-	
+
 	if (global_verbose)
 		alsaplayer_error("Loading Input plugin: %s", tmp->name);
 
@@ -445,14 +433,14 @@ int CorePlayer::RegisterPlugin(input_plugin *the_plugin)
 }
 
 
-int CorePlayer::GetCurrentTime(int frame)
+int CorePlayer::GetCurrentTime(int block)
 {
 	 long result = 0;
 
 	Lock();
 	if (plugin && the_object && the_object->ready) {
-		result = plugin->frame_to_sec(the_object, frame < 0 ? GetPosition()
-			: frame);
+		result = plugin->block_to_sec(the_object, block < 0 ? GetPosition()
+			: block);
 	}
 	Unlock();
 	if (result < 0) {
@@ -470,11 +458,11 @@ int CorePlayer::GetPosition()
 		return jump_point;
 	if (read_buf && plugin && the_object && the_object->ready) {
 		if (read_buf->start < 0)
-			return 0;		
+			return 0;
 
-		int frame_size = plugin->frame_size(the_object);
-		if (frame_size)  {
-			result = (read_buf->start + (read_buf->buf->read_index * 4) / frame_size);
+		int block_size = plugin->block_size(the_object);
+		if (block_size)  {
+			result = (read_buf->start + (read_buf->buf->read_index * 4) / block_size);
 			if (result < 0) {
 				alsaplayer_error("Found the error!");
 			}
@@ -487,13 +475,13 @@ int CorePlayer::GetPosition()
 }
 
 
-int CorePlayer::GetFrames()
+int CorePlayer::GetBlocks()
 {
 	int result = 0;
 
 	Lock();
 	if (plugin && the_object && the_object->ready)
-		result = plugin->nr_frames(the_object);
+		result = plugin->nr_blocks(the_object);
 	Unlock();
 
 	return result;
@@ -522,13 +510,13 @@ int CorePlayer::GetChannels()
 	return 2;	// Hardcoded for now
 }
 
-int CorePlayer::GetFrameSize()
+int CorePlayer::GetBlockSize()
 {
 	int result = 0;
 
 	Lock();
 	if (plugin && the_object && the_object->ready)
-		result = plugin->frame_size(the_object);
+		result = plugin->block_size(the_object);
 	Unlock();
 
 	return result;
@@ -545,7 +533,7 @@ int CorePlayer::GetSampleRate()
 	Unlock();
 
 	return result;
-}	
+}
 
 
 int CorePlayer::GetStreamInfo(stream_info *info)
@@ -553,20 +541,28 @@ int CorePlayer::GetStreamInfo(stream_info *info)
 	int result = 0;
 
 	memset(info, 0, sizeof(stream_info));
-	
+
 	Lock();
 	if (plugin && plugin->stream_info && info && the_object) {
 		result = plugin->stream_info(the_object, info);
 	}
+	if (plugin && info && the_object) {
+		if (plugin->stream_info) { // plugin provides its own metadata parser
+			result = plugin->stream_info(the_object, info);
+		}
+		else{ // we'll use generic metadata parser
+//TODO			result = get_stream_info (the_object->path, info);
+		}
+	}
 	Unlock();
 
-	return result;		
+	return result;
 }
 
 
 // This is the first function to get documented. Believe it or not
 // it was the most difficult to get right! We absolutely NEED to have
-// a known state of all the threads of our object! 
+// a known state of all the threads of our object!
 
 bool CorePlayer::Start()
 {
@@ -579,7 +575,7 @@ bool CorePlayer::Start()
 	if (!node) {
 		alsaplayer_error("CorePlayer does not have a node");
 		return false;
-	}	
+	}
 	Lock();
 	// First check if we have a filename to play
 	if (!the_object->ready) {
@@ -591,7 +587,7 @@ bool CorePlayer::Start()
 	jumped = false;
 
 	ResetBuffer();
-	
+
 	last_read = -1;
 	output_rate = node->SamplingRate();
 	input_rate = plugin->sample_rate(the_object);
@@ -606,8 +602,8 @@ bool CorePlayer::Start()
 	result = GetSpeed();
 
 	if (result < 0.0) {
-		int start_frame = plugin->nr_frames(the_object) - 16;
-		write_buf->start = start_frame > 0 ? start_frame : 0;
+		int start_block = plugin->nr_blocks(the_object) - 16;
+		write_buf->start = start_block > 0 ? start_block : 0;
 		SetDirection(DIR_BACK);
 	} else {
 		write_buf->start = 0;
@@ -621,7 +617,7 @@ bool CorePlayer::Start()
 	//alsaplayer_error("Prebuffering...");
 	// wait up to 4 seconds to get all (really: half) the PCM buffers filled
 	tries = 100;
-	while (--tries && (FilledBuffers() < NR_CBUF / 2) && producing) { 
+	while (--tries && (FilledBuffers() < NR_CBUF / 2) && producing) {
 		//alsaplayer_error("Waiting for buffers...");
 		dosleep(40000);
 	}
@@ -635,7 +631,7 @@ bool CorePlayer::Start()
 	//alsaplayer_error("Starting streamer thread...");
 	sub->EnterStream(streamer_func, this);
 	Unlock();
-	// Notify 
+	// Notify
 	std::set<coreplayer_notifier *>::const_iterator i;
 	LockNotifiers();
 	for (i = notifiers.begin(); i != notifiers.end(); i++) {
@@ -645,7 +641,7 @@ bool CorePlayer::Start()
 	}
 	UnlockNotifiers();
 	return true;
-}  
+}
 
 
 void CorePlayer::Stop()
@@ -660,25 +656,25 @@ void CorePlayer::Stop()
 	streaming = false;
 	//alsaplayer_error("Signalling 6...");
 	pthread_cond_signal(&producer_ready);
-#ifdef DEBUG	
+#ifdef DEBUG
 	alsaplayer_error("Waiting for producer_thread to finish...");
 #endif
 	if (producer_thread) {
 	    if (pthread_join(producer_thread, NULL)) {
 		    alsaplayer_error("producer_thread not running...");
-	    } 
+	    }
 	}
 	pthread_mutex_destroy(&counter_mutex);
-	
+
 	pthread_mutex_init(&counter_mutex, NULL);
 	producer_thread = 0;
-	
+
 	//alsaplayer_error("producer_func is gone now...");
-	
+
 	ResetBuffer();
 	Unlock();
 
-	// Notify 
+	// Notify
 	std::set<coreplayer_notifier *>::const_iterator i;
 	LockNotifiers();
 	for (i = notifiers.begin(); i != notifiers.end(); i++) {
@@ -695,15 +691,15 @@ void CorePlayer::SetVolume(float val)
 	if (val > 2.0) {
 		alsaplayer_error("ERROR: volume out of bound (%.2f)", val);
 		return;
-	}	
+	}
 	volume = val;
-	
+
 	std::set<coreplayer_notifier *>::const_iterator i;
 	LockNotifiers();
 	for (i = notifiers.begin(); i != notifiers.end(); i++) {
 		if ((*i)->volume_changed)
 			(*i)->volume_changed((*i)->data, volume);
-	}		
+	}
 	UnlockNotifiers();
 }
 
@@ -715,7 +711,7 @@ void CorePlayer::SetPan(float val)
 	LockNotifiers();
 	for (i = notifiers.begin(); i != notifiers.end(); i++) {
 		if ((*i)->pan_changed)
-			(*i)->pan_changed((*i)->data, pan);	
+			(*i)->pan_changed((*i)->data, pan);
 	}
 	UnlockNotifiers();
 }
@@ -737,7 +733,7 @@ int CorePlayer::SetSpeed(float val)
 {
 	if (val < 0.0 && !CanSeek())
 		val = 0.0; // Do not allow reverse play if we can't seek
-	//alsaplayer_error("new speed = %.2f", val);	
+	//alsaplayer_error("new speed = %.2f", val);
 	pitch_point = val;
 	repitched = 1;
 
@@ -747,7 +743,7 @@ int CorePlayer::SetSpeed(float val)
 	for (i = notifiers.begin(); i != notifiers.end(); i++) {
 		if ((*i)->speed_changed)
 			(*i)->speed_changed((*i)->data, pitch_point);
-	}	
+	}
 	UnlockNotifiers();
 
 	return 0;
@@ -769,7 +765,7 @@ bool CorePlayer::CanSeek()
 		if (the_object->flags & P_SEEK)
 			return true;
 	}
-	return false;		
+	return false;
 }
 
 int CorePlayer::Seek(int index)
@@ -780,11 +776,11 @@ int CorePlayer::Seek(int index)
 }
 
 
-int CorePlayer::FrameSeek(int index)
+int CorePlayer::BlockSeek(int index)
 {
 	if (plugin && the_object && the_object->ready)
-		index = plugin->frame_seek(the_object, index);
-	else 
+		index = plugin->block_seek(the_object, index);
+	else
 		index = -1;
 	return index;
 }
@@ -794,6 +790,7 @@ void CorePlayer::Close()
 {
 	Lock();
 	if (plugin && the_object && the_object->ready) {
+		free (the_object->path);
 		the_object->ready = 0;
 		plugin->close(the_object);
 	}
@@ -833,10 +830,10 @@ CorePlayer::GetPlayer(const char *path)
 bool CorePlayer::Open(const char *path)
 {
 	input_plugin *best_plugin;
-	int frame_size = 0;
-	
+	int block_size = 0;
+
 	Close();
-	
+
 	if (path == NULL || strlen(path) == 0) {
 		alsaplayer_error("No path supplied");
 		return false;
@@ -845,17 +842,17 @@ bool CorePlayer::Open(const char *path)
 		alsaplayer_error("No suitable plugin found for \"%s\"", path);
 		return false;
 	}
-	Lock();	
-	
+	Lock();
+
 	the_object->local_data = NULL;
 
 	if (best_plugin->open(the_object, path)) {
-		if ((frame_size = best_plugin->frame_size(the_object)) > BUF_SIZE) {
+		if ((block_size = best_plugin->block_size(the_object)) > BUF_SIZE) {
 			alsaplayer_error("CRITICAL ERROR: this plugin advertised a buffer size\n"
 					 "larger than %d bytes. This is not supported by AlsaPlayer!\n"
 					 "Contact the author to fix this problem.\n"
-					 "TECHNICAL: A possible solution is to divide the frame size\n"
-					 "in 2 and double the number of effective frames. This can be\n"
+					 "TECHNICAL: A possible solution is to divide the block size\n"
+					 "in 2 and double the number of effective blocks. This can be\n"
 					 "handled relatively easily in the plugin (hopefully).\n\n"
 					 "We will retreat, as chaos and despair await us on\n"
 					 "this chosen path........................................", BUF_SIZE);
@@ -863,8 +860,8 @@ bool CorePlayer::Open(const char *path)
 			Unlock();
 			return false;
 		} else {
-			if (!frame_size) {
-					alsaplayer_error("No framesize");
+			if (!block_size) {
+					alsaplayer_error("No blocksize");
 					best_plugin->close(the_object);
 					Unlock();
 					return false;
@@ -877,14 +874,17 @@ bool CorePlayer::Open(const char *path)
 		return false;
 	}
 
+	// Copy path into input_object structure for future use
+	the_object->path = strdup (path); // including trailing \0
+
 	the_object->ready = 1;
 	plugin = best_plugin;
-	
-	frames_in_buffer = read_buf->buf->GetBufferSizeBytes(plugin->frame_size(the_object)) / plugin->frame_size(the_object);
+
+	blocks_in_buffer = read_buf->buf->GetBufferSizeBytes(plugin->block_size(the_object)) / plugin->block_size(the_object);
 
 	Unlock();
-	
-	return true;	
+
+	return true;
 }
 
 #ifdef DEBUG
@@ -907,27 +907,27 @@ int CorePlayer::SetDirection(int direction)
 		switch(direction) {
 		 case DIR_FORWARD:
 			while (buffers < NR_BUF-1 && tmp_buf->next->start ==
-			       (tmp_buf->start + frames_in_buffer)) {
+			       (tmp_buf->start + blocks_in_buffer)) {
 				buffers++;
 				tmp_buf = tmp_buf->next;
 			}
 			break;
 	 	  case DIR_BACK:
 			while (buffers < NR_BUF-1 && tmp_buf->prev->start ==
-			       (tmp_buf->start - frames_in_buffer)) {
+			       (tmp_buf->start - blocks_in_buffer)) {
 				buffers++;
 				tmp_buf = tmp_buf->prev;
 			}
 		}
 		new_write_buf = tmp_buf;
-		new_frame_number = new_write_buf->start;
+		new_block_number = new_write_buf->start;
 		write_buf_changed = 1;
 		read_direction = direction;
 		//alsaplayer_error("Signalling...1");
-		pthread_cond_signal(&producer_ready);		
+		pthread_cond_signal(&producer_ready);
 	}
-	return 0;	
-	
+	return 0;
+
 }
 
 
@@ -939,7 +939,7 @@ int CorePlayer::FilledBuffers()
 	sample_buf *tmp = read_buf;
 
 	if (read_buf == write_buf) {
-		return 0;	
+		return 0;
 	}
 	switch (read_direction) {
 	 case DIR_FORWARD:
@@ -970,9 +970,9 @@ void CorePlayer::update_pitch()
 		pitch = pitch_point;
 	}
 	pitch *= pitch_multi;
-	
+
 	repitched = 0;
-}	
+}
 
 
 /*
@@ -986,7 +986,7 @@ int CorePlayer::Read32(void *data, int samples)
 	if (repitched) {
 		update_pitch();
 	}
-	
+
 	if (pitch == 0.0 || (read_buf == write_buf)) {
 		if (write_buf->next->start == -2 || !producing) {
 			return -2;
@@ -995,7 +995,7 @@ int CorePlayer::Read32(void *data, int samples)
 		memset(data, 0, samples * 4);
 		return samples;
 	}
-	int use_read_direction = read_direction;	
+	int use_read_direction = read_direction;
 	int *out_buf = (int *)data;
 	int *in_buf = (int *)read_buf->buf->buffer_data;
 	in_buf += read_buf->buf->read_index;
@@ -1004,7 +1004,7 @@ int CorePlayer::Read32(void *data, int samples)
 	int check_index = read_buf->buf->read_index;
 	int base_corr = 0;
 	float use_pitch = pitch;
-	
+
 	if (jumped) {
 		int i;
 		jumped = false;
@@ -1017,7 +1017,7 @@ int CorePlayer::Read32(void *data, int samples)
 		}
 		// ---- Testing area ----
 		new_write_buf->start = jump_point;
-		new_frame_number = jump_point;
+		new_block_number = jump_point;
 		write_buf_changed = 1;
 		//alsaplayer_error("Signalling 2...");
 		pthread_cond_signal(&producer_ready);
@@ -1040,7 +1040,7 @@ int CorePlayer::Read32(void *data, int samples)
 			if (samplesInBuffer <= 0) {
 				memset(data, 0, samples * 4);
 				return samples;
-			}	
+			}
 			// use rest of current block
 			if (samplesInBuffer)
 				memcpy(out_buf, in_buf, samplesInBuffer * 4);
@@ -1057,8 +1057,8 @@ int CorePlayer::Read32(void *data, int samples)
 			tmp_index = -1;
 		}
 	}
-	else 
-#endif		
+	else
+#endif
 	if (use_read_direction == DIR_FORWARD) {
 		while (buf_index < samples) {
 			tmp_index = (int) ((float)use_pitch*(float)(buf_index-base_corr));
@@ -1067,7 +1067,7 @@ int CorePlayer::Read32(void *data, int samples)
 					if (read_buf->next->start == -2) {
 						//alsaplayer_error("Next in queue (2)");
 						return -2;
-					}	
+					}
 					memset(data, 0, samples * 4);
 					if (!producing) {
 						//alsaplayer_error("blah 1");
@@ -1075,7 +1075,7 @@ int CorePlayer::Read32(void *data, int samples)
 					}
 					return samples;
 				} else if (read_buf->next->start !=
-					read_buf->start + frames_in_buffer) {
+					read_buf->start + blocks_in_buffer) {
 					alsaplayer_error("------ WTF!!? %d - %d",
 					read_buf->next->start,
 					read_buf->start);
@@ -1096,7 +1096,7 @@ int CorePlayer::Read32(void *data, int samples)
 	} else { // DIR_BACK
 		while (buf_index < samples) {
 			tmp_index = (int)((float)use_pitch*(float)(buf_index-base_corr));
-			if ((check_index - tmp_index) < 0) { 
+			if ((check_index - tmp_index) < 0) {
 				if (read_buf->prev->start < 0 ||
 					read_buf->prev == write_buf) {
 				//	printf("Back (%d %d) ", FilledBuffers(), read_buf->prev->start);
@@ -1136,39 +1136,39 @@ int CorePlayer::Read32(void *data, int samples)
 int CorePlayer::pcm_worker(sample_buf *dest, int start)
 {
 	int result = 0;
-	int frames_read = 0;
-	int bytes_written = 0;
-	char *sample_buffer;
+	int blocks_read = 0;
+	int samples_written = 0;
+	short *sample_buffer;
 
 	if (!dest || !the_object || !the_object->ready) {
 		return -1;
 	}
 
-	int count = dest->buf->GetBufferSizeBytes(plugin->frame_size(the_object));
+	int count = dest->buf->GetBufferSizeBytes(plugin->block_size(the_object));
 	dest->buf->Clear();
 	if (last_read != start) {
-		FrameSeek(start);
+		BlockSeek(start);
 	}
 	if (start < 0) {
 		return -1;
 	}
-	sample_buffer = (char *)dest->buf->buffer_data;
+	sample_buffer = dest->buf->buffer_data;
 	while (count > 0 && producing) {
-		if (!plugin->play_frame(the_object, sample_buffer + bytes_written)) {
+		if (!plugin->play_block(the_object, sample_buffer + samples_written)) {
 			dest->start = start;
 #ifdef DEBUG
-			alsaplayer_error("frames read = %d", frames_read);
+			alsaplayer_error("blocks read = %d", blocks_read);
 #endif
                         count = 0;
 		} else {
-			bytes_written += plugin->frame_size(the_object); // OPTIMIZE!
-			frames_read++;
+			samples_written += plugin->block_size(the_object); // OPTIMIZE!
+			blocks_read++;
 		}
-		count -= plugin->frame_size(the_object);
+		count -= plugin->block_size(the_object);
 	}
 	dest->start = start;
-	last_read = dest->start + frames_read;
-	dest->buf->SetSamples(bytes_written >> 2);
+	last_read = dest->start + blocks_read;
+	dest->buf->SetSamples(samples_written >> 1);
 
 	// Check if rates are still the same
 	if ((result = plugin->sample_rate(the_object)) != input_rate) {
@@ -1176,8 +1176,8 @@ int CorePlayer::pcm_worker(sample_buf *dest, int start)
 		input_rate = result;
 		SetSpeedMulti((float) input_rate / (float) output_rate);
 		update_pitch();
-	}	
-	return frames_read;
+	}
+	return blocks_read;
 }
 
 
@@ -1186,33 +1186,33 @@ int CorePlayer::pcm_worker(sample_buf *dest, int start)
 void CorePlayer::producer_func(void *data)
 {
 	CorePlayer *obj = (CorePlayer *)data;
-	int frames_read;
+	int blocks_read;
 
 #ifdef DEBUG
 	alsaplayer_error("Starting new producer_func...");
-#endif	
+#endif
 	while (obj->producing) {
 		if (obj->write_buf_changed) {
 			obj->write_buf_changed = 0;
 			obj->write_buf = obj->new_write_buf;
-			obj->write_buf->start = obj->new_frame_number;
+			obj->write_buf->start = obj->new_block_number;
 		}
 		// still at least one buffer left to fill?
 		if (obj->FilledBuffers() < (NR_CBUF-1)) {
 			//alsaplayer_error("producer: filling buffer");
 			switch (obj->read_direction) {
 			 case DIR_FORWARD:
-				frames_read = obj->pcm_worker(obj->write_buf, obj->write_buf->start);
-				if (frames_read != obj->frames_in_buffer) {
+				blocks_read = obj->pcm_worker(obj->write_buf, obj->write_buf->start);
+				if (blocks_read != obj->blocks_in_buffer) {
 					obj->producing = false;
 					obj->write_buf->next->start = -2;
 				}
 				obj->write_buf = obj->write_buf->next;
-				obj->write_buf->start = obj->write_buf->prev->start + obj->frames_in_buffer;
+				obj->write_buf->start = obj->write_buf->prev->start + obj->blocks_in_buffer;
 				break;
 			 case DIR_BACK:
-				frames_read = obj->pcm_worker(obj->write_buf, obj->write_buf->start);
-				if (frames_read != obj->frames_in_buffer) {
+				blocks_read = obj->pcm_worker(obj->write_buf, obj->write_buf->start);
+				if (blocks_read != obj->blocks_in_buffer) {
 					if (obj->write_buf->start >= 0)
 						alsaplayer_error("an ERROR occured or EOF (%d)",
 							obj->write_buf->start);
@@ -1220,8 +1220,8 @@ void CorePlayer::producer_func(void *data)
 					obj->producing = false;
 				}
 				obj->write_buf = obj->write_buf->prev;
-				obj->write_buf->start = obj->write_buf->next->start - 
-					obj->frames_in_buffer;
+				obj->write_buf->start = obj->write_buf->next->start -
+					obj->blocks_in_buffer;
 				break;
 			}
 		} else {
@@ -1232,7 +1232,7 @@ void CorePlayer::producer_func(void *data)
 			//alsaplayer_error("Got signalled...");
 			pthread_mutex_unlock(&obj->counter_mutex);
 			//alsaplayer_error("producer: unblocked");
-		}	
+		}
 	}
 	//alsaplayer_error("Exitting producer_func (producing = %d)", obj->producing);
 	pthread_exit(NULL);
@@ -1289,7 +1289,7 @@ bool CorePlayer::streamer_func(void *arg, void *data, int size)
 			//printf("v = %d, p = %d, left = %d, right = %d\n",
 			//	p, v, left, right);
 			volume_effect32(obj->input_buffer, count, left, right);
-		}	
+		}
 		// Now add the data to the current stream
 		int16_t *in_buffer;
 		int16_t *out_buffer;
@@ -1301,13 +1301,13 @@ bool CorePlayer::streamer_func(void *arg, void *data, int size)
 				if (level > 32767) level = 32767;
 				else if (level < -32768) level = -32768;
 				*(out_buffer++) = (int16_t) level;
-		}		
+		}
 		return true;
 	} else {
 		//alsaplayer_error("Exiting from streamer_func...");
 		obj->streaming = false;
 		return false;
-	}	
+	}
 }
 #endif
 
@@ -1320,7 +1320,7 @@ bool CorePlayer::IsPaused()
 
 void CorePlayer::Pause()
 {
-	save_speed = (GetSpeed () * ((float) output_rate / (float) input_rate));
+	save_speed = GetSpeed ();
 	SetSpeed (0.0);
 }
 
