@@ -1,5 +1,5 @@
 /*  Main.cpp - main() function and other utils
- *  Copyright (C) 1998-2004 Andy Lo A Foe <andy@alsaplayer.org>
+ *  Copyright (C) 1998-2004 Andy Lo A Foe <andy@loafoe.com>
  *
  *  This file is part of AlsaPlayer.
  *
@@ -16,8 +16,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
- *  $Id: Main.cpp 1298 2007-10-08 19:14:29Z dominique_libre $
- *
  *  VIM: set ts=8
  *
 */
@@ -37,6 +35,7 @@
 
 #include <cstdio>
 #include <cstdlib>
+#include <algorithm>
 #include <csignal>
 #include <cassert>
 #include <unistd.h>
@@ -64,8 +63,10 @@
 #include "utilities.h"
 #include "prefs.h"
 #include "alsaplayer_error.h"
-#include "message.c"		/* This is a dirty hack */
+#include "message.h"
+#include "control.h"
 #include "reader.h"
+#include "ap_string.h"
 
 #define MAX_REMOTE_SESSIONS	32
 
@@ -81,14 +82,14 @@ int global_quiet = 0;
 
 char *global_session_name = NULL;
 char *global_interface_script = NULL;
-char *global_pluginroot = NULL;
+const char *global_pluginroot = NULL;
 
 prefs_handle_t *ap_prefs = NULL;
 
 void control_socket_start(Playlist *, interface_plugin *ui);
 void control_socket_stop();
 
-static char *default_pcm_device = "default";
+static const char *default_pcm_device = "default";
 
 
 extern "C" {
@@ -127,7 +128,7 @@ void exit_sighandler(int x)
 	if (sigcount == 1) {
 		alsaplayer_error("alsaplayer interrupted by signal %d", x);
 		exit(1);
-	}	
+	}
 	if (sigcount > 5) {
 		kill(getpid(), SIGKILL);
 	}
@@ -137,7 +138,8 @@ void exit_sighandler(int x)
 interface_plugin_info_type load_interface(const char *name)
 {
 	void *handle;
-	char path[1024], *pluginroot;
+	char path[1024];
+	const char *pluginroot;
 	struct stat statbuf;
 
 	interface_plugin_info_type plugin_info;
@@ -152,7 +154,7 @@ interface_plugin_info_type load_interface(const char *name)
 		return NULL;
 
 	if (strchr(name, '.'))
-		strncpy(path, name, sizeof(path));
+		ap_strlcpy(path, name, sizeof(path));
 	else
 		snprintf(path, sizeof(path), "%s/interface/lib%s_interface.so", pluginroot, name);
 #ifdef DEBUG
@@ -185,7 +187,7 @@ interface_plugin_info_type load_interface(const char *name)
 		alsaplayer_error("Error loading %s", path);
 		alsaplayer_error("Please remove this file from your system");
 		return NULL;
-	}	
+	}
 
 	return plugin_info;
 }
@@ -195,13 +197,14 @@ interface_plugin_info_type load_interface(const char *name)
 extern int init_reverb();
 extern bool reverb_func(void *arg, void *data, int size);
 
-static char *copyright_string =
+static const char *copyright_string =
     "AlsaPlayer " VERSION
-    "\n(C) 1999-2004 Andy Lo A Foe <andy@alsaplayer.org> and others.";
+    "\n(C) 1999-2004 Andy Lo A Foe <andy@loafoe.com> and others.";
 
 static void list_available_plugins(const char *plugindir)
 {
-	char path[1024], *pluginroot;
+	char path[1024];
+	const char *pluginroot;
 	struct stat buf;
 	bool first = true;
 
@@ -229,7 +232,7 @@ static void list_available_plugins(const char *plugindir)
 		snprintf(path, sizeof(path), "%s/%s/%s", pluginroot, plugindir, entry->d_name);
 		if (stat(path, &buf)) {
 			continue;
-		}	
+		}
 		if (S_ISREG(buf.st_mode)) {
 			char *ext = strrchr(path, '.');
 			if (!ext)
@@ -244,7 +247,7 @@ static void list_available_plugins(const char *plugindir)
 				*ext = '\0';
 			if (strncmp(path, "lib", 3)) {
 				continue;
-			}	
+			}
 			char *name = path + 3;
 			if (strcmp(plugindir, "output") == 0) { // Remove trailing _out
 				ext = strrchr(name, '_');
@@ -254,8 +257,8 @@ static void list_available_plugins(const char *plugindir)
 				ext = strrchr(name, '_');
 				if (ext)
 					*ext = '\0';
-			}	
-			
+			}
+
 			if (first) { // don't print comma
 				first = false;
 			} else {
@@ -310,6 +313,10 @@ static void help()
 		"  --speed speed         floating point speed parameter\n"
 		"    1.0 = normal speed, -1.0 normal speed backwards\n"
 		"  --jump track          jump to specified playlist track\n"
+		"  --looplist on|off     switch playlist looping on or off [default off]\n"
+		"  --loopsong on|off     switch song looping on or off [default off]\n"
+        "  -t,--onebyone on|off  switch stop after each track on or off [default off]\n"
+		"  -S,--shuffle          shuffle playlist\n"
 		"  --clear               clear whole playlist\n"
 		"  --quit                quit session\n"
 		"\n"
@@ -331,8 +338,6 @@ static void help()
 		"\n"
 		"Experimental options:\n"
 		"\n"
-		"  -S,--loopsong         loop file\n"
-		"  -P,--looplist         loop playlist\n"
 		"  -x,--crossfade        crossfade playlist entries\n"
 		"\n");
 }
@@ -346,9 +351,9 @@ static void version()
 static int get_interface_from_argv0 (char *argv0, char *str)
 {
 	char *bs = strrchr (argv0, '/');
-	
+
 	if (bs)  argv0 = ++bs;
-	
+
 	if (sscanf(argv0, "alsaplayer-%s", str) == 1)  return 1;
   	if (sscanf(argv0, "jackplayer-%s", str) == 1)  return 1;
 
@@ -357,7 +362,7 @@ static int get_interface_from_argv0 (char *argv0, char *str)
 
 int main(int argc, char **argv)
 {
-	char *device_param = default_pcm_device;
+	const char *device_param = default_pcm_device;
 	char *prefsdir;
 	char thefile[1024];
 	char str[1024];
@@ -365,13 +370,13 @@ int main(int argc, char **argv)
 	int ap_result = 0;
 	int use_fragsize = -1; // Initialized
 	int use_fragcount = -1; // later
-	int do_reverb = 0;
 	int do_loopsong = 0;
 	int do_looplist = 0;
 	int do_enqueue = 0;
 	int do_replace = 0;
 	int do_realtime = 0;
 	int do_remote_control = 0;
+	int do_shuffle = 0;
 	int do_start = 0;
 	int do_stop = 0;
 	int do_prev = 0;
@@ -386,21 +391,26 @@ int main(int argc, char **argv)
 	int do_status = 0;
 	int do_speed = 0;
 	float speed_val = 0.0;
-		
+	int do_onebyone = 0;
+
 	int use_freq = OUTPUT_RATE;
 	float use_vol = 1.0;
 	int use_session = 0;
 	int do_crossfade = 0;
 	int do_save = 1;
 	int bool_val = 0;
-	char *use_output = NULL;
+	const char *use_output = NULL;
 	char *use_interface = NULL;
 	char *use_config = NULL;
-	
+	char *use_loopsong = NULL;
+	char *use_onebyone = NULL;
+	char *use_looplist = NULL;
+
 	int opt;
 	int option_index;
-	const char *options = "bCc:d:eEf:F:g:hi:JI:l:n:NMp:qrs:vRSQPVxo:";
+	const char *options = "Cc:d:eEf:F:g:hi:J:I:l:n:NMp:qrs:vRSQVxo:";
 	struct option long_options[] = {
+	/*	{ "long_option", take_argument, 0, 'short_option' }, */
 		{ "config", 1, 0, 'c' },
 		{ "device", 1, 0, 'd' },
 		{ "enqueue", 0, 0, 'e' },
@@ -421,13 +431,14 @@ int main(int argc, char **argv)
 		{ "version", 0, 0, 'v' },
 		{ "verbose", 0, 0, 'V' },
 		{ "reverb", 0, 0, 'R' },
-		{ "loopsong", 0, 0, 'L' },
-		{ "looplist", 0, 0, 'P' },
+		{ "loopsong", 1, 0, 'L' },
+		{ "looplist", 1, 0, 'P' },
 		{ "crossfade", 0, 0, 'x' },
 		{ "output", 1, 0, 'o' },
 		{ "stop", 0, 0, 'U' },
 		{ "pause", 0, 0, 'O' },
 		{ "start", 0, 0, 'T' },
+		{ "shuffle", 0, 0, 'S' },
 		{ "prev", 0, 0, 'Q' },
 		{ "next", 0, 0, 'M' },
 		{ "jump", 1, 0, 'J' },
@@ -438,9 +449,34 @@ int main(int argc, char **argv)
 		{ "startvolume", 1, 0, 'l' },
 		{ "quit", 0, 0, 'A' },
 		{ "status", 0, 0, 'B' },
+        { "onebyone", 1, 0, 't' },
+
+		// Options that we want to be able to pass on to gtk_init(). See man
+		// gtk-options(7).
+		// Give all of these an option number of 128 because we're going to
+		// ignore them option switch statement anyway.
+		{ "gtk-module", 1, 0, 128 },
+		{ "gtk-debug", 1, 0, 128 },
+		{ "gtk-no-debug", 1, 0, 128 },
+		{ "g-fatal-warnings", 0, 0, 128 },
+
+		{ "display", 1, 0, 128 },
+		{ "screen", 1, 0, 128 },
+		{ "sync", 0, 0, 128 },
+		{ "no-xshm", 0, 0, 128 },
+		{ "name", 1, 0, 128 },
+		{ "class", 1, 0, 128 },
+		{ "gxid_host", 1, 0, 128 },
+		{ "gxid_port", 1, 0, 128 },
+		{ "xim-preedit", 0, 0, 128 },
+		{ "xim-status", 0, 0, 128 },
+		{ "gdk-debug", 1, 0, 128 },
+		{ "gdk-no-debug", 1, 0, 128 },
+
+		// End of list marker.
 		{ 0, 0, 0, 0 }
-	};	
-		
+	};
+
 
 	// First setup signal handler
 	signal(SIGPIPE, nonfatal_sighandler);   // PIPE (socket control)
@@ -468,6 +504,10 @@ int main(int argc, char **argv)
 #if !defined(EMBEDDED)
 	init_effects();
 #endif
+
+	// Init random number generator
+	std::srand(time(0));
+
 	while ((opt = getopt_long(argc, argv, options, long_options, &option_index)) != EOF) {
 		switch(opt) {
 			case 'A':
@@ -501,21 +541,21 @@ int main(int argc, char **argv)
 					return 1;
 				}
 				if (use_fragsize > 32768) {
-					alsaplayer_error("fragment sizes larger than 32768 bytes are not supported");
+					alsaplayer_error("fragment size (%d) out of range (0-32768)", use_fragsize);
 					return 1;
-				}	
+				}
 				break;
 			case 'F':
 				use_freq = atoi(optarg);
 				if (use_freq < 8000 || use_freq > 48000) {
-					alsaplayer_error("frequency out of range (8000-48000)");
+					alsaplayer_error("frequency (%d) out of range (8000-48000)", use_freq);
 					return 1;
 				}
 				break;
 			case 'g':
 				use_fragcount = atoi(optarg);
 				if (use_fragcount < 2 || use_fragcount > 128) {
-					alsaplayer_error("fragcount out of range (2-128)");
+					alsaplayer_error("fragcount (%d) out of range (2-128)", use_fragcount);
 					return 1;
 				}
 				break;
@@ -534,16 +574,21 @@ int main(int argc, char **argv)
 			case 'l':
 				start_vol = atof(optarg);
 				if (start_vol < 0.0 || start_vol > 1.0) {
-					alsaplayer_error("volume out of range: using 1.0");
+					alsaplayer_error("volume (%.3f) out of range: using 1.0", start_vol);
 					start_vol = 1.0;
 				}
 				break;
+			case 'L':
+				do_remote_control = 1;
+				do_loopsong = 1;
+				use_loopsong = optarg;
+				break;
 			case 'Y':
 				do_remote_control = 1;
 				do_setvol = 1;
 				use_vol = atof(optarg);
 				if (use_vol < 0.0 || use_vol > 1.0) {
-					alsaplayer_error("volume out of range: using 1.0");
+					alsaplayer_error("volume (%.3f) out of range: using 1.0", use_vol);
 					use_vol = 1.0;
 				}
 				break;
@@ -558,7 +603,7 @@ int main(int argc, char **argv)
 				do_pause = 1;
 				break;
 			case 'p':
-				global_pluginroot = strdup(optarg);
+				global_pluginroot = optarg;
 				break;
 			case 'q':
 				global_quiet = 1;
@@ -580,13 +625,11 @@ int main(int argc, char **argv)
 				global_verbose = 1;
 				break;
 			case 'R':
-				do_reverb = 1;
-				break;
-			case 'S':
-				do_loopsong = 1;
 				break;
 			case 'P':
+				do_remote_control = 1;
 				do_looplist = 1;
+				use_looplist = optarg;
 				break;
 			case 'x':
 				do_crossfade = 1;
@@ -603,10 +646,14 @@ int main(int argc, char **argv)
 				do_remote_control = 1;
 				do_stop = 1;
 				break;
-			case 'T': 
+			case 'T':
 				do_remote_control = 1;
 				do_start = 1;
 				break;
+			case 'S':
+				do_remote_control = 1;
+				do_shuffle = 1;
+				break;
 			case 'Q':
 				do_remote_control = 1;
 				do_prev = 1;
@@ -632,24 +679,32 @@ int main(int argc, char **argv)
 				do_relative = 1;
 				do_seek = atoi(optarg);
 				break;
+            case 't':
+				do_remote_control = 1;
+				do_onebyone = 1;
+				use_onebyone = optarg;
+				break;
+			case 128:
+				// Gtk-option which we ignore.
+				break;
 			default:
 				alsaplayer_error("Unknown option '%c'", opt);
 				break;
-		}	
+		}
 	}
-	
+
 	prefsdir = get_prefsdir();
-	
+
 	mkdir(prefsdir, 0700);	/* XXX We don't do any error checking here */
 	snprintf(thefile, sizeof(thefile)-21, "%s/config", prefsdir);
-	if (use_config) 
+	if (use_config)
 		ap_prefs = prefs_load(use_config);
 	else
 		ap_prefs = prefs_load(thefile);
 	if (!ap_prefs) {
 		alsaplayer_error("Invalid config file %s\n", use_config ? use_config : thefile);
 		return 1;
-	}	
+	}
 	/* Initialize some settings (and populate the prefs system if needed */
 
 	if (use_fragsize < 0)
@@ -663,7 +718,7 @@ int main(int argc, char **argv)
 
 	if (!global_pluginroot) {
 		global_pluginroot = strdup (ADDON_DIR);
-	}	
+	}
 
 
 	if (use_session == 0) {
@@ -678,7 +733,7 @@ int main(int argc, char **argv)
 			if (do_remote_control) {
 				alsaplayer_error("No active sessions");
 				return 1;
-			}	
+			}
 			do_enqueue = 0;
 		} else {
 			//alsaplayer_error("Found session %d", use_session);
@@ -687,8 +742,8 @@ int main(int argc, char **argv)
 				//alsaplayer_error("Using session %d, not doing multiopen", use_session);
 				do_enqueue = 1;
 				do_replace = 1;
-			}	
-		}	
+			}
+		}
 	}
 
 	// Check if we're in remote control mode
@@ -720,17 +775,20 @@ int main(int argc, char **argv)
 				fprintf(stdout, "genre: %s\n", res);
 			if (ap_get_file_path(use_session, res) && strlen(res))
 				fprintf(stdout, "path: %s\n", res);
-			if (ap_get_frames(use_session, &ires)) 
-				fprintf(stdout, "frames: %d\n", ires);
+			if (ap_get_blocks(use_session, &ires))
+				fprintf(stdout, "blocks: %d\n", ires);
 			if (ap_get_length(use_session, &ires))
 				fprintf(stdout, "length: %d second%s\n", ires, (ires == 1) ? "": "s");
 			if (ap_get_position(use_session, &ires))
 				fprintf(stdout, "position: %d\n", ires);
-			fprintf(stdout, "-----------------------------------------\n");					
+			fprintf(stdout, "-----------------------------------------\n");
 			return 0;
 		} else if (do_setvol) {
 			ap_set_volume(use_session, use_vol);
 			return 0;
+		} else if (do_shuffle) {
+			ap_shuffle_playlist(use_session);
+			return 0;
 		} else if (do_start) {
 			ap_play(use_session);
 			return 0;
@@ -739,7 +797,7 @@ int main(int argc, char **argv)
 			return 0;
 		} else if (do_pause) {
 			if (ap_is_paused(use_session, &bool_val)) {
-				if (bool_val) 
+				if (bool_val)
 					ap_unpause(use_session);
 				else
 					ap_pause(use_session);
@@ -771,21 +829,39 @@ int main(int argc, char **argv)
 		} else if (do_seek >= 0) {
 			ap_set_position(use_session, do_seek);
 			return 0;
-		} else 	
+		} else if (do_loopsong) {
+			if (strcasecmp(use_loopsong, "on") != 0) {
+				do_loopsong = false;
+			}
+			ap_set_looping(use_session, do_loopsong);
+			return 0;
+		} else if (do_onebyone) {
+			if (strcasecmp(use_onebyone, "on") != 0) {
+                                do_onebyone = false;
+                        }
+                        ap_set_onebyone(use_session, do_onebyone);
+                        return 0;
+		} else if (do_looplist) {
+			if (strcasecmp(use_looplist, "on") != 0) {
+				do_looplist = false;
+			}
+			ap_set_playlist_looping(use_session, do_looplist);
+			return 0;
+		} else
 			alsaplayer_error("No remote control command executed.");
 	}
-				
-	
+
+
 	// Check if we need to enqueue the files
 	if (do_enqueue) {
 		char queue_name[2048];
 		int count = 0;
 		int was_playing = 0;
 		int playlist_length = 0;
-	
+
 		count = optind;
 		ap_result = 1;
-		
+
 		if (do_replace && count < argc) {
 			ap_is_playing(use_session, &was_playing);
 			if (was_playing) {
@@ -796,8 +872,8 @@ int main(int argc, char **argv)
 			ap_get_playlist_length(use_session, &playlist_length);
 			if (!playlist_length) { // Empty list so fire up after add
 				was_playing = 1;
-			}		
-		}	
+			}
+		}
 		while (count < argc && ap_result) {
 			if (is_playlist(argv[count])) {
 				ap_add_playlist(use_session, argv[count]);
@@ -812,14 +888,14 @@ int main(int argc, char **argv)
 					alsaplayer_error("error getting cwd");
 					return 1;
 				}
-				strcat(queue_name, "/");
-				strncat(queue_name, argv[count], sizeof(queue_name)-strlen(queue_name));
+				ap_strlcat(queue_name, "/", sizeof(queue_name));
+				ap_strlcat(queue_name, argv[count], sizeof(queue_name));
 			} else
-				strncpy(queue_name, argv[count], sizeof(queue_name)-strlen(queue_name));
+				ap_strlcpy(queue_name, argv[count], sizeof(queue_name));
 			count++;
 			//alsaplayer_error("Adding %s", queue_name);
 			ap_result = ap_add_path(use_session, queue_name);
-			//alsaplayer_error("ap_result = %d", ap_result);	
+			//alsaplayer_error("ap_result = %d", ap_result);
 		}
 		if (was_playing)
 			ap_jump_to(use_session, 1);
@@ -833,7 +909,7 @@ int main(int argc, char **argv)
 	if (strcmp(argv[0], "jackplayer") == 0) {
 		use_output = "jack";
 	}
-	
+
 	// Check the output option
 	if (use_output == NULL) {
 		use_output = prefs_get_string(ap_prefs, "main",
@@ -880,11 +956,11 @@ int main(int argc, char **argv)
 	// Initialise playlist - must be done before things try to register with it
 	playlist = new Playlist(node);
 
-	if (!prefs_get_bool(ap_prefs, "main", "play_on_start", false))	
+	if (!prefs_get_bool(ap_prefs, "main", "play_on_start", false))
 		playlist->Pause();
 	else
 		playlist->UnPause();
-	
+
 	if (!playlist) {
 		alsaplayer_error("Failed to create Playlist object");
 		return 1;
@@ -898,9 +974,9 @@ int main(int argc, char **argv)
 					alsaplayer_error("Loading playlist (%s)", argv[optind]);
 				playlist->Load(std::string(argv[optind++]),
 						playlist->Length(), false);
-			} else {	
+			} else {
 				newitems.push_back(std::string(argv[optind++]));
-			}	
+			}
 		}
 		playlist->Insert(newitems, playlist->Length());
 	} else {
@@ -908,7 +984,7 @@ int main(int argc, char **argv)
 		snprintf(thefile, sizeof(thefile)-28, "%s/alsaplayer.m3u", prefsdir);
 		playlist->Load(thefile, playlist->Length(), false);
 	}
-		
+
 	// Loop song
 	if (do_loopsong) {
 		playlist->LoopSong();
@@ -917,6 +993,10 @@ int main(int argc, char **argv)
 	if (do_looplist) {
 		playlist->LoopPlaylist();
 	}
+	// Play songs one by one
+	if (do_onebyone) {
+		playlist->SetOneByOne();
+	}
 	// Cross fading
 	if (do_crossfade) {
 		playlist->Crossfade();
@@ -924,20 +1004,20 @@ int main(int argc, char **argv)
 	// Set start volume
 	playlist->GetCorePlayer()->SetVolume(start_vol);
 
-	
+
 	interface_plugin_info_type interface_plugin_info;
 	interface_plugin *ui;
 
 	if (get_interface_from_argv0 (argv[0], str))
 		use_interface = str;
-	
+
 	if (use_interface && *use_interface) {
 		if (!(interface_plugin_info = load_interface(use_interface))) {
 			alsaplayer_error("Failed to load interface %s\n", use_interface);
 			goto _fatal_err;
 		}
 	} else {
-		char *interface = prefs_get_string
+		const char *interface = prefs_get_string
 			(ap_prefs, "main", "default_interface", "gtk2");
 		// if we're trying to use the old gtk-1 interface, use gtk-2 instead
 		if (strcmp (interface, "gtk") == 0)
@@ -969,11 +1049,11 @@ int main(int argc, char **argv)
 			ui->close();
 			// Unfortunately gtk+ is a pig when it comes to
 			// cleaning up its resources; it doesn't!
-			// so we can never safely dlclose gtk+ based 
+			// so we can never safely dlclose gtk+ based
 			// user interfaces, bah!
 			//dlclose(ui->handle);
 			control_socket_stop();
-		}	
+		}
 	}
 	// Save playlist before exit
 	prefsdir = get_prefsdir();
@@ -985,7 +1065,7 @@ int main(int argc, char **argv)
 		if (prefs_save(ap_prefs) < 0) {
 			alsaplayer_error("failed to save preferences.");
 		}
-	}	
+	}
 
 _fatal_err:
 	delete playlist;
@@ -993,8 +1073,6 @@ _fatal_err:
 	delete node;
 	if (global_session_name)
 		free(global_session_name);
-	if (global_pluginroot)
-		free(global_pluginroot);
 
 	return 0;
 }
