@@ -1,5 +1,5 @@
 /*  SampleBuffer.cpp - SampleBuffer object, used in the ringbuffer
- *  Copyright (C) 1998-2002 Andy Lo A Foe <andy@alsaplayer.org>
+ *  Copyright (C) 1998-2002 Andy Lo A Foe <andy@loafoe.com>
  *
  *  This file is part of AlsaPlayer.
  *
@@ -16,9 +16,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
- *  $Id: SampleBuffer.cpp 1251 2007-07-08 14:31:34Z dominique_libre $
- *
-*/ 
+*/
 
 #include "SampleBuffer.h"
 #include <cstdio>
@@ -40,9 +38,9 @@ SampleBuffer::SampleBuffer(int mode, int size)
 		alsaplayer_error("Unsupported SAMPLE size");
 		exit(1);
 	}
-	
+
 	// Allocate buffer data
-	if ((buffer_data = new char[size]) == NULL) {
+	if ((buffer_data = new short [size]) == NULL) {
 		alsaplayer_error("Out of memory in SampleBuffer::SampleBuffer()");
 		exit(1);
 	}
@@ -119,7 +117,7 @@ int SampleBuffer::ReadSamples(void *data, int nr)
 			//memcpy(data, src, (nr * sample_size));
 			for (int c=0; c < nr; c++) {
 				*(dest++) = *(src++);
-			}	
+			}
 			read_index += nr;
 			return nr;
 		} else {
@@ -181,14 +179,14 @@ int SampleBuffer::GetBufferSize()
 }
 
 
-int SampleBuffer::GetBufferSizeBytes(int frame_size)
+int SampleBuffer::GetBufferSizeBytes(int block_size)
 {
-	if (frame_size < 0) 
+	if (block_size < 0)
 		return buffer_size * sample_size;
 	else {
 		int byte_count = buffer_size * sample_size;
-		int frame_mul = frame_size * (byte_count / frame_size);
-		return frame_mul;
+		int block_mul = block_size * (byte_count / block_size);
+		return block_mul;
 	}
 }
 
@@ -207,7 +205,7 @@ int SampleBuffer::GetReadDirection()
 
 void SampleBuffer::ResetRead()
 {
-#if 1 
+#if 1
 	switch (GetReadDirection()) {
 		case DIR_FORWARD:
 			read_index = 0;
@@ -216,7 +214,7 @@ void SampleBuffer::ResetRead()
 			read_index = write_index;
 			break;
 	}
-#else	
+#else
 	read_index = 0;
 #endif
 	return;
@@ -229,7 +227,7 @@ int SampleBuffer::GetAvailableSamples()
 	 case DIR_FORWARD:
 		if (write_index < read_index) {
 			return 0;
-		} else {	 
+		} else {
 			return (write_index - read_index);
 		}
 		break;
