@@ -1,25 +1,21 @@
 /* fft.c: Iterative implementation of a FFT
  * Copyright (C) 1999 Richard Boulton <richard@tartarus.org>
  * Convolution stuff by Ralph Loader <suckfish@ihug.co.nz>
- * This file is part of AlsaPlayer (C) 1998 - 2002 <andy@alsaplayer.org>
+ * This file is part of AlsaPlayer (C) 1998 - 2002 <andy@loafoe.com>
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  AlsaPlayer is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation; either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  AlsaPlayer is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
- *
- *  $Id: fft.c 366 2002-03-25 16:30:47Z adnans $
- * 
  */
 
 /*
@@ -95,7 +91,7 @@ fft_state *fft_init(void) {
     fft_state *state;
     unsigned int i;
 
-    state = (fft_state *) malloc (sizeof(fft_state));
+    state = malloc (sizeof(fft_state));
     if(!state) return NULL;
 
     for(i = 0; i < FFT_BUFFER_SIZE; i++) {
@@ -103,9 +99,7 @@ fft_state *fft_init(void) {
     }
     for(i = 0; i < FFT_BUFFER_SIZE / 2; i++) {
 	double j = 2 * PI * i / FFT_BUFFER_SIZE;
-	float test;
 	costable[i] = cos(j);
-	test = cos(j);
 	sintable[i] = sin(j);
     }
 
@@ -157,7 +151,7 @@ static void fft_prepare(const sound_sample *input, double * re, double * im) {
     unsigned int i;
     double *realptr = re;
     double *imagptr = im;
-    
+
     /* Get input, in reverse bit order */
     for(i = 0; i < FFT_BUFFER_SIZE; i++) {
 	*realptr++ = input[bitReverse[i]];
@@ -170,7 +164,7 @@ static void fft_prepare(const sound_sample *input, double * re, double * im) {
  * Note: only produces half as many data points as the input had.
  * This is roughly a consequence of the Nyquist sampling theorm thingy.
  * (FIXME - make this comment better, and helpful.)
- * 
+ *
  * The two divisions by 4 are also a consequence of this: the contributions
  * returned for each frequency are split into two parts, one at i in the
  * table, and the other at FFT_BUFFER_SIZE - i, except for i = 0 and
@@ -182,11 +176,11 @@ static void fft_output(const double * re, const double * im, double *output) {
     const double *realptr   = re;
     const double *imagptr   = im;
     double *endptr    = output + FFT_BUFFER_SIZE / 2;
- 
+
 #ifdef DEBUG
     unsigned int i, j;
 #endif
- 
+
     while(outputptr <= endptr) {
 	*outputptr = (*realptr * *realptr) + (*imagptr * *imagptr);
 	outputptr++; realptr++; imagptr++;
@@ -224,7 +218,7 @@ static void fft_calculate(double * re, double * im) {
     double fact_real, fact_imag;
     double tmp_real, tmp_imag;
     unsigned int factfact;
-    
+
     /* Set up some variables to reduce calculation in the loops */
     exchanges = 1;
     factfact = FFT_BUFFER_SIZE / 2;
@@ -243,7 +237,7 @@ static void fft_calculate(double * re, double * im) {
 	     */
 	    fact_real = costable[j * factfact];
 	    fact_imag = sintable[j * factfact];
-	    
+
 	    /* Loop through all the exchange groups */
 	    for(k = j; k < FFT_BUFFER_SIZE; k += exchanges << 1) {
 		int k1 = k + exchanges;
