@@ -1,30 +1,30 @@
 /*  utilities.c
  *  Copyright (C) 1999 Richard Boulton <richard@tartarus.org>
- *  Copyright (C) 2003 Andy Lo A Foe <andy@alsaplayer.org>
+ *  Copyright (C) 2003 Andy Lo A Foe <andy@loafoe.com>
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This file is part of AlsaPlayer.
+ *
+ *  AlsaPlayer is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation; either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  AlsaPlayer is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
- *
- *  $Id: utilities.c 1030 2004-02-15 18:58:30Z adnans $
- *
-*/ 
+*/
 
 #include <ctype.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <limits.h>
+
 #include "utilities.h"
 #include "alsaplayer_error.h"
 
@@ -60,7 +60,7 @@ char *get_homedir(void)
 
 char *get_prefsdir(void)
 {
-	static char *prefs_path;
+	static char prefs_path [PATH_MAX];
 	static int prefs_path_init = 0;
 	char *homedir;
 
@@ -68,16 +68,13 @@ char *get_prefsdir(void)
 		return prefs_path;
 	}
 	homedir = get_homedir();
-	prefs_path = (char *)malloc(strlen(homedir) + 14);
-	if (!prefs_path) 
-		return NULL;
-	sprintf(prefs_path, "%s/.alsaplayer", homedir);
+	snprintf(prefs_path, sizeof (prefs_path), "%s/.alsaplayer", homedir);
 	prefs_path_init = 1;
 	return prefs_path;
 }
 
-
-void encode_percent_free(char *p)
+#if 0
+static voidencode_percent_free(char *p)
 {
 	if (p)
 		free(p);
@@ -87,7 +84,7 @@ char *encode_percent(const char *furi)
 {
 	char *res;
 	int c, i, o, t;
-	
+
 	if (!furi)
 		return NULL;
 	/* count the percent signs */
@@ -109,9 +106,10 @@ char *encode_percent(const char *furi)
 			}
 			res[o++] = furi[i++];
 		}
-	}	
+	}
 	return res;
 }
+#endif
 
 char *parse_file_uri(const char *furi)
 {
@@ -126,7 +124,7 @@ char *parse_file_uri(const char *furi)
 		return NULL;
 	}
 	t=strlen(furi);
-	res = (char *)malloc(t+1);
+	res = malloc(t+1);
 	r=5;
 	w=0;
 	e=0;
@@ -140,7 +138,7 @@ char *parse_file_uri(const char *furi)
 				} else {
 					percent = 1;
 					e = 0;
-				}	
+				}
 				break;
 			default:
 				if (percent) {
@@ -164,7 +162,7 @@ char *parse_file_uri(const char *furi)
 	}
 	alsaplayer_error("parsed to: %s", res);
 	return res;
-		
+
 }
 
 
