@@ -1,4 +1,4 @@
-/*  cdda.h (C) 1999-2003 by Andy Lo A Foe <andy@alsaplayer.org>
+/*  cdda.h (C) 1999-2003 by Andy Lo A Foe <andy@loafoe.com>
  *  This file is part of AlsaPlayer.
  *
  *  AlsaPlayer is free software; you can redistribute it and/or modify
@@ -26,7 +26,9 @@
 #include <arpa/inet.h>
 #include <sys/stat.h>		/* for fd operations		*/
 #include <fcntl.h>		/* for fd operations, too	*/
+#ifndef __USE_XOPEN
 #define __USE_XOPEN /* needed for swab */
+#endif
 #include <unistd.h>		/* for close()			*/
 #undef __USE_XOPEN
 #include <errno.h>		/* errno..     		        */
