@@ -1,7 +1,7 @@
 /*
- *  cdda_engine.c (C) 1999-2003 by Andy Lo A Foe <andy@alsaplayer.org>
+ *  cdda_engine.c (C) 1999-2003 by Andy Lo A Foe <andy@loafoe.com>
  *  CDDB lookup code by Anders Rune Jensen
- *	
+ *
  *	Based on code from dagrab 0.3 by Marcello Urbani <murbani@numerica.it>
  *
  *  This plugin reads music CD's in digital form, allowing for exceptional
@@ -27,8 +27,7 @@
  */
 
 #include "config.h"
-#include "cdda.h"
-#include "prefs.h"
+
 #include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -50,28 +49,33 @@
 #include <sys/types.h>
 #include <sys/param.h>
 #include <pthread.h>
+
+#include "cdda.h"
+#include "prefs.h"
 #include "input_plugin.h"
 #include "alsaplayer_error.h"
 #include "AlsaPlayer.h"
 #include "control.h"
 #include "prefs.h"
 #include "utilities.h"
+#include "ap_string.h"
+#include "ap_unused.h"
 
 #define MAX_TRACKS	128
 #define BUFFER_SIZE     4096
 
 #define DEFAULT_DEVICE	"/dev/cdrom"
-#define FRAME_LEN	4
+#define BLOCK_LEN	4
 #define N_BUF 8
 #define OVERLAY 3
 #define KEYLEN 12
 #define OFS 12
 #define RETRYS 20
-#define IFRAMESIZE (CD_FRAMESIZE_RAW/sizeof(int))
+#define IBLOCKSIZE (CD_FRAMESIZE_RAW/sizeof(int))
 #define BLEN 255
 
 /* global variables */
-static char *REAL_PATH = NULL;
+static char real_path [PATH_MAX];
 
 typedef struct
 {
@@ -151,7 +155,8 @@ static char *resttime(int sec)
 }
 */
 
-void toc_fail(struct cd_trk_list *tl)
+static void
+toc_fail(struct cd_trk_list *tl)
 {
 	free(tl->starts);
 	free(tl->types);
@@ -184,25 +189,25 @@ static int cd_getinfo(int *cdrom_fd, char *cd_dev, struct cd_trk_list *tl)
 	tl->min=Th.cdth_trk0; /* first track */
 	tl->max=Th.cdth_trk1; /* last track */
 
-	if((tl->starts=(int *)malloc((tl->max-tl->min+2)*sizeof(int)))==NULL){
+	if((tl->starts=malloc((tl->max-tl->min+2)*sizeof(int)))==NULL){
 		alsaplayer_error("CDDA: list data allocation failed");
 		return 1;
 	}
-	if((tl->types=(char *)malloc(tl->max-tl->min+2))==NULL){
+	if((tl->types=malloc(tl->max-tl->min+2))==NULL){
 		alsaplayer_error("CDDA: list data allocation failed");
 		return 1;
 	}
 
 	/* length */
-	if((tl->l_min=(int *)malloc((tl->max-tl->min+2)*sizeof(int)))==NULL){
+	if((tl->l_min=malloc((tl->max-tl->min+2)*sizeof(int)))==NULL){
 		alsaplayer_error("CDDA: list data allocation failed");
 		return 1;
 	}
-	if((tl->l_sec=(int *)malloc((tl->max-tl->min+2)*sizeof(int)))==NULL){
+	if((tl->l_sec=malloc((tl->max-tl->min+2)*sizeof(int)))==NULL){
 		alsaplayer_error("CDDA: list data allocation failed");
 		return 1;
 	}
-	if((tl->l_frame=(int *)malloc((tl->max-tl->min+2)*sizeof(int)))==NULL){
+	if((tl->l_frame=malloc((tl->max-tl->min+2)*sizeof(int)))==NULL){
 		alsaplayer_error("CDDA: list data allocation failed");
 		return 1;
 	}
@@ -256,7 +261,8 @@ static int cd_getinfo(int *cdrom_fd, char *cd_dev, struct cd_trk_list *tl)
  * create_socket - create a socket to communicate with the remote server
  * return the fd' int on success, or -1 on error.
  */
-int create_socket (unchar *unchar_address, int int_port)
+static int
+create_socket (const char *unchar_address, int int_port)
 {
 	int sock, len;
 	struct	hostent		*remote;
@@ -293,7 +299,7 @@ int create_socket (unchar *unchar_address, int int_port)
 	}
 
 	/* connect to the server */
-	if (connect (sock, (struct sockaddr *) &server, sizeof (server)) < 0) 
+	if (connect (sock, (struct sockaddr *) &server, sizeof (server)) < 0)
 	{
 		alsaplayer_error("%s\n", strerror (errno));
 		return (-1);
@@ -307,20 +313,21 @@ int create_socket (unchar *unchar_address, int int_port)
  * sent_to_server - send a message to the server, and return the server response
  * on success, or NULL on error
  */
-char * send_to_server (int server_fd, char *message)
+static char *
+send_to_server (int server_fd, char *message)
 {
 	ssize_t	total, i;
 	int 	len = BUFFER_SIZE;
 	char	*response, *temp;
 
-	temp = (char *) malloc(BUFFER_SIZE);
+	temp = malloc(BUFFER_SIZE);
 
 	/* write 'message' to the server */
-	if (send (server_fd, message, strlen (message), MSG_DONTWAIT) < 0) 
+	if (send (server_fd, message, strlen (message), MSG_DONTWAIT) < 0)
 	{
 		alsaplayer_error("%s: %s\n", message, strerror (errno));
 		free (temp);
-		return (NULL);
+		return NULL;
 	}
 
 	if (global_verbose) {
@@ -337,7 +344,7 @@ char * send_to_server (int server_fd, char *message)
 		{
 			alsaplayer_error("%s\n", strerror (errno));
 			free (temp);
-			return (NULL);
+			return NULL;
 		}
 		total += i;
 		if (total + BUFFER_SIZE > len)
@@ -351,7 +358,7 @@ char * send_to_server (int server_fd, char *message)
 	if (total < 2)
 	{
 		free (temp);
-		return (NULL);
+		return NULL;
 	}
 
 	temp[total-2] = '\0';		/* temp[total-1] == \r; temp[total] == \n	*/
@@ -370,20 +377,21 @@ char * send_to_server (int server_fd, char *message)
  * cddb_disc_id - generate the disc ID from the total music time
  * (routine token from the cddb.howto)
  */
-unsigned int cddb_disc_id (struct cd_trk_list *tl)
+static unsigned int
+cddb_disc_id (struct cd_trk_list *tl)
 {
 	int i, t = 0, n = 0;
 
 	/* n == total music time in seconds */
 	i = 0;
-	while (i < tl->max) 
+	while (i < tl->max)
 	{
 		n += cddb_sum((tl->l_min[i] * 60) + tl->l_sec[i]);
 		i++;
 	}
 
 	/* t == lead-out offset seconds - 1st music total time, in seconds */
-	t = ((tl->l_min[tl->max] * 60) + tl->l_sec[tl->max]) 
+	t = ((tl->l_min[tl->max] * 60) + tl->l_sec[tl->max])
 		- ((tl->l_min[0] * 60) + tl->l_sec[0]);
 
 	/*
@@ -403,7 +411,7 @@ int cddb_sum (int n)
 {
 	int ret = 0;
 
-	while (n > 0) 
+	while (n > 0)
 	{
 		ret = ret + (n % 10);
 		n = n/10;
@@ -417,33 +425,29 @@ int cddb_sum (int n)
  * information into the cddb directory. This function returns the filename on
  * success, or NULL on error.
  */
-char * cddb_save_to_disk(char *subdir, int cdID, char *message)
+static char *
+cddb_save_to_disk(char *subdir, int cdID, char *message)
 {
 	FILE *destination;
-	DIR *thedir; 
-	char *path, *retval;
-	char new[strlen (message)], *filename;
-	int i = 0, j = 0;
-
-	/* check if we already have the subdir created */
-	path = (char *) malloc ((strlen (subdir) + strlen (REAL_PATH) + 2) * sizeof (char));
+	DIR *thedir;
+	char path [PATH_MAX];
+	char *new, filename [PATH_MAX];
 
 	/* print the message sent to the server */
-	sprintf(path, "%s", REAL_PATH);
+	snprintf(path, sizeof (path), "%s", real_path);
 	if (! (thedir=opendir(path))) { /* No cddb directory yet! */
 		if ((mkdir(path, 0744)) < 0) {
 			perror("mkdir");
-			free(path);
-			return (NULL);
+			return NULL;
 		}
 	} else {
 		closedir(thedir);
-	}  
-	/* cddb directory should be there at this point */	 
+	}
+	/* cddb directory should be there at this point */
 
-	sprintf (path, "%s/%s", REAL_PATH, subdir);
+	snprintf (path, sizeof (path), "%s/%s", real_path, subdir);
 	if (global_verbose)
-		alsaplayer_error("path = %s", path);	
+		alsaplayer_error("path = %s", path);
 	/* check if we have the directory in the disk */
 	if (! (thedir = opendir (path))) {
 		/* given directory doesn't exist */
@@ -453,60 +457,47 @@ char * cddb_save_to_disk(char *subdir, int cdID, char *message)
 		/* try to create it.. */
 		if ((mkdir (path, 0744)) < 0) {
 			perror ("mkdir");
-			free(path);
-			return (NULL);
+			return NULL;
 		} else {
 			if (global_verbose)
 				printf ("directory created successfully\n");
 		}
 	} else {
 		closedir(thedir);
-	} 
+	}
 
-	while (message[i] != '\n')
-		i++;
-	i++;
-
-	for (; i < (int)strlen (message); i++, j++)
-		new[j] = message[i];
+	new = strchr (message, '\n');
+	new = new != NULL ? new + 1 : message;
 
 	/* save it into the disc */
-	filename = (char *) malloc ((strlen (subdir) + strlen (REAL_PATH) + 11) * sizeof (char));
-	sprintf (filename, "%s/%s/%08x", REAL_PATH, subdir, cdID);
-	retval = strdup (filename);
+	snprintf (filename, sizeof (filename), "%s/%s/%08x", real_path, subdir, cdID);
 	if (global_verbose)
 		alsaplayer_error("filename = %s", filename);
 	/* create the file */
 	destination = fopen (filename, "w");
-	free(filename);
 	if (! destination)
 	{
 		alsaplayer_error("error creating file");
-		free(path);
-		return (NULL);
+		return NULL;
 	}
 
 	/* copy the new string content into the file */
-	for (i = 0; i < (int)strlen (new); i++)
-		fputc (new[i], destination);
-
-	/* free path's memory */
-	free (path);
+	fputs(new, destination);
 
 	/* close the file */
 	fclose (destination);
 
-	return (retval);
+	return strdup (filename);
 }
 
 /*
  * search for the CD info in the hard disk CDDB, returning NULL on error or the
  * filename on success.
  */
-char * cddb_local_lookup (char *path, unsigned int cd_id)
+static char *
+cddb_local_lookup (char *path, unsigned int cd_id)
 {
 	int i, number, fd;
-	char *name;
 	char cdrom_id[9];
 	struct dirent	**directory;
 
@@ -514,9 +505,9 @@ char * cddb_local_lookup (char *path, unsigned int cd_id)
 		alsaplayer_error ("Searching for CDDB entries on %s ... ", path);
 
 	/* try to open the given directory */
-	if (! (opendir (path))) 
+	if (! (opendir (path)))
 	{
-		return (NULL);
+		return NULL;
 	}
 
 	/* get the number of subdirectories in the 'path' dir */
@@ -524,48 +515,45 @@ char * cddb_local_lookup (char *path, unsigned int cd_id)
 	if (number < 0)
 	{
 		alsaplayer_error("scandir\n");
-		return (NULL);
+		return NULL;
 	}
 
 	/* set the cdrom_id */
-	sprintf (cdrom_id, "%08x", cd_id);
+	snprintf (cdrom_id, sizeof (cdrom_id), "%08x", cd_id);
 	cdrom_id[8] = '\0';
 
-	for (i = 0; i < number; i++) 
+	for (i = 0; i < number; i++)
 	{
 		/* ignore '.' and '..' directories */
-		if ((strcmp (directory[i]->d_name, ".")) && (strcmp (directory[i]->d_name, ".."))) 
+		if ((strcmp (directory[i]->d_name, ".")) && (strcmp (directory[i]->d_name, "..")))
 		{
-			name = malloc((strlen (path) + strlen (directory[i]->d_name) + 15) * sizeof(char));
-			sprintf (name, "%s", path);
-			strcat (name, "/");
-			strncat (name, directory[i]->d_name, strlen (directory[i]->d_name));
-			strcat (name, "/");
-			strncat (name, cdrom_id, 8);
-			if ((fd = open (name, O_RDONLY)) >= 0) 
+			char name [PATH_MAX];
+
+			snprintf (name, sizeof (name), "%s/%s/%s", path, directory[i]->d_name, cdrom_id);
+			if ((fd = open (name, O_RDONLY)) >= 0)
 			{
 				if (global_verbose)
 					printf ("OK\n");
 				close (fd);
-				return (name);
+				return strdup (name);
 			}
-			free (name);
 		}
 	}
 
 	if (global_verbose)
 		printf ("not found\n");
-	return (NULL);
+	return NULL;
 }
 
-char*
+static char*
 cut_html_head(char *answer)
 {
 	if(!answer)
 		return NULL;
-		
+
 	char *new_answer;
-	int counter = 0, i = 0;
+	int counter = 0;
+	unsigned i = 0;
 
 	for (i = 0; i < strlen(answer); i++) {
 		if (*(answer+i) == '\n') {
@@ -576,19 +564,20 @@ cut_html_head(char *answer)
 			}
 			counter = 0;
 		}
-		
+
 		counter++;
 	}
 
 	free(answer);
-	return NULL; 
+	return NULL;
 }
 
 /*
  * search for the song in the CDDB given address/port, returning it's name, or
  * NULL if not found.
  */
-char * cddb_lookup (char *address, char *char_port, int discID, struct cd_trk_list *tl)
+static char *
+cddb_lookup (const char *address, const char *char_port, int discID, struct cd_trk_list *tl)
 {
 	int port = atoi (char_port);
 	int server_fd, i, j;
@@ -602,9 +591,9 @@ char * cddb_lookup (char *address, char *char_port, int discID, struct cd_trk_li
 		alsaplayer_error ("Opening Connection to %s:%d ... ", address, port);
 
 	/* get the server fd from the create_socket function */
-	server_fd = create_socket ((unchar *) address, port);
+	server_fd = create_socket (address, port);
 	if (server_fd < 0)
-		return (NULL);
+		return NULL;
 	else
 		if (global_verbose)
 			printf ("OK\n");
@@ -624,36 +613,36 @@ char * cddb_lookup (char *address, char *char_port, int discID, struct cd_trk_li
 
 	if (port > 80) {
 		snprintf (msg, sizeof (msg), "cddb hello %s %s %s %s\r\n\r\n", username, hostname,PACKAGE, VERSION);
-			
+
 		answer = send_to_server (server_fd, msg);
 		if (! answer)
 		{
 			alsaplayer_error("bad response from the server\n");
 			close (server_fd);
-			return (NULL);
+			return NULL;
 		}
 		separator=' ';
 	}
 	/* set another settings before querying the CDDB database */
 	tmpbuf[0] = '\0';
-	for (i = 0; i < tl->max; i++) 
+	for (i = 0; i < tl->max; i++)
 	{
-		/* put the frame offset of the starting location of each track in a string */
+		/* put the block offset of the starting location of each track in a string */
 		//snprintf (offsets, sizeof (offsets), "%s %d ", tmpbuf,
 		snprintf (offsets, sizeof (offsets), "%s%c%d", tmpbuf, separator,
 				tl->l_frame[i] + (75 * (tl->l_sec[i] + (60 * tl->l_min[i]))));
-		strcpy (tmpbuf, offsets);
+		ap_strlcpy (tmpbuf, offsets, sizeof (tmpbuf));
 		counter += tl->l_frame[i] + (75 * tl->l_sec[i] + (60 * tl->l_min[i]));
 	}
 
 	total_secs = tl->l_sec[tl->max] + (tl->l_min[tl->max] * 60);
 
 	/* send it */
-	if (port > 80) 
+	if (port > 80)
 		snprintf (msg, sizeof (msg), "cddb query %08x %d %s %d\r\n", discID, tl->max, offsets, total_secs);
 	else
 		snprintf (msg, sizeof (msg), "GET /~cddb/cddb.cgi?cmd=cddb+query+%08x+%d%s+%d&hello=%s+%s+%s+%s&proto=6 HTTP/1.0\r\n\r\n", discID, tl->max, offsets, total_secs, username, hostname,PACKAGE, VERSION);
-	
+
 	if (answer)
 		free(answer);
 	answer = send_to_server (server_fd, msg);
@@ -661,7 +650,7 @@ char * cddb_lookup (char *address, char *char_port, int discID, struct cd_trk_li
 	{
 		alsaplayer_error("bad response from the server\n");
 		close (server_fd);
-		return (NULL);
+		return NULL;
 	}
 
 	/*
@@ -669,13 +658,13 @@ char * cddb_lookup (char *address, char *char_port, int discID, struct cd_trk_li
 	 * if answer == "211...", found too many matches
 	 * if answer == "202...", found no matches
 	 */
-	
-	if (port <= 80)  
+
+	if (port <= 80)
 		answer = cut_html_head(answer);
-	
+
 	if (!answer)
 		return NULL;
-			 
+
 	i = 0;
 	if (! (strncmp (answer, "211", 3)))
 	{
@@ -687,7 +676,7 @@ char * cddb_lookup (char *address, char *char_port, int discID, struct cd_trk_li
 		}
 
 		/* copy the 1st match to the category */
-		j = 0; 
+		j = 0;
 		i++;
 		while (j < 19 && answer[i] != ' ') {
 			if (! answer[i])
@@ -702,7 +691,7 @@ char * cddb_lookup (char *address, char *char_port, int discID, struct cd_trk_li
 		}
 
 		/* get the new cdID given from the CDDB */
-		j = 0; 
+		j = 0;
 		i++;
 		while (j < 8 && answer[i] != ' ') {
 			if (! answer[i])
@@ -716,7 +705,7 @@ char * cddb_lookup (char *address, char *char_port, int discID, struct cd_trk_li
 			i++;
 		}
 
-	} 
+	}
 	else if (! (strncmp (answer, "200", 3)))
 	{
 		/* get it from the 1st line */
@@ -742,7 +731,7 @@ char * cddb_lookup (char *address, char *char_port, int discID, struct cd_trk_li
 		}
 
 		/* copy the new cdID */
-		j = 0; 
+		j = 0;
 		i++;
 		while (j < 8 && answer[i] != ' ') {
 			if (! answer[i])
@@ -755,25 +744,25 @@ char * cddb_lookup (char *address, char *char_port, int discID, struct cd_trk_li
 				goto error;
 			i++;
 		}
-	} 
-	else 
+	}
+	else
 	{
 	error:
 		alsaplayer_error("Could not find any matches for %08x\n\n", discID);
 		close (server_fd);
 		free(answer);
-		return (NULL);
+		return NULL;
 	}
 
 	/* read from the server */
 
 	if (port > 80)
-		sprintf (msg, "cddb read %s %s\r\n", categ, newID);
+		snprintf (msg, sizeof (msg), "cddb read %s %s\r\n", categ, newID);
 	else {
-		server_fd = create_socket ((unchar *) address, port);
+		server_fd = create_socket (address, port);
 		snprintf (msg, sizeof (msg), "GET /~cddb/cddb.cgi?cmd=cddb+read+%s+%s&hello=%s+%s+%s+%s&proto=6 HTTP/1.0\r\n\r\n", categ, newID, username, hostname,PACKAGE, VERSION);
 	}
-	
+
 	free(answer);
 	answer = send_to_server(server_fd, msg);
 
@@ -781,49 +770,49 @@ char * cddb_lookup (char *address, char *char_port, int discID, struct cd_trk_li
 	{
 		alsaplayer_error("could not receive the informations from %s\n", address);
 		close (server_fd);
-		return (NULL);
+		return NULL;
 	}
 
 	/* save the output into the disc */
 	if (global_verbose)
 	{
-		printf ("Saving CDDB information into %s/%s ...\n", REAL_PATH, newID);
+		printf ("Saving CDDB information into %s/%s ...\n", real_path, newID);
 		printf ("save_to_disk(%s)\n", answer);
 	}
 
-	filename = cddb_save_to_disk(categ, discID, answer); 
+	filename = cddb_save_to_disk(categ, discID, answer);
 	if (! filename)
 	{
 		alsaplayer_error("could not create the file %s/%s, check permission\n", categ, newID);
 		close (server_fd);
-		return (NULL);
+		return NULL;
 	}
 
-	if (global_verbose) 
+	if (global_verbose)
 		puts ("");
 
 	/* close the fd */
 	close (server_fd);
-	free(answer);	
+	free(answer);
 	return (filename);
 }
 
 /*
  * open the filename and put music title's into the global variable
  */
-void cddb_read_file (char *file, struct cdda_local_data *data)
+static void
+cddb_read_file (char *file, struct cdda_local_data *data)
 {
 	char line[BUFFER_SIZE], name[BUFFER_SIZE];
-	char *token = NULL, *tmp, *divider, *s, *post;
-	char album[BUFFER_SIZE];
-	int i, index = 1;
+	char *token = NULL, *tmp, *divider, *s;
+	int i, indx = 1;
 	FILE *f;
 
 	/* try to open the file */
 	f = fopen (file, "r");
 	if (! f) {
 		alsaplayer_error("couldn't open file");
-		return;	  
+		return;
 	}
 	/* read it line by line */
 	while (!feof (f)) {
@@ -832,7 +821,7 @@ void cddb_read_file (char *file, struct cdda_local_data *data)
 				/* check if is the music name.. */
 				if ((strstr (line, "TTITLE"))) {
 					token = strtok (line, "=");
-					if (!token) { 
+					if (!token) {
 						alsaplayer_error("error: TTITLE has no arguments");
 						continue;
 					}
@@ -840,42 +829,39 @@ void cddb_read_file (char *file, struct cdda_local_data *data)
 					if (!token) {
 						alsaplayer_error("TTITLE has no arguments");
 						continue;
-					}	  
+					}
 					/* seek for the \r character */
 					for (i = 0; i < (int)strlen (token); i++) {
 						if ((token[i] == '\n') || (token[i] == '\r'))
 							break;
 					}
-					if (sscanf(line, "TTITLE%d=", &index)) {
-						//alsaplayer_error("Found index %d", index);
-						index++;
+					if (sscanf(line, "TTITLE%d=", &indx)) {
+						//alsaplayer_error("Found indx %d", indx);
+						indx++;
 					} else {
-						index = 1;
-						alsaplayer_error("Error reading index number!");
+						indx = 1;
+						alsaplayer_error("Error reading indx number!");
 					}
 					token[i] = '\0';
-					sprintf (name, "%s", token);
-					if (data->tracks[index].track) {
-						post = (char *)malloc(strlen(data->tracks[index].track) + strlen(name) + 1);
-						*post = '\0';
-						strcat(post, data->tracks[index].track);
-						strcat(post, name);
-						free(data->tracks[index].track);
-						data->tracks[index].track = strdup(post);
-						free(post);
-					} else {	
-						data->tracks[index].track = strdup (name);
-					}	
+					snprintf (name, sizeof (name), "%s", token);
+					if (data->tracks[indx].track) {
+						char post [512];
+						snprintf (post, sizeof (post), "%s%s", data->tracks[indx].track, name);
+						free(data->tracks[indx].track);
+						data->tracks[indx].track = strdup(post);
+					} else {
+						data->tracks[indx].track = strdup (name);
+					}
 				}
 				continue;
 			} else {
 				// workaround album name on multiple lines, need improvment
 				if (data->tracks[1].album)
 					continue;
-				  
+
 				/* print the album name */
 				tmp = strtok (line, "=");
-				if (!tmp) { 
+				if (!tmp) {
 					alsaplayer_error ("error: no arguments given on %s", line);
 					continue;
 				}
@@ -885,22 +871,22 @@ void cddb_read_file (char *file, struct cdda_local_data *data)
 					continue;
 				}
 				divider = strstr (tmp, " / ");
-				if (!divider) { 
+				if (!divider) {
 					alsaplayer_error("No divider found in DTITLE");
-					
+
 					    	data->tracks[1].artist = strdup(tmp);
 						data->tracks[1].album = strdup(tmp);
 				} else {
 						data->tracks[1].album = strdup (divider+3);
 						tmp[strlen(tmp)-strlen(data->tracks[1].album)-3] = '\0';
 						data->tracks[1].artist = strdup (tmp);
-				}	
+				}
 				if ((s = strstr(data->tracks[1].artist, "\r"))) {
 					*s = '\0';
 				}
 				if ((s = strstr(data->tracks[1].artist, "\n"))) {
 					*s = '\0';
-				} 
+				}
 				if ((s = strstr(data->tracks[1].album, "\r"))) {
 					*s = '\0';
 				}
@@ -923,13 +909,14 @@ void cddb_read_file (char *file, struct cdda_local_data *data)
 }
 
 
-void cddb_update_info(struct cdda_local_data *data)
+static void
+cddb_update_info(struct cdda_local_data *data)
 {
 	char *file_name = NULL;
-	char *cddb_servername = NULL;
-	char *cddb_serverport = NULL;
+	const char *cddb_servername = NULL;
+	const char *cddb_serverport = NULL;
 	struct cd_trk_list *tl;
-	int index;
+	int indx;
 	unsigned int cd_id;
 
 	if (!data)
@@ -938,51 +925,45 @@ void cddb_update_info(struct cdda_local_data *data)
 	cd_id = cddb_disc_id(tl);
 
 	/* try to get the audio info from the hard disk.. */
-	file_name = cddb_local_lookup(REAL_PATH, cd_id);
+	file_name = cddb_local_lookup(real_path, cd_id);
 	if (file_name)
 	{
 		/* open the 'file_name' and search for album information */
 		cddb_read_file (file_name, data);
-	}	
-	else 
+	}
+	else
 	{
 		/* if could not, try to get it from the internet connection.. */
 		cddb_servername = prefs_get_string(ap_prefs, "cdda", "cddb_servername", "freedb.freedb.org");
-		cddb_serverport = prefs_get_string(ap_prefs, "cdda", "cddb_serverport", "80"); 
+		cddb_serverport = prefs_get_string(ap_prefs, "cdda", "cddb_serverport", "80");
 		if (global_verbose)
-			alsaplayer_error("CDDB server: %s:%s", cddb_servername, cddb_serverport);	    
+			alsaplayer_error("CDDB server: %s:%s", cddb_servername, cddb_serverport);
 		file_name = cddb_lookup(cddb_servername, cddb_serverport, cd_id, tl);
 		if (file_name) {
 
 			/* fine, now we have the hard disk access! so, let's use it's information */
 			free(file_name);
-			file_name = cddb_local_lookup(REAL_PATH, cd_id);
+			file_name = cddb_local_lookup(real_path, cd_id);
 			if (file_name)
 				cddb_read_file (file_name, data);
 			else {
-				for (index = 1; index <= tl->max; index++)
-					data->tracks[index].track = strdup ("unrecognized song");
+				for (indx = 1; indx <= tl->max; indx++)
+					data->tracks[indx].track = strdup ("unrecognized song");
 			}
 
 		} else {
-			for (index = 1; index <= tl->max; index++) 
-				data->tracks[index].track = strdup ("unrecognized song");
+			for (indx = 1; indx <= tl->max; indx++)
+				data->tracks[indx].track = strdup ("unrecognized song");
 		}
-	}	
+	}
 }
 
 
 static int cdda_init(void)
 {
-	char *prefsdir;
-
-	prefsdir = get_prefsdir();
-	REAL_PATH = (char *)malloc((strlen(prefsdir) + 8) * sizeof(char));
-	if (REAL_PATH) {
-		sprintf(REAL_PATH, "%s/cddb", prefsdir);
-		return 1;
-	}
-	return 0;
+	get_prefsdir();
+	real_path [0] = 0;
+	return 1;
 }
 
 
@@ -994,13 +975,15 @@ static float cdda_can_handle(const char *name)
 	if (ext)
 		if ((strcasecmp(ext, ".cdda") == 0)) {
 			return 1.0;
-		}				
-	return 0.0;			
+		}
+	return 0.0;
 }
 
 
 
-void cd_adder(void *data) {
+static void
+cd_adder(void *data)
+{
 	int i;
 	intptr_t nr_tracks;
 	char track_name[1024];
@@ -1011,7 +994,7 @@ void cd_adder(void *data) {
 	nr_tracks = (intptr_t)data;
 
 	for (i=1;i <= nr_tracks;i++) {
-		sprintf(track_name, "Track %02d.cdda", i);
+		snprintf(track_name, sizeof (track_name), "Track %02d.cdda", i);
 		ap_add_path(global_session_id, track_name);
 	}
 	pthread_exit(NULL);
@@ -1020,7 +1003,7 @@ void cd_adder(void *data) {
 
 static int cdda_open(input_object *obj, const char *name)
 {
-	struct cdda_local_data *data;	
+	struct cdda_local_data *data;
 	const char *fname;
 	char device_name[1024];
 
@@ -1033,10 +1016,10 @@ static int cdda_open(input_object *obj, const char *name)
 	else fname++; // Ignore '/'
 
 	if (ap_prefs) {
-		strcpy(device_name, prefs_get_string(ap_prefs, "cdda", "device", DEFAULT_DEVICE));
+		ap_strlcpy(device_name, prefs_get_string(ap_prefs, "cdda", "device", DEFAULT_DEVICE), sizeof (device_name));
 	} else {
-		strcpy(device_name, DEFAULT_DEVICE);
-	}	
+		ap_strlcpy(device_name, DEFAULT_DEVICE, sizeof (device_name));
+	}
 #ifdef DEBUG
 	alsaplayer_error("device = %s, name = %s\n", device_name, fname);
 #endif
@@ -1044,7 +1027,7 @@ static int cdda_open(input_object *obj, const char *name)
 	obj->local_data = malloc(sizeof(struct cdda_local_data));
 	if (!obj->local_data) {
 		return 0;
-	}		
+	}
 	data = (struct cdda_local_data *)obj->local_data;
 	memset(data->tracks, 0, sizeof(data->tracks));
 	if(cd_getinfo(&data->cdrom_fd, device_name, &data->tl)) {
@@ -1054,9 +1037,9 @@ static int cdda_open(input_object *obj, const char *name)
 	}
 
 
-#ifdef DEBUG	
-	cd_disp_TOC(&data->tl);	
-	alsaplayer_error("IFRAMESIZE = %d\n", IFRAMESIZE);
+#ifdef DEBUG
+	cd_disp_TOC(&data->tl);
+	alsaplayer_error("IBLOCKSIZE = %d\n", IBLOCKSIZE);
 #endif
 
 	obj->nr_channels = 2;
@@ -1065,7 +1048,7 @@ static int cdda_open(input_object *obj, const char *name)
 	data->track_start  = 0;
 	data->rel_pos = 0;
 	data->track_nr = 0;
-	strcpy(data->device_path, device_name);
+	ap_strlcpy(data->device_path, device_name, sizeof (data->device_path));
 
 	if (prefs_get_bool(ap_prefs, "cdda", "do_cddb_lookup", 1)) {
 		/* look up cd in cddb */
@@ -1085,11 +1068,11 @@ static int cdda_open(input_object *obj, const char *name)
 		obj->local_data = NULL;
 		return 0;
 	} else {
-#ifdef DEBUG		
+#ifdef DEBUG
 		alsaplayer_error("Found track number %d (%s)\n", data->track_nr, fname);
 #endif
 		data->track_start = data->tl.starts[data->track_nr-1];
-		data->track_length = data->tl.starts[data->track_nr] - 
+		data->track_length = data->tl.starts[data->track_nr] -
 			data->tl.starts[data->track_nr-1];
 		data->rel_pos = 0;
 	}
@@ -1115,14 +1098,14 @@ static void cdda_close(input_object *obj)
 	for (i = 0; i < MAX_TRACKS;i++) {
 		if (data->tracks[i].album) {
 			free(data->tracks[i].album);
-		}	
+		}
 		if (data->tracks[i].artist) {
 			free(data->tracks[i].artist);
-		}	
+		}
 		if (data->tracks[i].track) {
 			free(data->tracks[i].track);
-		}	
-	}	
+		}
+	}
 	close(data->cdrom_fd);
 	if (data->tl.starts) free(data->tl.starts);
 	data->tl.starts = NULL;
@@ -1142,82 +1125,94 @@ static void cdda_close(input_object *obj)
 }
 
 
-static int cdda_play_frame(input_object *obj, char *buf)
+static int cdda_play_block(input_object *obj, short *buf)
 {
 	struct cdda_local_data *data;
-	unsigned char bla[CD_FRAMESIZE_RAW*FRAME_LEN];
+	unsigned char bla[CD_FRAMESIZE_RAW*BLOCK_LEN];
 
 	if (!obj)
 		return 0;
-	data = (struct cdda_local_data *)obj->local_data;	
+	data = (struct cdda_local_data *)obj->local_data;
 
 	if (!data) {
 		return 0;
-	}		
-	if (!data->track_length || 
+	}
+	if (!data->track_length ||
 			(data->rel_pos > data->track_length)) {
-#ifdef DEBUG		
+#ifdef DEBUG
 		alsaplayer_error("rel_pos = %d, start = %d, end = %d\n",
-				data->rel_pos, data->track_start, 
+				data->rel_pos, data->track_start,
 				data->track_start + data->track_length);
-#endif		
+#endif
 		return 0;
 	}
 	memset(bla, 0, sizeof(bla));
-	if (cd_read_audio(data->cdrom_fd, data->track_start + data->rel_pos, FRAME_LEN, bla)) {
+	if (cd_read_audio(data->cdrom_fd, data->track_start + data->rel_pos, BLOCK_LEN, bla)) {
 		return 0;
 	}
-	data->rel_pos += FRAME_LEN;
+	data->rel_pos += BLOCK_LEN;
 	if (buf) {
-		memcpy(buf, bla, (CD_FRAMESIZE_RAW * FRAME_LEN));
+		memcpy(buf, bla, (CD_FRAMESIZE_RAW * BLOCK_LEN));
 #ifdef __BYTE_ORDER
 #if __BYTE_ORDER == __BIG_ENDIAN
-		swab (buf, buf, (CD_FRAMESIZE_RAW * FRAME_LEN));
-#endif	
-#endif		
+		swab (buf, buf, (CD_FRAMESIZE_RAW * BLOCK_LEN));
+#endif
+#endif
 	}
 	return 1;
 }
 
 
-static int cdda_frame_seek(input_object *obj, int index)
+static int cdda_block_seek(input_object *obj, int indx)
 {
-	struct cdda_local_data *data;	
+	struct cdda_local_data *data;
 	if (!obj)
 		return 0;
-	data = (struct cdda_local_data *)obj->local_data;	
+	data = (struct cdda_local_data *)obj->local_data;
 	if (data)
-		data->rel_pos = (index * FRAME_LEN);
+		data->rel_pos = (indx * BLOCK_LEN);
 	return 1;
 }
 
 
-static int cdda_frame_size(input_object *obj)
+static int cdda_block_size(input_object * UNUSED (obj))
 {
-	int size = (CD_FRAMESIZE_RAW * FRAME_LEN);
-	return size;
+	return (CD_FRAMESIZE_RAW * BLOCK_LEN) / sizeof (short) ;
 }
 
 
 
-static int cdda_nr_frames(input_object *obj)
+static int cdda_nr_blocks(input_object *obj)
 {
-	struct cdda_local_data *data;	
-	int nr_frames = 0;
+	struct cdda_local_data *data;
+	int nr_blocks = 0;
 
 	if (!obj)
 		return 0;
 	data = (struct cdda_local_data *)obj->local_data;
 
 	if (data)
-		nr_frames = data->track_length >> 2;	
-	return nr_frames;
+		nr_blocks = data->track_length >> 2;
+	return nr_blocks;
 }
 
-
-static  long cdda_frame_to_sec(input_object *obj, int frame)
+static int64_t
+cdda_frame_count (input_object *obj)
 {
-	unsigned long byte_count = FRAME_LEN * frame * CD_FRAMESIZE_RAW;
+	struct cdda_local_data *data;
+
+	if (!obj)
+		return 0;
+	data = (struct cdda_local_data *)obj->local_data;
+
+	if (data && data->track_length > 0)
+		return data->track_length;
+	return -1;
+}
+
+static  long cdda_block_to_sec(input_object * UNUSED (obj), int block)
+{
+	unsigned long byte_count = BLOCK_LEN * block * CD_FRAMESIZE_RAW;
 
 	return (byte_count / 1764); /* 44Khz stereo 16 bit, fixed */
 	/* 1764 = 176400 / 100 voor de 100ste seconden representatie */
@@ -1226,10 +1221,10 @@ static  long cdda_frame_to_sec(input_object *obj, int frame)
 
 static int cdda_sample_rate(input_object *obj)
 {
-	struct cdda_local_data *data;	
+	struct cdda_local_data *data;
 	if (!obj)
 		return 0;
-	data = (struct cdda_local_data *)obj->local_data;	
+	data = (struct cdda_local_data *)obj->local_data;
 	return (data ? data->samplerate : 0);
 }
 
@@ -1243,29 +1238,26 @@ static int cdda_channels(input_object *obj)
 
 static int cdda_stream_info(input_object *obj, stream_info *info)
 {
-	struct cdda_local_data *data;	
-	struct cd_trk_list *tl;
+	struct cdda_local_data *data;
 
 	assert(obj);
 	assert(info);
 
 	data = (struct cdda_local_data *)obj->local_data;
 
-	tl = &data->tl;
-
-	sprintf(info->stream_type, "CD Audio, 44KHz, stereo");
+	snprintf(info->stream_type, sizeof (info->stream_type), "CD Audio, 44KHz, stereo");
 	if (data->tracks[1].artist)
-	  sprintf(info->artist, "%s", data->tracks[1].artist);
+	  snprintf(info->artist, sizeof (info->artist), "%s", data->tracks[1].artist);
 	if (data->tracks[1].album)
-	  sprintf(info->album, "%s", data->tracks[1].album);
+	  snprintf(info->album, sizeof (info->album), "%s", data->tracks[1].album);
 	info->status[0] = 0;
 	if (data->track_nr < 0)
 		info->title[0] = 0;
 	else if (data->track_nr == 0)
-		sprintf(info->title, "Full CD length playback");
+		snprintf(info->title, sizeof (info->title), "Full CD length playback");
 	else if (data->tracks[data->track_nr].track)
-			sprintf(info->title, "%s", data->tracks[data->track_nr].track);
-	
+			snprintf(info->title, sizeof (info->title), "%s", data->tracks[data->track_nr].track);
+
 	//alsaplayer_error("title = %s\nalbum = %s\nartist = %s",
 	//		info->title, info->album, info->artist);
 	return 1;
@@ -1290,58 +1282,31 @@ static void cdda_shutdown(void)
 }
 
 
-static int cdda_track_seek(input_object *obj, int track)
+static int cdda_track_seek(input_object * UNUSED (obj), int UNUSED (track))
 {
 	return 1;
 }
 
 
 static input_plugin cdda_plugin;
-/*
-   = {
-   INPUT_PLUGIN_VERSION,
-   0,
-   "CDDA player v1.1",
-   "Andy Lo A Foe <andy@alsaplayer.org>",
-   NULL,
-   cdda_init,
-   cdda_shutdown,
-   NULL,
-   cdda_can_handle,
-   cdda_open,
-   cdda_close,
-   cdda_play_frame,
-   cdda_frame_seek,
-   cdda_frame_size,
-   cdda_nr_frames,
-   cdda_frame_to_sec,
-   cdda_sample_rate,
-   cdda_channels,
-   cdda_stream_info,
-   cdda_nr_tracks,
-   cdda_track_seek
-   };
-   */
-#ifdef __cplusplus
-extern "C" {
-#endif
 
 input_plugin *input_plugin_info(void)
 {
 	memset(&cdda_plugin, 0, sizeof(input_plugin));
 	cdda_plugin.version = INPUT_PLUGIN_VERSION;
 	cdda_plugin.name = "CDDA player v1.2";
-	cdda_plugin.author = "Andy Lo A Foe <andy@alsaplayer.org>";
+	cdda_plugin.author = "Andy Lo A Foe <andy@loafoe.com>";
 	cdda_plugin.init = cdda_init;
 	cdda_plugin.shutdown = cdda_shutdown;
 	cdda_plugin.can_handle = cdda_can_handle;
 	cdda_plugin.open = cdda_open;
 	cdda_plugin.close = cdda_close;
-	cdda_plugin.play_frame = cdda_play_frame;
-	cdda_plugin.frame_seek = cdda_frame_seek;
-	cdda_plugin.frame_size = cdda_frame_size;
-	cdda_plugin.nr_frames = cdda_nr_frames;
-	cdda_plugin.frame_to_sec = cdda_frame_to_sec;
+	cdda_plugin.play_block = cdda_play_block;
+	cdda_plugin.block_seek = cdda_block_seek;
+	cdda_plugin.block_size = cdda_block_size;
+	cdda_plugin.nr_blocks = cdda_nr_blocks;
+	cdda_plugin.frame_count = cdda_frame_count;
+	cdda_plugin.block_to_sec = cdda_block_to_sec;
 	cdda_plugin.sample_rate = cdda_sample_rate;
 	cdda_plugin.channels = cdda_channels;
 	cdda_plugin.stream_info = cdda_stream_info;
@@ -1349,7 +1314,3 @@ input_plugin *input_plugin_info(void)
 	cdda_plugin.track_seek = cdda_track_seek;
 	return &cdda_plugin;
 }
-
-#ifdef __cplusplus
-}
-#endif
