@@ -28,12 +28,15 @@
 
 #include <string>
 #include <cstdio>
-#include <malloc.h>
+#include <stdlib.h>
 #include <cstring>
 #include <cmath>
+
+#include "config.h"
+
 #include "input_plugin.h"
 #include "alsaplayer_error.h"
-#include "config.h"
+#include "ap_string.h"
 
 #include "FlacStream.h"
 #include "FlacEngine.h"
@@ -64,39 +67,47 @@ flac_sample_rate (input_object * obj)
 
 
 static int
-flac_frame_size (input_object * obj)
+flac_block_size (input_object * obj)
 {
     if (!obj)
 	return 0;
-    return obj->frame_size;
+    return obj->block_size;
 }
 
 
 static int
-flac_nr_frames (input_object * obj)
+flac_nr_blocks (input_object * obj)
 {
     if (!obj)
 	return 0;
-    return obj->nr_frames;
+    return obj->nr_blocks;
+}
+
+static int64_t
+flac_frame_count (input_object * obj)
+{
+    if (!obj)
+	return 0;
+    return ((int64_t) obj->nr_blocks) * obj->block_size;
 }
 
 
 static long
-flac_frame_to_centisec (input_object * obj, int frame)
+flac_block_to_centisec (input_object * obj, int block)
 {
     if (!obj)
 	return 0;
-    
+
     Flac::FlacStream * f = (Flac::FlacStream *) obj->local_data;
     if (!f)
 	return 0;
 
-    return (long) f->engine ()->frameTime (frame) * 100;
+    return (long) f->engine ()->blockTime (block) * 100;
 }
 
 
 static int
-flac_frame_seek (input_object * obj, int frame)
+flac_block_seek (input_object * obj, int block)
 {
     if (!obj)
 	return 0;
@@ -104,20 +115,20 @@ flac_frame_seek (input_object * obj, int frame)
     Flac::FlacStream * f = (Flac::FlacStream *) obj->local_data;
     if (!f)
       return 0;
-    return f->engine ()->seekToFrame (frame);
+    return f->engine ()->seekToBlock (block);
 }
 
 
 static int
-flac_play_frame (input_object * obj, char * buf)
+flac_play_block (input_object * obj, short * buf)
 {
     if (!obj || !buf)
 	return 0;
-    
+
     Flac::FlacStream * f = (Flac::FlacStream *) obj->local_data;
     if (!f)
 	return 0;
-    return f->engine ()->decodeFrame (buf);
+    return f->engine ()->decodeBlock (buf);
 }
 
 
@@ -150,12 +161,12 @@ flac_open (input_object * obj, const char * name)
 	    else
 		f = new Flac::FlacStream (name, rdr);
 	}
-#ifdef HAVE_LIBOGGFLC	
+#ifdef HAVE_LIBOGGFLC
 	else
 	{
 	    f = new Flac::OggFlacStream (name, rdr);
 	}
-#endif	
+#endif
     }
     catch (...)
     {
@@ -165,12 +176,12 @@ flac_open (input_object * obj, const char * name)
 	return 0;
     }
 
-    if (f->open ())
+    if (f && f->open ())
     {
-	obj->frame_size  = f->engine ()->apFrameSize ();
+	obj->block_size  = f->engine ()->apBlockSize ();
 
 	// attach a song info tag
-	
+
 	if (Flac::FlacTag::hasTag (f->name ()))
 	{
 	    Flac::FlacTag * t = Flac::FlacTag::newTag (f->name ());
@@ -183,7 +194,7 @@ flac_open (input_object * obj, const char * name)
 	    obj->flags |= P_FILEBASED;
 	obj->nr_channels  = f->engine ()->apChannels ();
 	obj->flags       |= P_REENTRANT;
-	obj->nr_frames    = f->engine ()->apFrames ();
+	obj->nr_blocks    = f->engine ()->apBlocks ();
 	obj->nr_tracks    = 1;
 	obj->ready        = 1;
 	obj->local_data   = (void *) f;
@@ -194,14 +205,13 @@ flac_open (input_object * obj, const char * name)
 	alsaplayer_error ("flac_open: unable to open flac stream or "
 			  "unsupported flac stream (%s)", name);
 	delete f;
-	obj->frame_size  = 0;
+	obj->block_size  = 0;
 	obj->nr_channels = 0;
 	obj->flags       = 0;
-	obj->nr_frames   = 0;
+	obj->nr_blocks   = 0;
 	obj->nr_tracks   = 0;
 	obj->ready       = 0;
 	obj->local_data  = 0;
-	alsaplayer_error ("flac_open: failed");
 	return 0;
     }
 }
@@ -217,7 +227,7 @@ flac_stream_info (input_object * obj, stream_info * info)
     if (!f)
 	return 0;
 
-    sprintf (info->stream_type, "%d-bit %dKhz %s flac", 
+    snprintf (info->stream_type, sizeof (info->stream_type), "%d-bit %dKhz %s flac",
 	     f->bps (), f->sampleRate () / 1000,
 	     f->channels () == 1 ? "mono" :
 	     f->channels () == 2 ? "stereo" : "multi-channel");
@@ -225,29 +235,26 @@ flac_stream_info (input_object * obj, stream_info * info)
     Flac::FlacTag * t = f->tag ();
     if (t && ! t->title ().empty ())
     {
-	// strncpy limits come from looking at input_plugin.h; these should
-        // really be defined as constants somewhere
-
-	strncpy (info->artist, t->artist ().c_str (), 128);
-	strncpy (info->title, t->title ().c_str (), 128);
-	strncpy (info->album, t->album ().c_str (), 128);
-	strncpy (info->genre, t->genre ().c_str (), 128);
-	strncpy (info->year, t->year ().c_str (), 10);
-	strncpy (info->track, t->track ().c_str (), 10);
-	strncpy (info->comment, t->comment ().c_str (), 128);
+	ap_strlcpy (info->artist, t->artist ().c_str (), sizeof (info->artist));
+	ap_strlcpy (info->title, t->title ().c_str (), sizeof (info->title));
+	ap_strlcpy (info->album, t->album ().c_str (), sizeof (info->album));
+	ap_strlcpy (info->genre, t->genre ().c_str (), sizeof (info->genre));
+	ap_strlcpy (info->year, t->year ().c_str (), sizeof (info->year));
+	ap_strlcpy (info->track, t->track ().c_str (), sizeof (info->track));
+	ap_strlcpy (info->comment, t->comment ().c_str (), sizeof (info->comment));
     }
     else
     {
 	// use stream name
-	char * fname = strrchr (f->name ().c_str (), '/');
+	const char * fname = strrchr (f->name ().c_str (), '/');
 	if (fname)
 	{
 	    fname++;
-	    strncpy (info->title, fname, 128);
+	    ap_strlcpy (info->title, fname, sizeof (info->title));
 	}
 	else
 	    info->title[0] = 0;
-	
+
 	info->artist[0]  = 0;
 	info->title[0]   = 0;
 	info->album[0]   = 0;
@@ -270,15 +277,15 @@ flac_can_handle (const char * name)
 	if (strncmp(name, "http://", 7) == 0) {
 		return 0.0;
 	}
-	char *ext = strrchr(name, '.');
+	const char *ext = strrchr(name, '.');
 	if (!ext)
 		return 0.0;
 	ext++;
 	if (strcasecmp(ext, "flac") == 0) /* Always support .flac files */
-		return 1.0;
+		return 0.8;
 	if (strcasecmp(ext, "ogg")) /* Ignore all non .ogg files */
 		return 0.0;
-	
+
 	res = Flac::FlacStream::isFlacStream (name);
 #ifdef HAVE_LIBOGGFLC
 	if (res != 1.0) {
@@ -327,9 +334,9 @@ input_plugin_info (void)
 
     flac_plugin.version      = INPUT_PLUGIN_VERSION;
 #ifdef LEGACY_FLAC
-    flac_plugin.name         = "flac player v1.1.2";
+    flac_plugin.name         = (char *)"flac player v1.1.2";
 #else
-    flac_plugin.name         = "flac player v1.1.3/1.1.4/1.2";
+    flac_plugin.name         = (char *)"flac player v1.1.3/1.1.4/1.2";
 #endif
     flac_plugin.author       = "Drew Hess";
     flac_plugin.init         = flac_init;
@@ -337,15 +344,16 @@ input_plugin_info (void)
     flac_plugin.can_handle   = flac_can_handle;
     flac_plugin.open         = flac_open;
     flac_plugin.close        = flac_close;
-    flac_plugin.play_frame   = flac_play_frame;
-    flac_plugin.frame_seek   = flac_frame_seek;
-    flac_plugin.frame_size   = flac_frame_size;
-    flac_plugin.nr_frames    = flac_nr_frames;
-    flac_plugin.frame_to_sec = flac_frame_to_centisec;
+    flac_plugin.play_block   = flac_play_block;
+    flac_plugin.block_seek   = flac_block_seek;
+    flac_plugin.block_size   = flac_block_size;
+    flac_plugin.nr_blocks    = flac_nr_blocks;
+    flac_plugin.frame_count  = flac_frame_count;
+    flac_plugin.block_to_sec = flac_block_to_centisec;
     flac_plugin.sample_rate  = flac_sample_rate;
     flac_plugin.channels     = flac_channels;
     flac_plugin.stream_info  = flac_stream_info;
-    
+
     return & flac_plugin;
 }
 
