@@ -4,19 +4,21 @@
 //
 // Copyright (c) 2002 by Drew Hess <dhess@bothan.net>
 //
-//    This program is free software; you can redistribute it and/or modify
-//    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2 of the License, or
-//    (at your option) any later version.
-//
-//    This program is distributed in the hope that it will be useful,
-//    but WITHOUT ANY WARRANTY; without even the implied warranty of
-//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//    GNU General Public License for more details.
-//
-//    You should have received a copy of the GNU General Public License
-//    along with this program; if not, write to the Free Software
-//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+/*  This file is part of AlsaPlayer.
+ *
+ *  AlsaPlayer is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  AlsaPlayer is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, see <http://www.gnu.org/licenses/>.
+ */
 //-------------------------------------------------------------------------
 
 #include "FlacEngine.h"
@@ -43,7 +45,7 @@ FlacStream::apError (const char * msg)
     if (!_reportErrors)
 	return;
     alsaplayer_error (msg);
-    
+
 } // FlacStream::apError
 
 
@@ -67,13 +69,13 @@ FlacStream::isFlacStream (const std::string & name)
         return false;
     FlacStream f (name, rdr, false);
     return f.open ();
-    
+
 } // FlacStream::isFlacStream
 
 
-FlacStream::FlacStream (const std::string & name,
+FlacStream::FlacStream (const std::string & name_,
 			reader_type * f,
-			bool reportErrors)
+			bool)
      : _engine (new FlacEngine (this)),
       _mcbSuccess (false),
       _datasource (f),
@@ -85,7 +87,7 @@ FlacStream::FlacStream (const std::string & name,
       _totalSamp (0),
       _decoder (0),
       _tag (0),
-      _name (name)
+      _name (name_)
 {
 } // FlacStream::FlacStream
 
@@ -112,7 +114,7 @@ FlacStream::open ()
 {
     // it's illegal to call this on an already open stream
     if (_decoder) {
-	apError ("FlacStream::open(): existing decoder");    
+	apError ("FlacStream::open(): existing decoder");
 	return false;
     }
 
@@ -125,7 +127,7 @@ FlacStream::open ()
     bool status = true;
     status &= FLAC__stream_decoder_set_read_callback (_decoder,
 						      readCallBack);
-    status &= FLAC__stream_decoder_set_write_callback (_decoder, 
+    status &= FLAC__stream_decoder_set_write_callback (_decoder,
 						     writeCallBack);
     status &= FLAC__stream_decoder_set_metadata_callback (_decoder,
 							metaCallBack);
@@ -134,11 +136,11 @@ FlacStream::open ()
     status &= FLAC__stream_decoder_set_client_data (_decoder, (void *) this);
 
     if (!status) {
-	apError("FlacStream::open(): status error, huh?");    
+	apError("FlacStream::open(): status error, huh?");
 	return false;
     }
     status = (FLAC__stream_decoder_init (_decoder) == FLAC__STREAM_DECODER_SEARCH_FOR_METADATA);
-    
+
     if (!status) {
 	apError("FlacStream::open(): can't initialize stream decoder");
 #else
@@ -160,31 +162,31 @@ FlacStream::open ()
 
     // this will invoke the metaCallBack
     if (!FLAC__stream_decoder_process_until_end_of_metadata (_decoder)) {
-	apError("FlacStream::open(): decoder error");    
+	apError("FlacStream::open(): decoder error");
 	return false;
     }
-    // now that we've opened the stream, tell the engine it's safe to 
+    // now that we've opened the stream, tell the engine it's safe to
     // initialize itself.
-    
+
     if (!_engine->init ()) {
 	apError("FlacStream::open(): engine init failed");
 	return false;
     }
     // return the metaCallBack's status
     return _mcbSuccess;
-    
+
 } // FlacStream::open
 
 
 bool
-FlacStream::processOneFrame ()
+FlacStream::processOneBlock ()
 {
     if (!_decoder)
 	return false;
 
     return FLAC__stream_decoder_process_single (_decoder);
 
-} // FlacStream::processOneFrame
+} // FlacStream::processOneBlock
 
 
 void
@@ -207,7 +209,7 @@ FlacStream::realMetaCallBack (const FLAC__StreamMetadata * md)
 
     // assume the worst!
     _mcbSuccess = false;
-    
+
     if (!FlacEngine::supportsBlockSizes (md->data.stream_info.min_blocksize,
 					 md->data.stream_info.max_blocksize))
 	return;
@@ -230,7 +232,7 @@ FlacStream::realMetaCallBack (const FLAC__StreamMetadata * md)
 
 // static
 void
-FlacStream::metaCallBack (const FLAC__StreamDecoder * decoder,
+FlacStream::metaCallBack (const FLAC__StreamDecoder *,
 			  const FLAC__StreamMetadata * md,
 			  void * client_data)
 {
@@ -243,30 +245,30 @@ FlacStream::metaCallBack (const FLAC__StreamDecoder * decoder,
     }
 
     f->realMetaCallBack (md);
-    
+
 } // FlacStream::metaCallBack
 
 
 void
-FlacStream::realErrCallBack (const char * name,
+FlacStream::realErrCallBack (const char * name_,
 			     FLAC__StreamDecoderErrorStatus status)
 {
     switch (status)
     {
     case FLAC__STREAM_DECODER_ERROR_STATUS_LOST_SYNC:
-	apError ("%s: the decoder lost synchronization", name);
+	apError ("%s: the decoder lost synchronization", name_);
 	break;
 
     case FLAC__STREAM_DECODER_ERROR_STATUS_BAD_HEADER:
-	apError ("%s: corrupted frame header", name);
+	apError ("%s: corrupted block header", name_);
 	break;
 
     case FLAC__STREAM_DECODER_ERROR_STATUS_FRAME_CRC_MISMATCH:
-	apError ("%s: frame CRC error", name);
+	apError ("%s: block CRC error", name_);
 	break;
 
     default:
-	apError ("%s: an unknown error occurred", name);
+	apError ("%s: an unknown error occurred", name_);
     }
 
 } // FlacStream::realErrCallBack
@@ -274,7 +276,7 @@ FlacStream::realErrCallBack (const char * name,
 
 // static
 void
-FlacStream::errCallBack (const FLAC__StreamDecoder * decoder,
+FlacStream::errCallBack (const FLAC__StreamDecoder *,
 			 FLAC__StreamDecoderErrorStatus status,
 			 void * client_data)
 {
@@ -291,11 +293,11 @@ FlacStream::errCallBack (const FLAC__StreamDecoder * decoder,
 
 
 FLAC__StreamDecoderWriteStatus
-FlacStream::realWriteCallBack (const FLAC__Frame * frame,
+FlacStream::realWriteCallBack (const FLAC__Frame * block,
 			       const FLAC__int32 * const buffer[])
 {
-    return _engine->writeBuf (frame, buffer, channels (), bps ()) ? 
-	FLAC__STREAM_DECODER_WRITE_STATUS_CONTINUE : 
+    return _engine->writeBuf (block, buffer, channels (), bps ()) ?
+	FLAC__STREAM_DECODER_WRITE_STATUS_CONTINUE :
 	FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
 
 } // FlacStream::realWriteCallBack
@@ -304,29 +306,29 @@ FlacStream::realWriteCallBack (const FLAC__Frame * frame,
 // static
 FLAC__StreamDecoderWriteStatus
 FlacStream::writeCallBack (const FLAC__StreamDecoder *,
-			 const FLAC__Frame * frame,
+			 const FLAC__Frame * block,
 			 const FLAC__int32 * const buffer[],
 			 void * client_data)
 {
     if (!client_data)
 	return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
-    
+
     FlacStream * f = (FlacStream *) client_data;
     if (!f)
 	return FLAC__STREAM_DECODER_WRITE_STATUS_ABORT;
 
-    return f->realWriteCallBack (frame, buffer);
-    
+    return f->realWriteCallBack (block, buffer);
+
 } // FlacStream::writeCallBack
 
 
 FLAC__StreamDecoderReadStatus
-FlacStream::realReadCallBack (FLAC__byte buffer[], unsigned * bytes)
+FlacStream::realReadCallBack (FLAC__byte buffer[], size_t * bytes)
 {
     *bytes = reader_read (buffer, *bytes, _datasource);
     return *bytes > 0 ? FLAC__STREAM_DECODER_READ_STATUS_CONTINUE :
-	reader_eof (_datasource) ? 
-	FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM : 
+	reader_eof (_datasource) ?
+	FLAC__STREAM_DECODER_READ_STATUS_END_OF_STREAM :
 	FLAC__STREAM_DECODER_READ_STATUS_ABORT;
 
 } // FlacStream::realReadCallBack
@@ -336,7 +338,7 @@ FlacStream::realReadCallBack (FLAC__byte buffer[], unsigned * bytes)
 FLAC__StreamDecoderReadStatus
 FlacStream::readCallBack (const FLAC__StreamDecoder *,
 			  FLAC__byte buffer[],
-			  unsigned * bytes,
+			  size_t * bytes,
 			  void * client_data)
 {
     if (!client_data)
