@@ -4,25 +4,33 @@
 //
 //  Copyright (c) 2002 by Drew Hess <dhess@bothan.net>
 //
-//    This program is free software; you can redistribute it and/or modify
-//    it under the terms of the GNU General Public License as published by
-//    the Free Software Foundation; either version 2 of the License, or
-//    (at your option) any later version.
-//
-//    This program is distributed in the hope that it will be useful,
-//    but WITHOUT ANY WARRANTY; without even the implied warranty of
-//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-//    GNU General Public License for more details.
-//
-//    You should have received a copy of the GNU General Public License
-//    along with this program; if not, write to the Free Software
-//    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+/*  This file is part of AlsaPlayer.
+ *
+ *  AlsaPlayer is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
+ *
+ *  AlsaPlayer is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, see <http://www.gnu.org/licenses/>.
+ */
 //-------------------------------------------------------------------------
 
 
 #ifndef _FLAC_STREAM_H_
 #define _FLAC_STREAM_H_
 
+#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT < 8
+#define LEGACY_FLAC
+#else
+#undef LEGACY_FLAC
+#endif
+
 #include <string>
 #include "reader.h"
 
@@ -49,7 +57,7 @@ class FlacStream
 
     static bool isFlacStream (const std::string & name);
 
-    
+
  public:
 
     //---------------------------------------------------------------
@@ -141,7 +149,7 @@ class FlacStream
 
 
     //----------------------------------------------------------------
-    // Process the next flac frame in the stream.  This has the side-
+    // Process the next flac block in the stream.  This has the side-
     // effect of calling the write callback if there is data to fetch
     // in the stream.
     //
@@ -149,7 +157,7 @@ class FlacStream
     // there's an error in the stream decoder.
     //----------------------------------------------------------------
 
-    virtual bool                processOneFrame ();
+    virtual bool                processOneBlock ();
 
 
     //-------------------------------------------------------------------
@@ -179,8 +187,8 @@ class FlacStream
     void         realErrCallBack (const char * name,
 				  FLAC__StreamDecoderErrorStatus status);
 
-    FLAC__StreamDecoderWriteStatus 
-                 realWriteCallBack (const FLAC__Frame * frame,
+    FLAC__StreamDecoderWriteStatus
+                 realWriteCallBack (const FLAC__Frame * block,
 				    const FLAC__int32 * const buffer[]);
 
 
@@ -189,9 +197,9 @@ class FlacStream
     // streams.
     //------------------------------------------------------------
 
-    FLAC__StreamDecoderReadStatus 
+    FLAC__StreamDecoderReadStatus
                  realReadCallBack (FLAC__byte buffer[],
-				   unsigned * bytes);
+				   size_t * bytes);
 
 
  protected:
@@ -211,7 +219,7 @@ class FlacStream
     unsigned int  _sampPerBlock;  // number of samples per block of data
     FLAC__uint64  _totalSamp;     // total number of samples in the stream
 
-    
+
  private:
 
     //-----------------------------------------------------------------
@@ -224,16 +232,16 @@ class FlacStream
 			      const FLAC__StreamMetadata * md,
 			      void * cilent_data);
 
-    static FLAC__StreamDecoderWriteStatus 
+    static FLAC__StreamDecoderWriteStatus
 	writeCallBack (const FLAC__StreamDecoder * decoder,
-		       const FLAC__Frame * frame,
+		       const FLAC__Frame * block,
 		       const FLAC__int32 * const buffer[],
 		       void * client_data);
 
     static FLAC__StreamDecoderReadStatus
 	readCallBack (const FLAC__StreamDecoder * decoder,
 		      FLAC__byte buffer[],
-		      unsigned * bytes,
+		      size_t * bytes,
 		      void * client_data);
 
     static void errCallBack (const FLAC__StreamDecoder * decoder,
@@ -268,9 +276,9 @@ FlacStream::engine () const
 }
 
 inline void
-FlacStream::setTag (FlacTag * tag)
+FlacStream::setTag (FlacTag * tag_)
 {
-    _tag = tag;
+    _tag = tag_;
 }
 
 inline const std::string &
@@ -310,7 +318,7 @@ FlacStream::channels () const
 }
 
 inline bool
-FlacStream::seekAbsolute (FLAC__uint64 sample)
+FlacStream::seekAbsolute (FLAC__uint64)
 {
     return false;
 }
