@@ -1,7 +1,7 @@
 /*
  * AlsaPlayer MAD plugin.
  *
- * Copyright (C) 2001-2003 Andy Lo A Foe <andy@alsaplayer.org>
+ * Copyright (C) 2001-2003 Andy Lo A Foe <andy@loafoe.com>
  *
  *  This file is part of AlsaPlayer.
  *
@@ -17,9 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, see <http://www.gnu.org/licenses/>.
- *
- *  $Id: mad_engine.c 1301 2007-10-11 20:07:39Z dominique_libre $
- */ 
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -38,10 +36,12 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/mman.h>
+
 #include "input_plugin.h"
 #include "alsaplayer_error.h"
 #include "reader.h"
 #include "prefs.h"
+#include "ap_string.h"
 
 #define MAD_BUFSIZE (32 * 1024)
 
@@ -50,10 +50,6 @@ extern "C" { 	/* Make sure MAD symbols are not mangled
 		 * since we compile them with regular gcc */
 #endif
 
-#ifndef HAVE_LIBMAD /* Needed if we use our own (outdated) copy */
-#include "frame.h"
-#include "synth.h"
-#endif	
 #include <mad.h>
 
 #ifdef __cplusplus
@@ -65,7 +61,7 @@ extern "C" { 	/* Make sure MAD symbols are not mangled
 #define BLOCK_SIZE 4096
 #define MAX_NUM_SAMPLES 8192
 #define STREAM_BUFFER_SIZE	(32 * 1024)
-#define FRAME_RESERVE	2000
+#define BLOCK_RESERVE	2000
 
 # if !defined(O_BINARY)
 #  define O_BINARY  0
@@ -77,12 +73,12 @@ struct mad_local_data {
 	long map_offset;
 	int bytes_avail;
 	struct stat stat;
-	ssize_t	*frames;
-	int highest_frame;
-	int current_frame;
+	ssize_t	*blocks;
+	int highest_block;
+	int current_block;
 	char path[FILENAME_MAX+1];
 	char filename[FILENAME_MAX+1];
-	struct mad_synth  synth; 
+	struct mad_synth  synth;
 	struct mad_stream stream;
 	struct mad_frame  frame;
 	struct xing xing;
@@ -98,18 +94,18 @@ struct mad_local_data {
 	int parse_id3;
 	int parsed_id3;
 	char str[17];
-};		
+};
 
 
 /*
  * NAME:	error_str()
  * DESCRIPTION:	return a string describing a MAD error
  */
-	static
-char const *error_str(enum mad_error error, char *str)
+static char const *
+error_str(struct mad_local_data *data)
 {
 
-	switch (error) {
+	switch (data->stream.error) {
 		case MAD_ERROR_BUFLEN:
 		case MAD_ERROR_BUFPTR:
 			/* these errors are handled specially and/or should not occur */
@@ -124,7 +120,7 @@ char const *error_str(enum mad_error error, char *str)
 		case MAD_ERROR_BADCRC:		 return ("CRC check failed");
 		case MAD_ERROR_BADBITALLOC:	 return ("forbidden bit allocation value");
 		case MAD_ERROR_BADSCALEFACTOR:	 return ("bad scalefactor index");
-		case MAD_ERROR_BADFRAMELEN:	 return ("bad frame length");
+		case MAD_ERROR_BADFRAMELEN:	 return ("bad block length");
 		case MAD_ERROR_BADBIGVALUES:	 return ("bad big_values count");
 		case MAD_ERROR_BADBLOCKTYPE:	 return ("reserved block_type");
 		case MAD_ERROR_BADSCFSI:	 return ("bad scalefactor selection info");
@@ -136,8 +132,8 @@ char const *error_str(enum mad_error error, char *str)
 		default:;
 	}
 
-	sprintf(str, "error 0x%04x", error);
-	return str;
+	snprintf(data->str, sizeof (data->str), "error 0x%04x", data->stream.error);
+	return data->str;
 }
 
 
@@ -167,7 +163,7 @@ static void fill_buffer(struct mad_local_data *data, long offset)
 		bytes_read = reader_read(data->mad_map, MAD_BUFSIZE, data->mad_fd);
 		data->bytes_avail = bytes_read;
 		data->map_offset = offset;
-	} else {	
+	} else {
 		memmove(data->mad_map, data->mad_map + MAD_BUFSIZE - data->bytes_avail, data->bytes_avail);
 		bytes_read = reader_read(data->mad_map + data->bytes_avail, MAD_BUFSIZE - data->bytes_avail, data->mad_fd);
 		data->map_offset += (MAD_BUFSIZE - data->bytes_avail);
@@ -186,7 +182,7 @@ static void mad_init_decoder(struct mad_local_data *data)
 }
 
 
-static int mad_frame_seek(input_object *obj, int frame)
+static int mad_frame_seek(input_object *obj, int block)
 {
 	struct mad_local_data *data;
 	struct mad_header header;
@@ -200,17 +196,17 @@ static int mad_frame_seek(input_object *obj, int frame)
 	if (!data || !data->seekable)
 		return 0;
 
-	//alsaplayer_error("frame_seek(..., %d)", frame);
+	//alsaplayer_error("block_seek(..., %d)", block);
 	mad_header_init(&header);
 
 	data->bytes_avail = 0;
-	if (frame <= data->highest_frame) {
+	if (block <= data->highest_block) {
 		skip = 0;
 
-		if (frame > 4) {
+		if (block > 4) {
 			skip = 3;
 		}
-		byte_offset = data->frames[frame-skip];
+		byte_offset = data->blocks[block-skip];
 
 		/* Prepare the buffer for a read */
 		fill_buffer(data, byte_offset);
@@ -223,44 +219,44 @@ static int mad_frame_seek(input_object *obj, int frame)
 			if (skip == 0)
 				mad_synth_frame (&data->synth, &data->frame);
 		}
-		data->bytes_avail = data->stream.bufend - 
+		data->bytes_avail = data->stream.bufend -
 			data->stream.next_frame;
-		data->current_frame = frame;
+		data->current_block = block;
 		data->seeking = 0;
-		return data->current_frame;
+		return data->current_block;
 	}
 
 	data->seeking = 1;
-	fill_buffer(data, data->frames[data->highest_frame]);
+	fill_buffer(data, data->blocks[data->highest_block]);
 	mad_stream_buffer(&data->stream, data->mad_map, data->bytes_avail);
-	while (data->highest_frame < frame) {
+	while (data->highest_block < block) {
 		if (data->bytes_avail < 3072) {
 			fill_buffer(data, data->map_offset + MAD_BUFSIZE - data->bytes_avail);
 			mad_stream_buffer(&data->stream, data->mad_map, data->bytes_avail);
-		}	
+		}
 		if (mad_header_decode(&header, &data->stream) == -1) {
 			if (!MAD_RECOVERABLE(data->stream.error)) {
 				fill_buffer(data, 0);
 				mad_stream_buffer(&data->stream, data->mad_map, data->bytes_avail);
 				data->seeking = 0;
 				return 0;
-			}				
+			}
 		}
-		data->frames[++data->highest_frame] =
+		data->blocks[++data->highest_block] =
 			data->map_offset + data->stream.this_frame - data->mad_map;
 		data->bytes_avail = data->stream.bufend - data->stream.next_frame;
 	}
 
-	data->current_frame = data->highest_frame;
-	if (data->current_frame > 4) {
+	data->current_block = data->highest_block;
+	if (data->current_block > 4) {
 		skip = 3;
-		fill_buffer(data, data->frames[data->current_frame-skip]);
+		fill_buffer(data, data->blocks[data->current_block-skip]);
 		mad_stream_buffer(&data->stream, data->mad_map, data->bytes_avail);
 		skip++;
-		while (skip != 0) { 
+		while (skip != 0) {
 			skip--;
 			mad_frame_decode(&data->frame, &data->stream);
-			if (skip == 0) 
+			if (skip == 0)
 				mad_synth_frame (&data->synth, &data->frame);
 			data->bytes_avail = data->stream.bufend -
 				data->stream.next_frame;
@@ -268,24 +264,24 @@ static int mad_frame_seek(input_object *obj, int frame)
 	}
 
 	data->seeking = 0;
-	return data->current_frame;
+	return data->current_block;
 
 	return 0;
 }
 
-static int mad_frame_size(input_object *obj)
+static int mad_block_size(input_object *obj)
 {
-	if (!obj || !obj->frame_size) {
-		puts("No frame size!");
+	if (!obj || !obj->block_size) {
+		puts("No block size!");
 		return 0;
 	}
-	return obj->frame_size;
+	return obj->block_size / sizeof (short);
 }
 
 
 /* #define MAD_DEBUG */
 
-static int mad_play_frame(input_object *obj, char *buf)
+static int mad_play_block(input_object *obj, short *buf)
 {
 	struct mad_local_data *data;
 	struct mad_pcm *pcm;
@@ -314,33 +310,33 @@ static int mad_play_frame(input_object *obj, char *buf)
 	if (mad_frame_decode(&data->frame, &data->stream) == -1) {
 		if (!MAD_RECOVERABLE(data->stream.error)) {
 			/*
-			   alsaplayer_error("MAD error: %s (%d). fatal", 
-			   error_str(data->stream.error, data->str),
+			   alsaplayer_error("MAD error: %s (%d). fatal",
+			   error_str(data),
 			   data->bytes_avail);
-			   */	
+			   */
 			mad_frame_mute(&data->frame);
 			return 0;
 		} else {
 			if (reader_eof(data->mad_fd)) {
 				return 0;
-			}	
-			//alsaplayer_error("MAD error: %s (not fatal)", error_str(data->stream.error, data->str)); 
-			memset(buf, 0, obj->frame_size);
+			}
+			//alsaplayer_error("MAD error: %s (not fatal)", error_str(data));
+			memset(buf, 0, obj->block_size);
 			return 1;
 		}
 	}
-	data->current_frame++;
-	if (data->seekable && data->current_frame < (obj->nr_frames + FRAME_RESERVE)) {
-		data->frames[data->current_frame] = 
+	data->current_block++;
+	if (data->seekable && data->current_block < (obj->nr_blocks + BLOCK_RESERVE)) {
+		data->blocks[data->current_block] =
 			data->map_offset + data->stream.this_frame - data->mad_map;
-		if (data->current_frame > 3 && 
-				(data->frames[data->current_frame] -
-				 data->frames[data->current_frame-3]) < 6) {
+		if (data->current_block > 3 &&
+				(data->blocks[data->current_block] -
+				 data->blocks[data->current_block-3]) < 6) {
 			return 0;
-		}		
-		if (data->highest_frame < data->current_frame)
-			data->highest_frame = data->current_frame;
-	}				
+		}
+		if (data->highest_block < data->current_block)
+			data->highest_block = data->current_block;
+	}
 
 	mad_synth_frame (&data->synth, &data->frame);
 
@@ -350,24 +346,24 @@ static int mad_play_frame(input_object *obj, char *buf)
 		nsamples = pcm->length;
 		nchannels = pcm->channels;
 		if (nchannels != obj->nr_channels) {
-			alsaplayer_error("ERROR: bad data stream! (channels: %d != %d, frame %d)",
-					nchannels, 
+			alsaplayer_error("ERROR: bad data stream! (channels: %d != %d, block %d)",
+					nchannels,
 					obj->nr_channels,
-					data->current_frame);
+					data->current_block);
 			mad_frame_mute(&data->frame);
-			memset(buf, 0, obj->frame_size);
+			memset(buf, 0, obj->block_size);
 			return 1;
-		}	
+		}
 		obj->nr_channels = nchannels;
 		if (data->samplerate != data->frame.header.samplerate) {
-			alsaplayer_error("ERROR: bad data stream! (samplerate: %d != %d, frame %d)",
-					data->samplerate, 
+			alsaplayer_error("ERROR: bad data stream! (samplerate: %d != %d, block %d)",
+					data->samplerate,
 					data->frame.header.samplerate,
-					data->current_frame);
+					data->current_block);
 			mad_frame_mute(&data->frame);
-			memset(buf, 0, obj->frame_size);
+			memset(buf, 0, obj->block_size);
 			return 1;
-		}	
+		}
 		data->samplerate = data->frame.header.samplerate;
 		left_ch = pcm->samples[0];
 		right_ch = pcm->samples[1];
@@ -377,7 +373,7 @@ static int mad_play_frame(input_object *obj, char *buf)
 				*output++ = my_scale(*(left_ch-1));
 			} else { /* nchannels == 2 */
 				*output++ = my_scale(*(right_ch++));
-			}	
+			}
 
 		}
 	}
@@ -386,7 +382,7 @@ static int mad_play_frame(input_object *obj, char *buf)
 }
 
 
-static  long mad_frame_to_sec(input_object *obj, int frame)
+static  long mad_block_to_sec(input_object *obj, int block)
 {
 	struct mad_local_data *data;
 	unsigned long sec = 0;
@@ -395,18 +391,27 @@ static  long mad_frame_to_sec(input_object *obj, int frame)
 		return 0;
 	data = (struct mad_local_data *)obj->local_data;
 	if (data) {
-		sec = data->samplerate ? frame * (obj->frame_size >> 2) /
+		sec = data->samplerate ? block * (obj->block_size >> 2) /
 			(data->samplerate / 100) : 0;
-	}			
+	}
 	return sec;
 }
 
 
-static int mad_nr_frames(input_object *obj)
+static int mad_nr_blocks(input_object *obj)
 {
 	if (!obj)
 		return 0;
-	return obj->nr_frames;
+	return obj->nr_blocks;
+}
+
+static int64_t
+mad_frame_count(input_object *obj)
+{
+	if (!obj)
+		return -1;
+	return ((int64_t) obj->nr_blocks) * 1148;
+
 }
 
 /* TODO: Move all id3 code into the separated file. */
@@ -422,49 +427,47 @@ static void rstrip (char *s) {
 }
 
 /* Convert from synchsafe integer */
-static unsigned int from_synchsafe4 (unsigned char *buf)
+static unsigned int from_synchsafe4 (const char *buf)
 {
 	return (buf [3]) + (buf [2] << 7) + (buf [1] << 14) + (buf [0] << 21);
 }
 
-static unsigned int from_synchsafe3 (unsigned char *buf)
+static unsigned int from_synchsafe3 (const char *buf)
 {
-	return (buf [3]) + (buf [2] << 7) + (buf [1] << 14);
+	return (buf [2]) + (buf [1] << 7) + (buf [0] << 14);
 }
 
 /* Fill filed */
-static void fill_from_id3v2 (char *dst, char *src, int max, int size)
+static void fill_from_id3v2 (char *dst, const char *src, int max, int size)
 {
 	// it can be iso8851-1==0 utf16==1 utf16be==2 and utf8==3
-	if (( *src < 0) || (*src > 3))
+	if ((*src < 0) || (*src > 3))
 		return;
 
 	char *conv = NULL;
 
-	int min = size-1 > max ? max : size-1;
-	
 #ifdef HAVE_GLIB2
 	if (*src == 0)
-		conv = g_convert((const gchar *)src+1, (gssize) size, "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
+		conv = g_convert(src+1, (gssize) size-1, "UTF-8", "ISO-8859-1", NULL, NULL, NULL); // gsti: size -1
 	else if (*src == 1)
-		conv = g_convert((const gchar *)src+1, (gssize) size, "UTF-8", "UTF-16", NULL, NULL, NULL);
+		conv = g_convert(src+1, (gssize) size-1, "UTF-8", "UTF-16", NULL, NULL, NULL);
 	else if (*src == 2)
-		conv = g_convert((const gchar *)src+1, (gssize) size, "UTF-8", "UTF-16BE", NULL, NULL, NULL);
+		conv = g_convert(src+1, (gssize) size-1, "UTF-8", "UTF-16BE", NULL, NULL, NULL);
  	else
 		conv = g_strndup(src+1, size);
 #else
-//	alsaplayer_error ("Warning: Without glib2 you get different encodings.");
+	alsaplayer_error ("Warning: Without glib2 you get different encodings.");
 	conv = strndup(src+1, size);
 #endif
-	
+
 	if (!conv)	// convert error
 		return;
-	
-	strncpy (dst, conv, min);
-	
+
+	ap_strlcpy (dst, conv, max);
+
 	if (conv)
 		free(conv);
-	
+
 }
 
 static char *genres[] = {
@@ -498,7 +501,7 @@ static char *genres[] = {
 static void parse_id3 (const char *path, stream_info *info)
 {
 	void *fd;
-	unsigned char buf [2024];
+	char buf [2024];
 	unsigned char g;
 
 	/* Open stream */
@@ -521,9 +524,10 @@ static void parse_id3 (const char *path, stream_info *info)
 		int f_extended_header = buf [5] & (1<<6);
 		int f_experimental = buf [5] & (1<<5);
 		int header_size = from_synchsafe4 (buf + 6);
+		unsigned int f_seek = 0;
 		int name_size = buf [3] == 2 ? 3 : 4;
 		int ext_size = 0;
-		
+
 		if (f_extended_header) {
 //			alsaplayer_error ("FIXME: Extended header found in mp3."
 //					"Please contact alsaplayer team.\n"
@@ -559,24 +563,25 @@ static void parse_id3 (const char *path, stream_info *info)
 					ext_size = from_synchsafe3 (b);
 				else
 					ext_size = from_synchsafe4 (b);
-				
+
 				if (reader_seek (fd, ext_size - 4, SEEK_CUR) < 0) {
 					reader_close (fd);
 					return;
 				}
-			
+
 			}
-			
-		/* -- -- read frames -- -- */
-		while (reader_tell (fd) <= header_size + 10) {
-			unsigned int size;
-			
-			
-			/* Get name of this frame */
+
+		/* -- -- read blocks -- -- */
+		while (f_seek <= header_size + 10) {
+			unsigned int size = 0;
+
+
+			/* Get name of this block */
 			if (reader_read (buf, name_size, fd) != (unsigned)name_size) {
 				reader_close (fd);
 				return;
 			}
+			f_seek += name_size;
 
 			if (buf [0] == '\0')  break;
 			if (buf [0] < 'A')  break;
@@ -590,6 +595,7 @@ static void parse_id3 (const char *path, stream_info *info)
 					reader_close (fd);
 					return;
 				}
+				f_seek += 3;
 
 				size = from_synchsafe3 (sb);
 			} else {
@@ -599,24 +605,25 @@ static void parse_id3 (const char *path, stream_info *info)
 					reader_close (fd);
 					return;
 				}
+				f_seek += 4;
 
 				size = from_synchsafe4 (sb);
 			}
 
-			/* skip frame flags */
+			/* skip block flags */
 //			if (reader_seek (fd, 1, SEEK_CUR) == -1) {
 //				reader_close (fd);
 //				return;
 //			}
 
-			int start = 0;		
+			int start = 0;
 			// read them
 			char b[2];
 			if (reader_read (b, 2, fd) != 2) {
 				reader_close (fd);
 				return;
 			} else {
-			
+
 				if (b[1] & (1 << 6)) {
 //					printf ("Grouping added\n");
 					start++;
@@ -633,12 +640,13 @@ static void parse_id3 (const char *path, stream_info *info)
 				if (b[1] & (1 << 0)) {
 //					printf ("Length added\n");
 					start+=4;
-				}	
+				}
 			}
-				
-				
-				
-				if (size>=1024) {
+			f_seek += 2;
+
+
+
+			if (size>=1024) {
 				/* I will not support such long tags...
 				 * Only if someone ask for it...
 				 * not now... */
@@ -647,6 +655,10 @@ static void parse_id3 (const char *path, stream_info *info)
 					reader_close (fd);
 					return;
 				}
+				if (f_seek + size < f_seek) { // size is negative. WHY?
+					break;
+				}
+				f_seek += size;
 
 				continue;
 			}
@@ -657,8 +669,14 @@ static void parse_id3 (const char *path, stream_info *info)
 				reader_close (fd);
 				return;
 			}
+			f_seek += size;
+			if(f_seek + size < f_seek) {
+				break;
+			}
+			/* make sure buffer is zero-terminated for sscanf */
+			buf[name_size + size] = 0;
 
-			/* !!! Ok. There we have frame name and data. */
+			/* !!! Ok. There we have block name and data. */
 			/* Lets use it. */
 			if (name_size == 4) {
 				if (memcmp (buf, "TIT2", 4)==0)
@@ -697,7 +715,7 @@ static void parse_id3 (const char *path, stream_info *info)
 				}
 			} /* end of 'if name_size == 4' */
 
-		} /* end of frames read */
+		} /* end of blocks read */
 
 		/* end parsing */
 		reader_close (fd);
@@ -717,26 +735,26 @@ static void parse_id3 (const char *path, stream_info *info)
 	}
 
 	if (memcmp(buf, "TAG", 3) == 0) {
-		/* ID3v1 frame found */
+		/* ID3v1 block found */
 
 		/* title */
-		strncpy (info->title, buf + 3, 30);
+		ap_strlcpy (info->title, buf + 3, 30);
 		rstrip (info->title);
 
 		/* artist */
-		strncpy (info->artist, buf + 33, 30);
+		ap_strlcpy (info->artist, buf + 33, 30);
 		rstrip (info->artist);
 
 		/* album */
-		strncpy (info->album, buf + 63, 30);
+		ap_strlcpy (info->album, buf + 63, 30);
 		rstrip (info->album);
 
 		/* year */
-		strncpy (info->year, buf + 93, 4);
+		ap_strlcpy (info->year, buf + 93, 4);
 		rstrip (info->year);
 
 		/* comment */
-		strncpy (info->comment, buf + 97, 28);
+		ap_strlcpy (info->comment, buf + 97, 28);
 		rstrip (info->comment);
 
 		/* track number */
@@ -758,7 +776,7 @@ static void parse_id3 (const char *path, stream_info *info)
 
 static int mad_stream_info(input_object *obj, stream_info *info)
 {
-	struct mad_local_data *data;	
+	struct mad_local_data *data;
 	unsigned len;
 	char metadata[256];
 	char *s, *p;
@@ -770,7 +788,7 @@ static int mad_stream_info(input_object *obj, stream_info *info)
 
 	if (data) {
 		if (!data->parse_id3) {
-			sprintf(data->sinfo.title, "%s", data->filename);
+			snprintf(data->sinfo.title, sizeof (data->sinfo.title), "%s", data->filename);
 		} else if (!data->parsed_id3) {
 			if (reader_seekable(data->mad_fd)) {
 				parse_id3 (data->path, &data->sinfo);
@@ -785,7 +803,7 @@ static int mad_stream_info(input_object *obj, stream_info *info)
 						*(s--) = '\0';
 				}
 			}
-			strncpy (data->sinfo.path, data->path, sizeof(data->sinfo.path));
+			ap_strlcpy (data->sinfo.path, data->path, sizeof(data->sinfo.path));
 			data->parsed_id3 = 1;
 		}
 		memset(metadata, 0, sizeof(metadata));
@@ -800,19 +818,19 @@ static int mad_stream_info(input_object *obj, stream_info *info)
 					alsaplayer_error("Malformed metadata: \"%s\"", metadata);
 				}
 			}
-		}	
+		}
 		/* Restore permanently filled info */
 		memcpy (info, &data->sinfo, sizeof (data->sinfo));
 
 		/* Compose path, stream_type and status fields */
-		sprintf(info->stream_type, "MP3 %dKHz %s %-3ldkbit",
+		snprintf(info->stream_type, sizeof (info->stream_type), "MP3 %dKHz %s %-3ldkbit",
 				data->frame.header.samplerate / 1000,
 				obj->nr_channels == 2 ? "stereo" : "mono",
 				data->frame.header.bitrate / 1000);
 
 		if (data->seeking)
-			sprintf(info->status, "Seeking...");
-	}				
+			snprintf(info->status, sizeof (info->status), "Seeking...");
+	}
 	return 1;
 }
 
@@ -824,24 +842,21 @@ static int mad_sample_rate(input_object *obj)
 	if (!obj)
 		return 44100;
 	data = (struct mad_local_data *)obj->local_data;
-	if (data)	
+	if (data)
 		return data->samplerate;
 	return 44100;
 }
 
-static int mad_init(void) 
+static int mad_init(void)
 {
 	return 1;
-}		
+}
 
 
 static int mad_channels(input_object *obj)
 {
-	struct mad_local_data *data;
-
 	if (!obj)
 		return 2; /* Default to 2, this is flaky at best! */
-	data = (struct mad_local_data *)obj->local_data;
 
 	return obj->nr_channels;
 }
@@ -849,7 +864,7 @@ static int mad_channels(input_object *obj)
 
 static float mad_can_handle(const char *path)
 {
-	char *ext;      
+	char *ext;
 	ext = strrchr(path, '.');
 
 	if (strncmp(path, "http://", 7) == 0)
@@ -866,9 +881,9 @@ static float mad_can_handle(const char *path)
 }
 
 
-static ssize_t find_initial_frame(uint8_t *buf, int size)
+static ssize_t find_initial_block(uint8_t *buf, int size)
 {
-	uint8_t *data = buf;			
+	uint8_t *data = buf;
 	int ext_header = 0;
 	int pos = 0;
 	ssize_t header_size = 0;
@@ -877,17 +892,17 @@ static ssize_t find_initial_frame(uint8_t *buf, int size)
 			pos += 2;
 		if (data[pos] == 0xff && (data[pos+1] == 0xfb
 					|| data[pos+1] == 0xfa
-					|| data[pos+1] == 0xf3 
+					|| data[pos+1] == 0xf3
 					|| data[pos+1] == 0xf2
 					|| data[pos+1] == 0xe2
 					|| data[pos+1] == 0xe3)) {
 			return pos;
-		}	
+		}
 		if (pos == 0 && data[pos] == 0x0d && data[pos+1] == 0x0a) {
 			return -1; /* Let MAD figure this out */
-		}	
+		}
 		if (pos == 0 && (data[pos] == 'I' && data[pos+1] == 'D' && data[pos+2] == '3')) {
-			header_size = (data[pos + 6] << 21) + 
+			header_size = (data[pos + 6] << 21) +
 				(data[pos + 7] << 14) +
 				(data[pos + 8] << 7) +
 				data[pos + 9]; /* syncsafe integer */
@@ -904,7 +919,7 @@ static ssize_t find_initial_frame(uint8_t *buf, int size)
 			if (header_size > STREAM_BUFFER_SIZE) {
 				//alsaplayer_error("Header larger than 32K (%d)", header_size);
 				return header_size;
-			}	
+			}
 			return header_size;
 		} else if (data[pos] == 'R' && data[pos+1] == 'I' &&
 				data[pos+2] == 'F' && data[pos+3] == 'F') {
@@ -920,12 +935,12 @@ static ssize_t find_initial_frame(uint8_t *buf, int size)
 			}
 			puts("MAD debug: invalid header");
 			return -1;
-		} else if (pos == 0 && data[pos] == 'T' && data[pos+1] == 'A' && 
+		} else if (pos == 0 && data[pos] == 'T' && data[pos+1] == 'A' &&
 				data[pos+2] == 'G') {
 			return 128;	/* TAG is fixed 128 bytes, we assume! */
 		}  else {
 			pos++;
-		}				
+		}
 	}
 	alsaplayer_error(
 			"MAD debug: potential problem file or unhandled info block\n"
@@ -949,8 +964,8 @@ static void reader_status(void *ptr, const char *str)
 	if (data) {
 		//fprintf(stdout, "%s     \r", str);
 		//fflush(stdout);
-		strcpy(data->sinfo.status, str);
-	}	
+		ap_strlcpy(data->sinfo.status, str, sizeof (data->sinfo.status));
+	}
 }
 
 
@@ -958,7 +973,6 @@ static int mad_open(input_object *obj, const char *path)
 {
 	struct mad_local_data *data;
 	char *p;
-	int mode;
 
 	if (!obj)
 		return 0;
@@ -981,7 +995,7 @@ static int mad_open(input_object *obj, const char *path)
 
 	if (strncasecmp(path, "http://", 7) == 0) {
 		obj->flags |= P_STREAMBASED;
-		strcpy(data->sinfo.status, "Prebuffering");
+		ap_strlcpy(data->sinfo.status, "Prebuffering", sizeof (data->sinfo.status));
 	} else {
 		obj->flags |= P_FILEBASED;
 	}
@@ -1001,13 +1015,13 @@ static int mad_open(input_object *obj, const char *path)
 	fill_buffer(data, -1);
 	//alsaplayer_error("initial bytes_avail = %d", data->bytes_avail);
 	if (obj->flags & P_PERFECTSEEK) {
-		data->offset = find_initial_frame(data->mad_map, 
+		data->offset = find_initial_block(data->mad_map,
 				data->bytes_avail < STREAM_BUFFER_SIZE ? data->bytes_avail :
 				STREAM_BUFFER_SIZE);
 	} else {
 		data->offset = 0;
-	}	
-	data->highest_frame = 0;
+	}
+	data->highest_block = 0;
 	if (data->offset < 0) {
 		//fprintf(stderr, "mad_open() couldn't find valid MPEG header\n");
 		data->offset = 0;
@@ -1022,8 +1036,8 @@ static int mad_open(input_object *obj, const char *path)
 		mad_stream_buffer(&data->stream, data->mad_map + data->offset,
 				data->bytes_avail - data->offset);
 		data->bytes_avail -= data->offset;
-	}	
-first_frame:
+	}
+first_block:
 
 	if ((mad_header_decode(&data->frame.header, &data->stream) == -1)) {
 		switch (data->stream.error) {
@@ -1032,22 +1046,22 @@ first_frame:
 			case MAD_ERROR_LOSTSYNC:
 			case MAD_ERROR_BADEMPHASIS:
 			case MAD_ERROR_BADBITRATE:
-			case MAD_ERROR_BADLAYER:	
-			case MAD_ERROR_BADSAMPLERATE:	
-				//alsaplayer_error("Error %x (frame %d)", data->stream.error, data->current_frame);
+			case MAD_ERROR_BADLAYER:
+			case MAD_ERROR_BADSAMPLERATE:
+				//alsaplayer_error("Error %x (block %d)", data->stream.error, data->current_block);
 				data->bytes_avail-=(data->stream.next_frame - data->stream.this_frame);
-				goto first_frame;
+				goto first_block;
 				break;
 			case MAD_ERROR_BADBITALLOC:
 				return 0;
 			case MAD_ERROR_BADCRC:
-				alsaplayer_error("MAD_ERROR_BADCRC: %s", error_str(data->stream.error, data->str));
+				alsaplayer_error("MAD_ERROR_BADCRC: %s", error_str(data));
 			case MAD_ERROR_BADBIGVALUES:
 			case MAD_ERROR_BADDATAPTR:
 				break;
 			default:
-				alsaplayer_error("ERROR: %s", error_str(data->stream.error, data->str));
-				alsaplayer_error("No valid frame found at start (pos: %d, error: 0x%x --> %x %x %x %x) (%s)", data->offset, data->stream.error, 
+				alsaplayer_error("ERROR: %s", error_str(data));
+				alsaplayer_error("No valid block found at start (pos: %d, error: 0x%x --> %x %x %x %x) (%s)", data->offset, data->stream.error,
 						data->stream.this_frame[0],
 						data->stream.this_frame[1],
 						data->stream.this_frame[2],
@@ -1057,23 +1071,21 @@ first_frame:
 	}
 
 	mad_frame_decode(&data->frame, &data->stream);
-	/*	
-		alsaplayer_error("xing parsing...%x %x %x %x (%x %d)", 
+	/*
+		alsaplayer_error("xing parsing...%x %x %x %x (%x %d)",
 		data->stream.this_frame[0], data->stream.this_frame[1],
 		data->stream.this_frame[2], data->stream.this_frame[3],
 		data->stream.anc_ptr, data->stream.anc_bitlen);
-		*/		
+		*/
 	if (xing_parse(&data->xing, data->stream.anc_ptr, data->stream.anc_bitlen) == 0) {
 		// We use the xing data later on
 	}
 
-	mode = (data->frame.header.mode == MAD_MODE_SINGLE_CHANNEL) ?
-		1 : 2;
 	data->samplerate = data->frame.header.samplerate;
 	data->bitrate	= data->frame.header.bitrate;
 	mad_synth_frame (&data->synth, &data->frame);
 	{
-		struct mad_pcm *pcm = &data->synth.pcm;			
+		struct mad_pcm *pcm = &data->synth.pcm;
 
 		obj->nr_channels = pcm->channels;
 		//alsaplayer_error("nr_channels = %d", obj->nr_channels);
@@ -1082,9 +1094,9 @@ first_frame:
 	/* Calculate some values */
 	data->bytes_avail = data->stream.bufend - data->stream.next_frame;
 	{
-		int64_t time;
+		int64_t duration;
 		int64_t samples;
-		int64_t frames;
+		int64_t blocks;
 
 		long oldpos = reader_tell(data->mad_fd);
 		reader_seek(data->mad_fd, 0, SEEK_END);
@@ -1094,43 +1106,43 @@ first_frame:
 
 		reader_seek(data->mad_fd, oldpos, SEEK_SET);
 		if (data->bitrate)
-			time = (data->filesize * 8) / (data->bitrate);
+			duration = (data->filesize * 8) / (data->bitrate);
 		else
-			time = 0;
+			duration = 0;
 
 		samples = 32 * MAD_NSBSAMPLES(&data->frame.header);
 
-		obj->frame_size = (int) samples << 2; /* Assume 16-bit stereo */
-		frames = data->samplerate * (time+1) / samples;
-		if (data->xing.flags & XING_FRAMES) {
-			obj->nr_frames = data->xing.frames;
-		} else {	
-			obj->nr_frames = (int) frames;
-		}	
+		obj->block_size = (int) samples << 2; /* Assume 16-bit stereo */
+		blocks = data->samplerate * (duration+1) / samples;
+		if (data->xing.flags & XING_BLOCKS) {
+			obj->nr_blocks = data->xing.blocks;
+		} else {
+			obj->nr_blocks = (int) blocks;
+		}
 		obj->nr_tracks = 1;
 	}
-	/* Determine if nr_frames makes sense */
+	/* Determine if nr_blocks makes sense */
 	if (!(obj->flags & P_SEEK) && (obj->flags & P_STREAMBASED)) {
-		obj->nr_frames = -1;
-	}	
+		obj->nr_blocks = -1;
+	}
 
-	/* Allocate frame index */
-	if (!data->seekable  || obj->nr_frames > 1000000 ||
-			(data->frames = (ssize_t *)malloc((obj->nr_frames + FRAME_RESERVE) * sizeof(ssize_t))) == NULL) {
+	/* Allocate block index */
+	if (!data->seekable  || obj->nr_blocks > 1000000 ||
+			(data->blocks = malloc((obj->nr_blocks + BLOCK_RESERVE) * sizeof(ssize_t))) == NULL) {
 		data->seekable = 0; // Given really
 	}	else {
 		data->seekable = 1;
-		data->frames[0] = 0;
-	}	
+		data->blocks[0] = 0;
+	}
 	data->mad_init = 1;
 
 	p = strrchr(path, '/');
 	if (p) {
-		strcpy(data->filename, ++p);
+		ap_strlcpy(data->filename, ++p, sizeof (data->filename));
 	} else {
-		strcpy(data->filename, path);
+		ap_strlcpy(data->filename, path, sizeof (data->filename));
 	}
-	strcpy(data->path, path);
+	ap_strlcpy(data->path, path, sizeof (data->path));
 
 	data->parse_id3 = prefs_get_bool(ap_prefs, "mad", "parse_id3", 1);
 
@@ -1154,12 +1166,12 @@ static void mad_close(input_object *obj)
 			mad_stream_finish(&data->stream);
 			data->mad_init = 0;
 		}
-		if (data->frames) {
-			free(data->frames);
-		}				
+		if (data->blocks) {
+			free(data->blocks);
+		}
 		free(obj->local_data);
 		obj->local_data = NULL;
-	}				
+	}
 }
 
 
@@ -1186,11 +1198,12 @@ input_plugin *input_plugin_info (void)
 	mad_plugin.can_handle = mad_can_handle;
 	mad_plugin.open = mad_open;
 	mad_plugin.close = mad_close;
-	mad_plugin.play_frame = mad_play_frame;
-	mad_plugin.frame_seek = mad_frame_seek;
-	mad_plugin.frame_size = mad_frame_size;
-	mad_plugin.nr_frames = mad_nr_frames;
-	mad_plugin.frame_to_sec = mad_frame_to_sec;
+	mad_plugin.play_block = mad_play_block;
+	mad_plugin.block_seek = mad_frame_seek;
+	mad_plugin.block_size = mad_block_size;
+	mad_plugin.nr_blocks = mad_nr_blocks;
+	mad_plugin.frame_count = mad_frame_count;
+	mad_plugin.block_to_sec = mad_block_to_sec;
 	mad_plugin.sample_rate = mad_sample_rate;
 	mad_plugin.channels = mad_channels;
 	mad_plugin.stream_info = mad_stream_info;
