@@ -2,21 +2,21 @@
  * mad - MPEG audio decoder
  * Copyright (C) 2000-2001 Robert Leslie
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ *  This file is part of AlsaPlayer.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ *  AlsaPlayer is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  AlsaPlayer is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
- * $Id: xing.c 495 2002-05-02 23:07:56Z adnans $
  */
 
 # ifdef HAVE_CONFIG_H
@@ -50,11 +50,11 @@ int xing_parse(struct xing *xing, struct mad_bitptr ptr, unsigned int bitlen)
   xing->flags = mad_bit_read(&ptr, 32);
   bitlen -= 64;
 
-  if (xing->flags & XING_FRAMES) {
+  if (xing->flags & XING_BLOCKS) {
     if (bitlen < 32)
       goto fail;
 
-    xing->frames = mad_bit_read(&ptr, 32);
+    xing->blocks = mad_bit_read(&ptr, 32);
     bitlen -= 32;
   }
 
