@@ -1,30 +1,34 @@
 /*
  * mikmod_engine.c
  * Copyright (C) 1999 Paul N. Fisher <rao@gnu.org>
- * Copyright (C) 2002 Andy Lo A Foe <andy@alsaplayer.org>
+ * Copyright (C) 2002 Andy Lo A Foe <andy@loafoe.com>
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ *  This file is part of AlsaPlayer.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ *  AlsaPlayer is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 3 of the License, or
+ *  (at your option) any later version.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  AlsaPlayer is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
 #include <unistd.h>
 #include <string.h>
 #include <mikmod.h>
-#include "input_plugin.h"
 
-#define MIKMOD_FRAME_SIZE	4608
+#include "input_plugin.h"
+#include "ap_string.h"
+#include "ap_unused.h"
+
+#define MIKMOD_BLOCK_SIZE	4608
 
 extern MDRIVER drv_alsaplayer;
 
@@ -32,7 +36,7 @@ struct mikmod_local_data {
 	MODULE *mf;
 	char *fname;
 	SBYTE *audio_buffer;
-};	
+};
 
 /* Unfortunately libmikmod is *NOT* reentrant :-( */
 static pthread_mutex_t mikmod_mutex;
@@ -77,11 +81,11 @@ static float mikmod_can_handle (const char *name)
 		return 0.0;
 	ext++;
 
-	if (strstr(name, "MOD.") ||
-			strstr(name, "mod.") ||
-			strstr(name, "xm.") ||
-			strstr(name, "XM.")) {
-		return 1.0;	  
+	if (strstr(ext, "MOD") ||
+			strstr(ext, "mod") ||
+			strstr(ext, "xm") ||
+			strstr(ext, "XM")) {
+		return 1.0;
 	}
 	if (!strcasecmp (ext, "669") ||
 			!strcasecmp (ext, "amf") ||
@@ -100,7 +104,7 @@ static float mikmod_can_handle (const char *name)
 			!strcasecmp (ext, "uni") ||
 			!strcasecmp (ext, "xm")) {
 		return 1.0;
-	}	
+	}
 	return 0.0;
 }
 
@@ -114,7 +118,7 @@ static int mikmod_open (input_object *obj, const char *name)
 		/* printf("mikmod already in use :(\n"); */
 		obj->local_data = NULL;
 		return 0;
-	}		
+	}
 
 	if (!(mf = Player_Load ((char *)name, 255, 0))) {
 		printf ("error loading module: %s\n", name);
@@ -125,19 +129,19 @@ static int mikmod_open (input_object *obj, const char *name)
 	obj->local_data = malloc(sizeof(struct mikmod_local_data));
 
 	if (!obj->local_data) {
-		Player_Free(mf);	
+		Player_Free(mf);
 		pthread_mutex_unlock(&mikmod_mutex);
 		return 0;
 	}
 	data = (struct mikmod_local_data *)obj->local_data;
 
-	if (!(data->audio_buffer = (SBYTE *)malloc(MIKMOD_FRAME_SIZE))) {
+	if (!(data->audio_buffer = malloc(MIKMOD_BLOCK_SIZE))) {
 		Player_Free(mf);
 		free(obj->local_data);
 		obj->local_data = NULL;
 		pthread_mutex_unlock(&mikmod_mutex);
 		return 0;
-	}	
+	}
 
 	data->fname = strrchr(name, '/');
 	data->fname = (data->fname) ? data->fname + 1 : (char *)name;
@@ -164,14 +168,11 @@ static void mikmod_close (input_object *obj)
 }
 
 
-static int mikmod_play_frame (input_object *obj, char *buf)
+static int mikmod_play_block (input_object *obj, short *buf)
 {
 	struct mikmod_local_data *data =
 		(struct mikmod_local_data *)obj->local_data;
 
-	int length = 0;
-	/*  printf ("playing frame\n"); */
-
 	if (obj && obj->local_data == NULL) {
 		printf("HUUUUUUUUUUUUUHHH??????????????????\n");
 		return 0;
@@ -179,46 +180,51 @@ static int mikmod_play_frame (input_object *obj, char *buf)
 	if (!Player_Active ())
 		return 0;
 
-	length = VC_WriteBytes((SBYTE *) data->audio_buffer, MIKMOD_FRAME_SIZE);
+	VC_WriteBytes(data->audio_buffer, MIKMOD_BLOCK_SIZE);
 
 	if (buf)
-		memcpy(buf, data->audio_buffer, MIKMOD_FRAME_SIZE);
+		memcpy(buf, data->audio_buffer, MIKMOD_BLOCK_SIZE);
 	return 1;
 }
 
 
-static int mikmod_frame_seek (input_object *obj, int frame)
+static int mikmod_block_seek (input_object * UNUSED (obj), int UNUSED (block))
 {
 	/* printf ("unable to seek in modules!\n"); */
 	return 0;
 }
 
 
-static int mikmod_frame_size (input_object *obj)
+static int mikmod_block_size (input_object * UNUSED (obj))
 {
-	return MIKMOD_FRAME_SIZE;
+	return MIKMOD_BLOCK_SIZE / sizeof (short);
 }
 
 
-static int mikmod_nr_frames (input_object *obj)
+static int mikmod_nr_blocks (input_object * UNUSED (obj))
+{
+	return 0;
+}
+
+static int64_t mikmod_frame_count (input_object * UNUSED (obj))
+{
+	return -1;
+}
+
+
+static long mikmod_block_to_sec (input_object * UNUSED (obj), int UNUSED (block))
 {
 	return 0;
 }
 
 
-static long mikmod_frame_to_sec (input_object *obj, int frame)
-{
-	return 0;
-}
-
-
-static int mikmod_sample_rate (input_object *obj)
+static int mikmod_sample_rate (input_object * UNUSED (obj))
 {
 	return 44100;
 }
 
 
-static int mikmod_channels (input_object *obj)
+static int mikmod_channels (input_object * UNUSED (obj))
 {
 	return 2;
 }
@@ -232,43 +238,40 @@ static int mikmod_stream_info (input_object *obj, stream_info *info)
 		return 0;
 	data = (struct mikmod_local_data *)obj->local_data;
 
-	sprintf (info->stream_type, "%i channel %s", 
+	snprintf (info->stream_type, sizeof (info->stream_type), "%i channel %s",
 			data->mf->numchn, data->mf->modtype);
 
 	info->artist[0] = 0;
-	strcpy (info->status, "No time data");
-	strcpy (info->title, (data->mf->songname[0]) ? data->mf->songname : data->fname);
+	ap_strlcpy (info->status, "No time data", sizeof (info->status));
+	ap_strlcpy (info->title, (data->mf->songname[0]) ? data->mf->songname : data->fname, sizeof (info->title));
 
 	return 1;
 }
 
-
-input_plugin mikmod_plugin = {
-	INPUT_PLUGIN_VERSION,
-	0,
-	"MikMod player v1.0",
-	"Paul Fisher <rao@gnu.org>",
-	NULL,
-	mikmod_init,
-	mikmod_shutdown,
-	NULL,
-	mikmod_can_handle,
-	mikmod_open,
-	mikmod_close,
-	mikmod_play_frame,
-	mikmod_frame_seek,
-	mikmod_frame_size,
-	mikmod_nr_frames,
-	mikmod_frame_to_sec,
-	mikmod_sample_rate,
-	mikmod_channels,
-	mikmod_stream_info,
-	NULL,
-	NULL
-};
+static input_plugin mikmod_plugin;
 
 
 input_plugin *input_plugin_info (void)
 {
+	memset (&mikmod_plugin, 0, sizeof (mikmod_plugin));
+
+	mikmod_plugin.version = INPUT_PLUGIN_VERSION;
+	mikmod_plugin.name = "MikMod player v1.0";
+	mikmod_plugin.author = "Paul Fisher <rao@gnu.org>";
+	mikmod_plugin.init = mikmod_init;
+	mikmod_plugin.shutdown = mikmod_shutdown;
+	mikmod_plugin.can_handle = mikmod_can_handle;
+	mikmod_plugin.open = mikmod_open;
+	mikmod_plugin.close = mikmod_close;
+	mikmod_plugin.play_block = mikmod_play_block;
+	mikmod_plugin.block_seek = mikmod_block_seek;
+	mikmod_plugin.block_size = mikmod_block_size;
+	mikmod_plugin.nr_blocks = mikmod_nr_blocks;
+	mikmod_plugin.frame_count = mikmod_frame_count;
+	mikmod_plugin.block_to_sec = mikmod_block_to_sec;
+	mikmod_plugin.sample_rate = mikmod_sample_rate;
+	mikmod_plugin.channels = mikmod_channels;
+	mikmod_plugin.stream_info = mikmod_stream_info;
+
 	return &mikmod_plugin;
 }
