@@ -1,5 +1,5 @@
 /*  message.c - external control for alsaplayer
- *  Copyright (C) 2002 Andy Lo A Foe <andy@alsaplayer.org>
+ *  Copyright (C) 2002 Andy Lo A Foe <andy@loafoe.com>
  *
  *  This file is part of AlsaPlayer.
  *
@@ -16,11 +16,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
- *  $Id: message.c 1255 2007-07-08 16:08:22Z dominique_libre $
- *
- */ 
-#include "message.h"
-#include "control.h"
+ */
 #include <sys/un.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
@@ -32,12 +28,17 @@
 #include <string.h>
 #include <unistd.h>
 
+#include "message.h"
+#include "control.h"
+#include "ap_string.h"
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 
-int ap_connect_session (int session)
+static int
+ap_connect_session (int session)
 {
 	int socket_fd;
 	struct sockaddr_un saddr;
@@ -47,7 +48,7 @@ int ap_connect_session (int session)
 
 	if ((socket_fd = socket (AF_UNIX, SOCK_STREAM, 0)) != -1) {
 		saddr.sun_family = AF_UNIX;
-		sprintf (saddr.sun_path, "/tmp/alsaplayer_%s_%d", pwd == NULL ?
+		snprintf (saddr.sun_path, sizeof (saddr.sun_path), "/tmp/alsaplayer_%s_%d", pwd == NULL ?
 				"anonymous" : pwd->pw_name, session);
 		if (connect (socket_fd, (struct sockaddr *) &saddr, sizeof (saddr)) != -1) {
 			if(socket_fd<0)
@@ -77,30 +78,24 @@ ap_message_t *ap_message_new(void)
 ap_key_t *ap_key_new(const char *key_id)
 {
 	ap_key_t *key;
-	key = (ap_key_t *)malloc(sizeof(ap_key_t));
-	
+	key = (ap_key_t *)calloc(1, sizeof(ap_key_t));
+
 	if (key) {
-		memset(key, 0, sizeof(ap_key_t));
-		if (strlen(key_id) > KEYID_LEN) {
-			strncpy(key->key_id, key_id, KEYID_LEN-1);
-			key->key_id[KEYID_LEN] = 0;
-		} else {
-			strcpy(key->key_id, key_id);
-		}	
+		ap_strlcpy(key->key_id, key_id, sizeof (key->key_id));
 		return key;
-	}	
+	}
 	return NULL;
 }
 
-
-void ap_key_delete(ap_key_t *key)
+static void
+ap_key_delete(ap_key_t *key)
 {
 	if (key) {
 		if (key->data) {
 			free(key->data);
 		}
 		free(key);
-	}	
+	}
 }
 
 
@@ -108,25 +103,26 @@ void ap_message_delete(ap_message_t *msg)
 {
 	int c;
 	ap_key_t *last;
-	
+
 	if (msg) {
 		//printf("going to delete msg (%d keys)\n", msg->header.nr_keys);
 		for (c = 0, msg->current = msg->root; c < msg->header.nr_keys; c++) {
 			last = msg->current;
 			msg->current = msg->current->next;
 			ap_key_delete(last);
-		}	
+		}
 		free(msg);
 		//puts("delete msg successful");
-	}	
+	}
 }
 
 
-void ap_message_add_key(ap_message_t *msg, ap_key_t *key)
+static void
+ap_message_add_key(ap_message_t *msg, ap_key_t *key)
 {
 	if (!msg || !key)
 		return;
-	
+
 	if (msg->root == NULL) {
 		//printf("adding first key... (%s)\n", key->key_id);
 		msg->root = key;
@@ -149,31 +145,31 @@ ap_message_t *ap_message_receive(int fd)
 	ap_message_t *msg;
 
 	msg = ap_message_new();
-	
+
 	if (!msg)
 		return NULL;
-	
+
 	memset(msg, 0, sizeof(ap_message_t));
-	
+
 	if (read (fd, msg, sizeof (ap_message_t)) != sizeof (ap_message_t)) {
 		ap_message_delete(msg);
 		return NULL;
-	}	
-	
-	// Check version before parsing	
+	}
+
+	// Check version before parsing
 	if (msg->header.version != AP_CONTROL_VERSION) {
 		fprintf(stderr, "protocol version mismatch (client): %x != %x",
 			msg->header.version, AP_CONTROL_VERSION);
 		ap_message_delete(msg);
 		return NULL;
-	}	
+	}
 	// List variables are invalid for this address space so reset them
 	msg->root = msg->tail = msg->current = NULL;
-	
+
 	// Also reset nr_keys
 	keys_to_read = msg->header.nr_keys;
 	msg->header.nr_keys = 0;
-	
+
 	//printf("receiving message with %d keys (msg->root = %x)\n",
 	//		keys_to_read, msg->root);
 	for (c = 0; c < keys_to_read; c++) {
@@ -209,7 +205,7 @@ ap_message_t *ap_message_receive(int fd)
 }
 
 
-int ap_message_add_float(ap_message_t *msg, char *key, float val)
+int ap_message_add_float(ap_message_t *msg, const char *key, float val)
 {
 	ap_key_t *new_key;
 
@@ -220,13 +216,13 @@ int ap_message_add_float(ap_message_t *msg, char *key, float val)
 	*((float*)new_key->data) = val;
 	new_key->length = sizeof(float);
 
-	ap_message_add_key(msg, new_key);	
-	
+	ap_message_add_key(msg, new_key);
+
 	return 1;
 }
 
 
-int ap_message_add_int32(ap_message_t *msg, char *key, int32_t val)
+int ap_message_add_int32(ap_message_t *msg, const char *key, int32_t val)
 {
 	ap_key_t *new_key;
 
@@ -237,17 +233,18 @@ int ap_message_add_int32(ap_message_t *msg, char *key, int32_t val)
 	*((int32_t*)new_key->data) = val;
 	//printf("key value = %d (%d)\n", *((int32_t*)new_key->data), val);
 	new_key->length = sizeof(int32_t);
-	
-	ap_message_add_key(msg, new_key);	
-	
+
+	ap_message_add_key(msg, new_key);
+
 	return 1;
 }
 
 
-ap_key_t *ap_message_find_key(ap_message_t *msg, char *key, int32_t key_type)
+static ap_key_t *
+ap_message_find_key(ap_message_t *msg, const char *key, int32_t key_type)
 {
 	ap_key_t *current;
-	
+
 	if (msg) {
 		current = msg->root;
 		while (current) {
@@ -262,7 +259,7 @@ ap_key_t *ap_message_find_key(ap_message_t *msg, char *key, int32_t key_type)
 }
 
 
-float *ap_message_find_float(ap_message_t *msg, char *key_id)
+float *ap_message_find_float(ap_message_t *msg, const char *key_id)
 {
 	ap_key_t *key;
 
@@ -273,7 +270,7 @@ float *ap_message_find_float(ap_message_t *msg, char *key_id)
 }
 
 
-int32_t *ap_message_find_int32(ap_message_t *msg, char *key_id)
+int32_t *ap_message_find_int32(ap_message_t *msg, const char *key_id)
 {
 	ap_key_t *key;
 
@@ -284,7 +281,7 @@ int32_t *ap_message_find_int32(ap_message_t *msg, char *key_id)
 }
 
 
-char *ap_message_find_string(ap_message_t *msg, char *key_id)
+char *ap_message_find_string(ap_message_t *msg, const char *key_id)
 {
 	ap_key_t *key;
 
@@ -295,18 +292,18 @@ char *ap_message_find_string(ap_message_t *msg, char *key_id)
 }
 
 
-int ap_message_add_string(ap_message_t *msg, char *key_id, const char *val)
+int ap_message_add_string(ap_message_t *msg, const char *key_id, const char *val)
 {
 	ap_key_t *new_key;
 	char *str;
-	
+
 	new_key = ap_key_new(key_id);
 	new_key->key_type = KTYPE_STRING;
 	new_key->data = str = strdup(val);
 	new_key->length = strlen(val)+1; // Also copy Null pointer!
 
 	ap_message_add_key(msg, new_key);
-	
+
 	return 1;
 }
 
@@ -315,8 +312,8 @@ int ap_message_add_string(ap_message_t *msg, char *key_id, const char *val)
 int ap_message_send(int fd, ap_message_t *msg)
 {
 	int c;
-	ap_key_t *current;	
-	
+	ap_key_t *current;
+
 	if (!msg)
 		return 0;
 
@@ -329,7 +326,7 @@ int ap_message_send(int fd, ap_message_t *msg)
 		if (current == NULL) {
 			fprintf(stderr, "problem!\n");
 			continue;
-		}	
+		}
 		if (write(fd, current, sizeof(ap_key_t)) != sizeof(ap_key_t)) {
 			fprintf(stderr, "error writing key\n");
 			continue;
@@ -337,7 +334,7 @@ int ap_message_send(int fd, ap_message_t *msg)
 		if (write(fd, current->data, current->length) != current->length) {
 			fprintf(stderr, "error writing key data\n");
 			continue;
-		}	
+		}
 	}
 	//printf("successfully sent message with %d keys\n", msg->header.nr_keys);
 	return 1;
@@ -352,9 +349,9 @@ int ap_session_running(int session)
 
 	pwd = getpwuid(geteuid());
 
-	sprintf(path, "/tmp/alsaplayer_%s_%d", pwd == NULL ? "anonymous" :
+	snprintf(path, sizeof (path), "/tmp/alsaplayer_%s_%d", pwd == NULL ? "anonymous" :
 			pwd->pw_name, session);
-	if (stat(path, &statbuf) != 0) 
+	if (stat(path, &statbuf) != 0)
 		return 0;
 	if (S_ISSOCK(statbuf.st_mode)) {
 		if (ap_ping(session))
@@ -381,14 +378,14 @@ int ap_find_session(char *session_name, int *session)
 
 	pwd = getpwuid(geteuid());
 
-	sprintf(username, pwd == NULL ? "anonymous" : pwd->pw_name);
+	snprintf(username, sizeof (username), "%s", pwd == NULL ? "anonymous" : pwd->pw_name);
 
-	sprintf(test_path, "alsaplayer_%s_", username);
+	snprintf(test_path, sizeof (test_path), "alsaplayer_%s_", username);
 
 	if (dir) {
 		while ((entry = readdir(dir)) != NULL) {
 			if (strncmp(entry->d_name, test_path, strlen(test_path)) == 0) {
-				sprintf(tmp, "%s%%d", test_path);
+				snprintf(tmp, sizeof (tmp), "%s%%d", test_path);
 				if (sscanf(entry->d_name, tmp, &session_id) == 1) {
 					if (ap_session_running(session_id) == 1) {
 						if (ap_get_session_name(session_id, remote_name)) {
@@ -396,11 +393,11 @@ int ap_find_session(char *session_name, int *session)
 								*session = session_id;
 								closedir(dir);
 								return 1;
-							} 
-						}	
+							}
+						}
 					}
 				}
-			}	
+			}
 		}
 		closedir(dir);
 	}
@@ -422,7 +419,7 @@ int ap_set_speed(int session, float speed)
 	int fd;
 	ap_message_t *msg, *reply;
 	int32_t *result;
-	
+
 	fd = ap_connect_session(session);
 	if (fd < 0)
 		return 0;
@@ -432,10 +429,10 @@ int ap_set_speed(int session, float speed)
 	ap_message_send(fd, msg);
 	ap_message_delete(msg);
 	msg = NULL;
-	
+
 	reply = ap_message_receive(fd);
 	close(fd);
-		
+
 	if ((result = ap_message_find_int32(reply, "ack"))) {
 		ap_message_delete(reply);
 		return 1;
@@ -450,7 +447,7 @@ int ap_get_speed(int session, float *val)
 	int fd;
 	ap_message_t *msg, *reply;
 	float *result;
-	
+
 	fd = ap_connect_session(session);
 	if (fd < 0)
 		return 0;
@@ -462,7 +459,7 @@ int ap_get_speed(int session, float *val)
 
 	reply = ap_message_receive(fd);
 	close(fd);
-	
+
 	if ((result = ap_message_find_float(reply, "speed"))) {
 		*val = *result;
 		ap_message_delete(reply);
@@ -474,12 +471,13 @@ int ap_get_speed(int session, float *val)
 
 
 /* Convenience function for commands that take a single int */
-int ap_cmd_set_int(int session, int32_t cmd, int val)
+static int
+ap_cmd_set_int(int session, int32_t cmd, int val)
 {
 	int fd;
 	ap_message_t *msg, *reply;
 	int32_t *result;
-	
+
 	fd = ap_connect_session(session);
 	if (fd < 0)
 		return 0;
@@ -504,12 +502,13 @@ int ap_cmd_set_int(int session, int32_t cmd, int val)
 
 
 /* Convenience function for commands that take a single float */
-int ap_cmd_set_float(int session, int32_t cmd, float val)
+static int
+ap_cmd_set_float(int session, int32_t cmd, float val)
 {
 	int fd;
 	ap_message_t *msg, *reply;
 	int32_t *result;
-	
+
 	fd = ap_connect_session(session);
 	if (fd < 0)
 		return 0;
@@ -540,12 +539,13 @@ int ap_cmd_set_float(int session, int32_t cmd, float val)
  * @param float *val Variable where the result will be copied to
  * @return true if command succeeded, false otherwise
  */
-int ap_cmd_get_float(int session, int32_t cmd, float *val)
+static int
+ap_cmd_get_float(int session, int32_t cmd, float *val)
 {
 	int fd;
 	ap_message_t *msg, *reply;
 	float *res;
-	
+
 	fd = ap_connect_session(session);
 	if (fd < 0)
 		return 0;
@@ -577,12 +577,13 @@ int ap_cmd_get_float(int session, int32_t cmd, float *val)
  * @param int *val Variable where the result will be copied to
  * @return true if command succeeded, false otherwise
  */
-int ap_cmd_get_int(int session, int32_t cmd, int *val)
+static int
+ap_cmd_get_int(int session, int32_t cmd, int *val)
 {
 	int fd;
 	ap_message_t *msg, *reply;
 	int32_t *res;
-	
+
 	fd = ap_connect_session(session);
 	if (fd < 0)
 		return 0;
@@ -619,13 +620,23 @@ int ap_is_looping(int session, int *looping)
 int ap_set_playlist_looping(int session, int looping)
 {
 	return (ap_cmd_set_int(session, AP_SET_PLAYLIST_LOOPING, looping));
-} 
+}
 
 int ap_is_playlist_looping(int session, int *looping)
 {
 	return (ap_cmd_get_int(session, AP_IS_PLAYLIST_LOOPING, looping));
 }
 
+int ap_set_onebyone(int session, int onebyone)
+{
+	return (ap_cmd_set_int(session, AP_SET_ONEBYONE, onebyone));
+}
+
+int ap_is_onebyone(int session, int *onebyone)
+{
+        return (ap_cmd_get_int(session, AP_IS_ONEBYONE, onebyone));
+}
+
 int ap_is_paused(int session , int *paused)
 {
 	 return (ap_cmd_get_int(session, AP_IS_PAUSED, paused));
@@ -682,9 +693,9 @@ int ap_get_playlist_length(int session, int *length)
 	return (ap_cmd_get_int(session, AP_GET_PLAYLIST_LENGTH, length));
 }
 
-int ap_set_frame(int session, int frame)
+int ap_set_block(int session, int block)
 {
-	return (ap_cmd_set_int(session, AP_SET_POS_FRAME, frame));
+	return (ap_cmd_set_int(session, AP_SET_POS_BLOCK, block));
 }
 
 int ap_jump_to(int session, int pos)
@@ -692,14 +703,14 @@ int ap_jump_to(int session, int pos)
 	return (ap_cmd_set_int(session, AP_JUMP_TO, pos));
 }
 
-int ap_get_frame(int session, int *val)
+int ap_get_block(int session, int *val)
 {
-	return (ap_cmd_get_int(session, AP_GET_POS_FRAME, val));
+	return (ap_cmd_get_int(session, AP_GET_POS_BLOCK, val));
 }
 
-int ap_get_frames(int session, int *val)
+int ap_get_blocks(int session, int *val)
 {
-	return (ap_cmd_get_int(session, AP_GET_SONG_LENGTH_FRAME, val));
+	return (ap_cmd_get_int(session, AP_GET_SONG_LENGTH_BLOCK, val));
 }
 
 int ap_is_playing(int session, int *val)
@@ -708,7 +719,8 @@ int ap_is_playing(int session, int *val)
 }
 
 /* Convenience function for commands that return a single string */
-int ap_get_single_string_command(int session, int32_t cmd, char *str, int maxlen)
+static int
+ap_get_single_string_command(int session, int32_t cmd, char *str, int maxlen)
 {
 	int fd;
 	ap_message_t *msg, *reply;
@@ -719,26 +731,22 @@ int ap_get_single_string_command(int session, int32_t cmd, char *str, int maxlen
 
 	str[0] = 0; // Make sure the string is always NULL terminated
 	            // Even if we fail to get the data
-	
+
 	fd = ap_connect_session(session);
 	if (fd < 0)
 		return 0;
 	msg = ap_message_new();
 	msg->header.cmd = cmd;
-	
+
 	ap_message_send(fd, msg);
 	ap_message_delete(msg);
 	msg = NULL;
 
 	reply = ap_message_receive(fd);
 	close(fd);
-	
+
 	if ((result = ap_message_find_string(reply, "string"))) {
-		if (strlen(result) > (size_t)maxlen) {
-			strncpy(str, result, maxlen-1);
-			str[maxlen] = 0;
-		} else 	
-			strcpy(str, result);
+		ap_strlcpy(str, result, maxlen);
 		ap_message_delete(reply);
 		return 1;
 	}
@@ -749,7 +757,7 @@ int ap_get_single_string_command(int session, int32_t cmd, char *str, int maxlen
 
 int ap_get_session_name(int session, char *str)
 {
-	return (ap_get_single_string_command(session, 
+	return (ap_get_single_string_command(session,
 				AP_GET_SESSION_NAME, str, AP_SESSION_MAX));
 }
 
@@ -767,7 +775,7 @@ int ap_get_file_path(int session, char *str)
 
 int ap_get_artist(int session, char *str)
 {
-	return (ap_get_single_string_command(session, 
+	return (ap_get_single_string_command(session,
 				AP_GET_ARTIST, str, AP_ARTIST_MAX));
 }
 
@@ -785,19 +793,19 @@ int ap_get_comment(int session, char *str)
 
 int ap_get_album(int session, char *str)
 {
-	return (ap_get_single_string_command(session, 
+	return (ap_get_single_string_command(session,
 				AP_GET_ALBUM, str, AP_ALBUM_MAX));
 }
 
 int ap_get_year(int session, char *str)
 {
-	return (ap_get_single_string_command(session, 
+	return (ap_get_single_string_command(session,
 				AP_GET_YEAR, str, AP_YEAR_MAX));
 }
 
 int ap_get_track_number(int session, char *str)
 {
-	return (ap_get_single_string_command(session, 
+	return (ap_get_single_string_command(session,
 				AP_GET_TRACK_NUMBER, str, AP_TRACK_NUMBER_MAX));
 }
 
@@ -817,7 +825,6 @@ int ap_ping(int session)
 {
 	int fd;
 	int32_t *pong;
-	int32_t ret_val;
 	ap_message_t *msg, *reply;
 
 	fd = ap_connect_session(session);
@@ -828,12 +835,11 @@ int ap_ping(int session)
 	ap_message_send(fd, msg);
 	ap_message_delete(msg);
 	msg = NULL;
-	
+
 	reply = ap_message_receive(fd);
 	close(fd);
-		
+
 	if ((pong = ap_message_find_int32(reply, "pong"))) {
-		ret_val = *pong;
 		ap_message_delete(reply);
 		// ret_val not used
 		return 1;
@@ -845,7 +851,7 @@ int ap_ping(int session)
 int ap_add_and_play(int session, const char *path)
 {
 	int fd;
-	int32_t *result, ret_val;
+	int32_t *result;
 	ap_message_t *msg, *reply;
 
 	fd = ap_connect_session(session);
@@ -857,12 +863,11 @@ int ap_add_and_play(int session, const char *path)
 	ap_message_send(fd, msg);
 	ap_message_delete(msg);
 	msg = NULL;
-	
+
 	reply = ap_message_receive(fd);
 	close(fd);
-	
+
 	if ((result = ap_message_find_int32(reply, "ack"))) {
-		ret_val = *result;
 		ap_message_delete(reply);
 		return 1;
 	}
@@ -875,7 +880,7 @@ int ap_add_and_play(int session, const char *path)
 int ap_add_path(int session, const char *path)
 {
 	int fd;
-	int32_t *result, ret_val;
+	int32_t *result;
 	ap_message_t *msg, *reply;
 
 	fd = ap_connect_session(session);
@@ -887,12 +892,11 @@ int ap_add_path(int session, const char *path)
 	ap_message_send(fd, msg);
 	ap_message_delete(msg);
 	msg = NULL;
-	
+
 	reply = ap_message_receive(fd);
 	close(fd);
-	
+
 	if ((result = ap_message_find_int32(reply, "ack"))) {
-		ret_val = *result;
 		ap_message_delete(reply);
 		return 1;
 	}
@@ -905,7 +909,7 @@ int ap_add_path(int session, const char *path)
 int ap_add_playlist(int session, const char *playlistfile)
 {
 	int fd;
-	int32_t *result, ret_val;
+	int32_t *result;
 	ap_message_t *msg, *reply;
 
 	fd = ap_connect_session(session);
@@ -922,7 +926,6 @@ int ap_add_playlist(int session, const char *playlistfile)
 	close(fd);
 
 	if ((result = ap_message_find_int32(reply, "ack"))) {
-		ret_val = *result;
 		ap_message_delete(reply);
 		return 1;
 	}
@@ -935,7 +938,7 @@ int ap_add_playlist(int session, const char *playlistfile)
 int ap_sort (int session, char *seq)
 {
 	int fd;
-	int32_t *result, ret_val;
+	int32_t *result;
 	ap_message_t *msg, *reply;
 
 	fd = ap_connect_session(session);
@@ -947,12 +950,11 @@ int ap_sort (int session, char *seq)
 	ap_message_send(fd, msg);
 	ap_message_delete(msg);
 	msg = NULL;
-	
+
 	reply = ap_message_receive(fd);
 	close(fd);
-	
+
 	if ((result = ap_message_find_int32(reply, "ack"))) {
-		ret_val = *result;
 		ap_message_delete(reply);
 		return 1;
 	}
@@ -967,22 +969,22 @@ static int ap_do_command_only(int session, int32_t cmd)
 {
 	int fd;
 	int32_t *result, ret_val;
-	
+
 	ap_message_t *msg, *reply;
 
 	fd = ap_connect_session(session);
 	if (fd < 0)
 		return 0;
-	
+
 	msg = ap_message_new();
 	msg->header.cmd = cmd;
 	ap_message_send(fd, msg);
 	ap_message_delete(msg);
 	msg = NULL;
-	
+
 	reply = ap_message_receive(fd);
-	close(fd);	
-	
+	close(fd);
+
 	if ((result = ap_message_find_int32(reply, "ack"))) {
 		ret_val = *result;
 		ap_message_delete(reply);
@@ -1047,7 +1049,8 @@ int ap_get_playlist_position(int session, int *pos)
 	return (ap_cmd_get_int(session, AP_GET_PLAYLIST_POSITION, pos));
 }
 
-int ap_get_string_set_int(int session, int32_t cmd, char* str, int maxlen, int val)
+static int
+ap_get_string_set_int(int session, int32_t cmd, char* str, int maxlen, int val)
 {
 	int fd, *ack;
 	ap_message_t *msg, *reply;
@@ -1070,11 +1073,7 @@ int ap_get_string_set_int(int session, int32_t cmd, char* str, int maxlen, int v
 	ack = ap_message_find_int32(reply, "ack");
 	if(*ack == 1) {
 	  if ((result = ap_message_find_string(reply, "string"))) {
-	    if (strlen(result) > (size_t)maxlen) {
-	      strncpy(str, result, maxlen-1);
-	      str[maxlen] = 0;
-	    } else 	
-	      strcpy(str, result);
+	    ap_strlcpy(str, result, maxlen);
 	    ap_message_delete(reply);
 	    return 1;
 	  }
@@ -1088,7 +1087,8 @@ int ap_get_file_path_for_track(int session, char* path, int track)
         return (ap_get_string_set_int(session, AP_GET_FILE_PATH_FOR_TRACK, path, AP_FILE_PATH_MAX, track));
 }
 
-int ap_set_string_set_int(int session, int32_t cmd, const char* string, int val)
+static int
+ap_set_string_set_int(int session, int32_t cmd, const char* string, int val)
 {
 	int fd;
 	int32_t *result;
@@ -1117,29 +1117,29 @@ int ap_set_string_set_int(int session, int32_t cmd, const char* string, int val)
 	}
 	ap_message_delete(reply);
 	return 0;
-}        
+}
 
 int ap_get_playlist(int session, int *argc, char ***the_list) {
 	int fd;
 	char **list;
 	ap_message_t *msg, *reply;
-	int *result, nritems, c, t;
+	int *result, nritems, c;
 	char strnum[64];
 	char *res;
-	
+
 	fd = ap_connect_session(session);
 	if (fd < 0)
 		return 0;
 	msg = ap_message_new();
 	msg->header.cmd = AP_GET_PLAYLIST;
-	
+
 	ap_message_send(fd, msg);
 	ap_message_delete(msg);
 	msg = NULL;
 
 	reply = ap_message_receive(fd);
 	close(fd);
-	
+
 	if ((result = ap_message_find_int32(reply, "items"))) {
 		nritems = *result;
 		if (nritems > 0) {
@@ -1148,14 +1148,14 @@ int ap_get_playlist(int session, int *argc, char ***the_list) {
 			*argc = 0;
 			ap_message_delete(reply);
 			return 1;
-		}	
+		}
 		list = (char **)malloc(sizeof(char *) * nritems);
 		if (!list) {
 			ap_message_delete(reply);
 			return 0;
-		}	
+		}
 		for (c=0; c < nritems; c++) {
-			sprintf(strnum, "%d", c+1);
+			snprintf(strnum, sizeof (strnum), "%d", c+1);
 			res = ap_message_find_string(reply, strnum);
 			if (res) {
 				list[c] = strdup(res);
