@@ -1,5 +1,5 @@
 /*  amessage.h
- *  Copyright (C) 2002 Andy Lo A Foe <andy@alsaplayer.org>
+ *  Copyright (C) 2002 Andy Lo A Foe <andy@loafoe.com>
  *
  *  This file is part of AlsaPlayer.
  *
@@ -16,7 +16,7 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
-*/ 
+*/
 #ifndef __AMESSAGE_H__
 #define __AMESSAGE_H__
 
@@ -55,11 +55,11 @@
 #define AP_GET_YEAR             0xcc14
 #define AP_ADD_PATH             0xcc15
 #define AP_GET_SONG_LENGTH_SECOND	0xcc16
-#define AP_GET_SONG_LENGTH_FRAME	0xcc17
+#define AP_GET_SONG_LENGTH_BLOCK	0xcc17
 #define AP_SET_POS_SECOND               0xcc18
 #define AP_GET_POS_SECOND               0xcc19
-#define AP_SET_POS_FRAME                0xcc1a
-#define AP_GET_POS_FRAME                0xcc1b
+#define AP_SET_POS_BLOCK                0xcc1a
+#define AP_GET_POS_BLOCK                0xcc1b
 #define AP_GET_STREAM_TYPE              0xcc1c
 #define AP_GET_STATUS			0xcc1d
 #define AP_SET_POS_SECOND_RELATIVE	0xcc1e
@@ -89,18 +89,21 @@
 #define AP_SET_CURRENT          0xcc35
 
 #define AP_GET_PLAYLIST		0xcc36
+
+#define AP_SET_ONEBYONE         0xcc37
+#define AP_IS_ONEBYONE          0xcc38
 /* Add new commands here */
 
 #ifdef __cplusplus
 extern "C" {
-#endif	
+#endif
 
 struct _ap_hdr {
 	int32_t version;
 	int32_t cmd;
 	int32_t result;
 	int32_t nr_keys;
-};	
+};
 
 struct _ap_key {
 	int32_t length;
@@ -108,7 +111,7 @@ struct _ap_key {
 	char key_id[KEYID_LEN];
 	void *data;
 	struct _ap_key *next;
-};	
+};
 
 typedef struct _ap_hdr ap_hdr_t;
 typedef struct _ap_key ap_key_t;
@@ -127,21 +130,21 @@ typedef struct _ap_message ap_message_t;
 
 ap_message_t *ap_message_new(void);
 ap_key_t *ap_key_new(const char *keyid);
-int ap_message_add_float(ap_message_t *, char *, float);
-int ap_message_add_int32(ap_message_t *, char *, int32_t);
-int ap_message_add_string(ap_message_t *, char *, const char *);
+int ap_message_add_float(ap_message_t *, const char *, float);
+int ap_message_add_int32(ap_message_t *, const char *, int32_t);
+int ap_message_add_string(ap_message_t *, const char *, const char *);
 void ap_message_delete(ap_message_t *);
 
 int ap_message_send(int fd, ap_message_t *);
 ap_message_t *ap_message_receive(int fd);
 
-float *ap_message_find_float(ap_message_t *, char *);
-int32_t *ap_message_find_int32(ap_message_t *, char *);
-char *ap_message_find_string(ap_message_t *, char *);
+float *ap_message_find_float(ap_message_t *, const char *);
+int32_t *ap_message_find_int32(ap_message_t *, const char *);
+char *ap_message_find_string(ap_message_t *, const char *);
 
 #ifdef __cplusplus
 }
-#endif	
+#endif
 
 
 
