@@ -1,29 +1,34 @@
-/*  null.c - NULL output plugin, useful, trust me :) 
+/*  null.c - NULL output plugin, useful, trust me :)
  *  Copyright (C) 1999 Andy Lo A Foe <andy@alsa-project.org>
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This file is part of AlsaPlayer.
+ *
+ *  AlsaPlayer is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation; either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  AlsaPlayer is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-*/ 
+ *  along with this program; if not, see <http://www.gnu.org/licenses/>.
+ *
+*/
 
 #include "config.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
+
 #include "output_plugin.h"
 #include "alsaplayer_error.h"
 #include "utilities.h"
+#include "ap_unused.h"
 
 static int null_init(void)
 {
@@ -32,7 +37,7 @@ static int null_init(void)
 	return 1;
 }
 
-static int null_open(const char *name)
+static int null_open(const char * UNUSED (name))
 {
 	return 1;
 }
@@ -44,14 +49,14 @@ static void null_close(void)
 }
 
 
-static int null_write(void *data, int count)
+static int null_write(short * UNUSED (data), int UNUSED (count))
 {
 	dosleep(10000);
 	return 1;
 }
 
 
-static int null_set_buffer(int *fragment_size, int *fragment_count, int *channels)
+static int null_set_buffer(int * UNUSED (fragment_size), int * UNUSED (fragment_count), int * UNUSED (channels))
 {
 	return 1;
 }
