@@ -1,20 +1,22 @@
 /*  oss.c - Output driver for OSS/Lite and OSS/Linux
- *  Copyright (C) 1999-2002 Andy Lo A Foe <andy@alsaplayer.org>
+ *  Copyright (C) 1999-2002 Andy Lo A Foe <andy@loafoe.com>
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This file is part of AlsaPlayer.
+ *
+ *  AlsaPlayer is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation; either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  AlsaPlayer is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-*/ 
+ *  along with this program; if not, see <http://www.gnu.org/licenses/>.
+ *
+*/
 
 #include "config.h"
 #include <stdio.h>
@@ -63,7 +65,7 @@ static void oss_close(void)
 }
 
 
-static int oss_write(void *data, int count)
+static int oss_write(short *data, int count)
 {
 	write(oss_fd, data, count);
 	return 1;
@@ -76,12 +78,12 @@ static int oss_set_buffer(int *fragment_size, int *fragment_count, int *channels
 	int hops;
 	int size;
 	int count;
-	
+
 	size = *fragment_size;
 	count = *fragment_count;
-	
+
 	for (hops=0; size >>=1; hops++);
-	
+
 	val = (count << 16) + hops;
 	ioctl(oss_fd,SNDCTL_DSP_SETFRAGMENT,&val);
 	val = AFMT_S16_NE;
@@ -98,7 +100,7 @@ static unsigned int oss_set_sample_rate(unsigned int rate)
 	if (ioctl(oss_fd,SNDCTL_DSP_SPEED,&rate) < 0) {
 					alsaplayer_error("error setting sample_rate");
 					return 0;
-	}				
+	}
 	return rate;
 }
 
