@@ -1,20 +1,26 @@
 /*  sgi.c - SGI Audio Library output module
  *  Copyright (C) 1999 Michael Pruett <michael@68k.org>
  *
- *  This program is free software; you can redistribute it and/or modify
+ *  This file is part of AlsaPlayer.
+ *
+ *  AlsaPlayer is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
+ *  the Free Software Foundation; either version 3 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  AlsaPlayer is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
-*/ 
+ *  along with this program; if not, see <http://www.gnu.org/licenses/>.
+ *
+ * As an exception to the GPL-v3 licence, you are authorised to compile,
+ * link and distribute AlsaPlayer on a SGI system. See the COPYING file
+ * in the main AlsaPlayer directory.
+ *
+*/
 
 #include "config.h"
 #include <stdio.h>
@@ -38,7 +44,7 @@ static int sgi_init ()
 
 static int sgi_open (const char *device)
 {
-	if (port != NULL) 
+	if (port != NULL)
 		return 1;
 	return 0;
 }
@@ -51,7 +57,7 @@ static void sgi_close ()
 static int sgi_write (void *data, int count)
 {
 	ALconfig	config;
-	int			frameSize, channelCount;
+	int			blockSize, channelCount;
 
 	config = alGetConfig(port);
 	if (config == NULL)
@@ -61,15 +67,15 @@ static int sgi_write (void *data, int count)
 	}
 
 	channelCount = alGetChannels(config);
-	frameSize = alGetWidth(config);
+	blockSize = alGetWidth(config);
 
-	if (frameSize == 0 || channelCount == 0)
+	if (blockSize == 0 || channelCount == 0)
 	{
 		printf("SGI: bad configuration.\n");
 		return 0;
 	}
 
-	alWriteFrames(port, data, count / (frameSize * channelCount));
+	alWriteBlocks(port, data, count / (blockSize * channelCount));
 	return 1;
 }
 
