@@ -1,6 +1,6 @@
 /*   http.c
  *   Copyright (C) 2002 Evgeny Chukreev <codedj@echo.ru>
- *   Copyright (C) 2002 Andy Lo A Foe <andy@alsaplayer.org>
+ *   Copyright (C) 2002 Andy Lo A Foe <andy@loafoe.com>
  *
  *  This file is part of AlsaPlayer.
  *
@@ -17,13 +17,11 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
- *   $Id: http.c 1258 2007-07-08 16:38:45Z dominique_libre $
- *
  */
 
 #include "config.h"
 
-#include <malloc.h>
+#include <stdlib.h>
 #include <netdb.h>
 #include <stdlib.h>
 #include <string.h>
@@ -40,7 +38,8 @@
 #include "alsaplayer_error.h"
 #include "utilities.h"
 #include "prefs.h"
-
+#include "ap_string.h"
+#include "ap_unused.h"
 
 typedef struct http_desc_t_ {
     /* Info about stream */
@@ -108,10 +107,10 @@ static int cond_timedwait_relative (pthread_cond_t *cond, pthread_mutex_t *mut,
     struct timespec timeout;
 
     gettimeofday (&now, NULL);
-	    
+
     timeout.tv_sec = now.tv_sec + delay / 1000000;
     timeout.tv_nsec = (now.tv_usec + delay % 1000000) * 1000;
-	    
+
     return pthread_cond_timedwait (cond, mut, &timeout);
 }
 
@@ -142,20 +141,21 @@ static int calc_time_to_wait (http_desc_t *desc)
 }
 
 /* ------------------------------------------------ */
-void shrink_buffer (http_desc_t *desc)
+static void
+shrink_buffer (http_desc_t *desc)
 {
     int low_pos = desc->begin + ((long int)http_buffer_size * (long int)desc->played_parts) / 16;
-    
+
     /* trying to shrink buffer */
     if (desc->len + HTTP_BLOCK_SIZE > http_buffer_size && desc->pos > low_pos) {
 	void *newbuf;
 	int shrinking_len = desc->pos - low_pos;
-     
+
 	desc->len -= shrinking_len;
 	desc->begin += shrinking_len;
-	
+
 	/* allocate new buffer with the part of old one */
-	newbuf = malloc (desc->len);    
+	newbuf = malloc (desc->len);
 	memcpy (newbuf, desc->buffer + shrinking_len, desc->len);
 
 	/* replace old buffer */
@@ -165,14 +165,15 @@ void shrink_buffer (http_desc_t *desc)
 }
 
 /* ------------------------------------------------ */
-void status_notify (http_desc_t *desc)
+static void
+status_notify (http_desc_t *desc)
 {
     char sbuf[1024];
 
     if (!desc->dont_wait && desc->status) {
 	int bck_len = desc->pos - desc->begin;
 	int fwd_len = desc->len - bck_len;
-	
+
 	if (desc->seekable)
 	    snprintf (sbuf, 1023, "Buf %dK | %dK", fwd_len/1024, bck_len/1024);
 	else
@@ -189,7 +190,7 @@ static void print_debug_info (http_desc_t *desc)
     struct timeval now;
 
     gettimeofday (&now, NULL);
-    
+
     printf ("%lld %ld %d %d %ld %d\n", (long long int)now.tv_sec * 1000 + (long long int)now.tv_usec/1000,
 	    desc->size, desc->begin, desc->begin + desc->len,
 	    desc->pos, desc->buffer_pos);
@@ -201,23 +202,23 @@ static void print_debug_info (http_desc_t *desc)
 
 /* ******************************************************************* */
 /* Parse URI.                                                           */
-static int parse_uri (const char *uri, char **host, int *port, char **path) 
+static int parse_uri (const char *uri, char **host, int *port, char **path)
 {
     char *slash, *colon;
     int l;
 
     *port = 80;
-    
+
      /* Trying to find end of a host part */
     slash = strchr (uri+7, '/');
     colon = strchr (uri+7, ':');
-    
+
     if ((slash && colon && slash > colon) || (!slash && colon)) {
 	/* As I see, there is port specified */
 	char *s;
-	
+
 	*port = (int)strtol (colon+1, &s, 10);
-	
+
 	/* Test, port should be digit */
 	if ((slash && s!=slash) || (!slash && *s!='\0')) {
 	    alsaplayer_error ("\nHTTP: Couldn't open %s: Port -- parse error.", uri);
@@ -228,19 +229,18 @@ static int parse_uri (const char *uri, char **host, int *port, char **path)
 	l = colon - uri - 7;
     } else {
 	/* Calculate host part length */
-	l = slash  ?  slash - uri - 7  :  strlen (uri+7);
+	l = slash  ?  slash - uri - 7  :  (int) strlen (uri+7);
     }
-   
+
     /* Reset port if URI looks like 'foo.bar:/aaa.mp3' */
     if (colon && slash && slash==colon+1)
 	*port = 80;
-    
+
     /* Split URI */
-    //if (*host) 
+    //if (*host)
     //	    free(*host);
-    *host = malloc ((l+1) * sizeof(char));
-    strncpy (*host, uri+7, l);
-    (*host) [l] = '\0';
+    *host = malloc (l+1);
+    ap_strlcpy (*host, uri+7, l+1);
 
     //if (*path)
     //	    free(*path);
@@ -254,12 +254,12 @@ static int sleep_for_data (int sock)
 {
     fd_set set;
     struct timeval tv;
-    
+
     tv.tv_sec = 5;
     tv.tv_usec = 0;
     FD_ZERO (&set);
     FD_SET (sock, &set);
-    
+
     if (select (sock+1, &set, NULL, NULL, &tv) < 1) {
 	alsaplayer_error ("HTTP: Connection is too slow.");
 	return 1;
@@ -283,17 +283,17 @@ static int get_response_head (int sock, char *response, int max)
 
 	/* wait for data */
 	if (sleep_for_data (sock))  return 1;
-	
+
 	/* read */
 	if (read (sock, response + len, 1) <=0 )
 	    break;
-	
+
 	len += 1;
     }
-    
+
     /* terminate string */
     response [len] = '\0';
-    
+
     return 0;
 } /* end of: get_response_head */
 
@@ -303,16 +303,16 @@ static int get_response_head (int sock, char *response, int max)
 static int read_data (int sock, void *ptr, size_t size)
 {
     int len;
-    
+
     /* wait for data */
     if (sleep_for_data (sock)) {
 	    return -1;
     }
     len = recv (sock, ptr, size, 0);
-    
+
     if (len == -1 && errno == EAGAIN) {
 	return 0;
-    }	
+    }
     return len;
 } /* end of: read_data */
 
@@ -327,16 +327,16 @@ static void buffer_thread (http_desc_t *desc)
     int rest = 0;
     int metasize = 0, metapos = 0, extra_read = 0;
     char *p;
-       
+
     /* Init */
     pthread_mutex_init (&mut, NULL);
-    
+
     if (desc->icy_metaint) {
 	BLOCK_SIZE = (HTTP_BLOCK_SIZE > desc->icy_metaint) ? desc->icy_metaint : HTTP_BLOCK_SIZE;
     }
 
     ibuffer = malloc (BLOCK_SIZE << 1);
-    
+
     /* Process while main thread allow it. */
     while (desc->going) {
 	void *newbuf;
@@ -347,7 +347,7 @@ static void buffer_thread (http_desc_t *desc)
 #endif
 
 	rest = metasize = 0;
-	
+
 	/* trying to shrink buffer */
 	pthread_mutex_lock (&desc->buffer_lock);
 	shrink_buffer (desc);
@@ -361,7 +361,7 @@ static void buffer_thread (http_desc_t *desc)
 	    /* Notice waiting function that the new block of data has arrived */
 	    desc->new_datablock = 1;
 	    pthread_cond_signal (&desc->new_datablock_signal);
-    
+
 	    /* Make pause */
 	    if (!desc->dont_wait) {
 		pthread_mutex_lock (&mut);
@@ -373,7 +373,7 @@ static void buffer_thread (http_desc_t *desc)
 
 	    continue;
 	}
-	
+
 	/* read to internal buffer */
         readed = read_data (desc->sock, ibuffer, BLOCK_SIZE);
 
@@ -385,13 +385,13 @@ static void buffer_thread (http_desc_t *desc)
 	    desc->going = 0;
 	} else {
 	    /* Something readed */
-	    
+
 	    /* Metadata stuff */
-	    if (desc->icy_metaint > 0 && 
+	    if (desc->icy_metaint > 0 &&
 		(desc->buffer_pos+readed) >  desc->icy_metaint) {
 		/* Metadata block is next! */
 		rest = (desc->buffer_pos+readed) - desc->icy_metaint;
-	
+
 		p = ((char *)ibuffer);
 		p += (readed-rest);
 		metapos = (readed-rest);
@@ -403,8 +403,8 @@ static void buffer_thread (http_desc_t *desc)
 			extra_read = read_data (desc->sock, ibuffer+readed, metasize);
 			readed += extra_read;
 			rest += extra_read;
-		    }	
-		    if (metasize > 4080) { 
+		    }
+		    if (metasize > 4080) {
 			alsaplayer_error("Invalid metasize (%d)", metasize);
 		    } else if (metasize > 0) {
 			p++;
@@ -412,8 +412,8 @@ static void buffer_thread (http_desc_t *desc)
 			pthread_mutex_lock (&desc->meta_lock);
 			if (desc->metadata) {
 			    free(desc->metadata);
-			}	
-			desc->metadata = (char *)malloc(strlen(p)+1);
+			}
+			desc->metadata = malloc(strlen(p)+1);
 			memcpy(desc->metadata, p, strlen(p));
 			pthread_mutex_unlock (&desc->meta_lock);
 		    } else {
@@ -430,7 +430,7 @@ static void buffer_thread (http_desc_t *desc)
             /* These operations are fast. -> doesn't break reader_read */
 	    /* ---------------- lock buffer ( */
 	    pthread_mutex_lock (&desc->buffer_lock);
-	
+
 	    /* enlarge buffer */
 	    newbuf = malloc (desc->len + (BLOCK_SIZE * 2)); /* HTTP_BLOCK_SIZE */
 	    memcpy (newbuf, desc->buffer, desc->len);
@@ -439,7 +439,7 @@ static void buffer_thread (http_desc_t *desc)
 		    memcpy(newbuf + desc->len + metapos, ibuffer+metapos+metasize, rest - metasize);
 		    readed -= metasize;
 		    desc->buffer_pos = rest - metasize;
-	    } else {    
+	    } else {
 		    memcpy (newbuf + desc->len, ibuffer, readed);
 	    }
 	    /* switch buffers */
@@ -450,7 +450,7 @@ static void buffer_thread (http_desc_t *desc)
 	    /* unlock buffer ) */
 	    pthread_mutex_unlock (&desc->buffer_lock);
 	}
-	
+
 	/* Notice waiting function that the new block of data has arrived */
 	desc->new_datablock = 1;
 	pthread_cond_signal (&desc->new_datablock_signal);
@@ -461,11 +461,11 @@ static void buffer_thread (http_desc_t *desc)
 	    cond_timedwait_relative (&desc->dont_wait_signal, &mut, calc_time_to_wait (desc));
 	    pthread_mutex_unlock (&mut);
 	}
-	
+
 	if (desc->dont_wait)
 	    desc->dont_wait--;
     }
-    
+
     free (ibuffer);
     pthread_exit (NULL);
 } /* end of: buffer_thread */
@@ -478,7 +478,8 @@ static int reconnect (http_desc_t *desc, char *redirect)
     char request [2048];
     char response [10240];
     char *s;
-    int error, error_len;
+    int error;
+    socklen_t error_len;
     struct hostent *hp;
     struct sockaddr_in address;
     fd_set set;
@@ -488,7 +489,7 @@ static int reconnect (http_desc_t *desc, char *redirect)
 
     /* Clear error status */
     desc->error = 0;
-    
+
     /* Stop filling thread */
     if (desc->going) {
 	desc->going = 0;
@@ -496,7 +497,7 @@ static int reconnect (http_desc_t *desc, char *redirect)
 	pthread_cond_signal (&desc->dont_wait_signal);
 	pthread_join (desc->buffer_thread, NULL);
     }
-    
+
     /* Close connection */
     if (desc->sock) {
 	close (desc->sock);
@@ -510,7 +511,7 @@ static int reconnect (http_desc_t *desc, char *redirect)
     }
     desc->begin = 0;
     desc->len = 0;
-    
+
     /* Look up for host IP */
     if (!(hp = gethostbyname (desc->host))) {
 	alsaplayer_error ("HTTP: Couldn't look up host %s.", desc->host);
@@ -523,7 +524,7 @@ static int reconnect (http_desc_t *desc, char *redirect)
 	alsaplayer_error ("HTTP: Couldn't open socket.");
 	return 1;
     }
-    
+
     flags = fcntl (desc->sock, F_GETFL, 0);
     fcntl (desc->sock, F_SETFL, flags | O_NONBLOCK);
 
@@ -545,7 +546,7 @@ static int reconnect (http_desc_t *desc, char *redirect)
     tv.tv_usec = 0;
     FD_ZERO (&set);
     FD_SET (desc->sock, &set);
-    
+
     if (select (desc->sock+1, NULL, &set, NULL, &tv) < 1) {
 	alsaplayer_error ("HTTP: Connection is too slow.");
 	return 1;
@@ -569,10 +570,11 @@ static int reconnect (http_desc_t *desc, char *redirect)
 			     "\r\n",
 			     desc->path, desc->host, PACKAGE, VERSION,
 			     desc->pos);
-    //alsaplayer_error("%s", request); 
+    //alsaplayer_error("%s", request);
     write (desc->sock, request, strlen (request));
-    desc->begin = desc->buffer_pos = desc->pos;
- 
+    desc->begin = desc->pos;
+    desc->buffer_pos = 0;
+
     /* Get response */
     if (get_response_head (desc->sock, response, 10240))
 	return 1;
@@ -591,18 +593,18 @@ static int reconnect (http_desc_t *desc, char *redirect)
     } else if (!strncmp (response, "ICY 404", 7)) {
 	 rc = 404;
     } else {
-	if (strlen(response)) {	
+	if (strlen(response)) {
 		alsaplayer_error ("HTTP: Wrong server protocol for http://%s:%u%s",
 			desc->host, desc->port, desc->path);
 		alsaplayer_error("ERROR:\n\"%s\"", response);
-	}	
+	}
 	return 1;
     }
 
     /* Check return code */
     if (strstr(response, "HTTP"))
     	rc = atoi (response + 9);
-    
+
     if (rc != 200 && rc != 206) {
 	/* Wrong code */
 	if (rc == 404) {
@@ -619,28 +621,28 @@ static int reconnect (http_desc_t *desc, char *redirect)
 			    /* Parse redirect */
 			    if (sscanf(s, "Location: %[^\r]", redirect)) {
 				    /* alsaplayer_error("Redirection: %s", redirect); */
-			    }		    
+			    }
 		    }
 		    return 1;
 	    }
 	} else if (rc == 400) {
 		if (desc->status) {
 			desc->status(desc->data, "Server is full");
-		}	
+		}
 		if (redirect)
 			redirect[0] = 0;
 		return 1;
 	} else if (rc == 401) {
 		if (desc->status) {
 			desc->status(desc->data, "Unauthorized access");
-		}	
+		}
 		if (redirect)
 			redirect[0] = 0;
 		return 1;
 	} else if (rc == 404) {
 		if (desc->status) {
 			desc->status(desc->data, "Resource not found");
-		}	
+		}
 		if (redirect)
 			redirect[0] = 0;
 		return 1;
@@ -648,7 +650,7 @@ static int reconnect (http_desc_t *desc, char *redirect)
 		/* unknown */
 		alsaplayer_error ("HTTP: We don't support %d response code: http://%s:%u%s",
 				rc, desc->host, desc->port, desc->path);
-		if (redirect) 
+		if (redirect)
 			redirect[0] = 0;
 		return 1;
 	}
@@ -669,11 +671,11 @@ static int reconnect (http_desc_t *desc, char *redirect)
 	//alsaplayer_error("Found metaint: %d", desc->icy_metaint);
     } else {
     	desc->icy_metaint = 0;
-    }	
-    
+    }
+
     /* Setup played parts */
     desc->played_parts = desc->seekable ? 5 : 0;
-    
+
     /* Attach thread to fill a buffer */
     desc->going = 1;
     pthread_create (&desc->buffer_thread, NULL, (void* (*)(void *)) buffer_thread, desc);
@@ -689,7 +691,7 @@ static int reconnect (http_desc_t *desc, char *redirect)
 static void http_close(void *d)
 {
     http_desc_t *desc = (http_desc_t*)d;
-    
+
     /* stop buffering thread */
     if (desc->going) {
 	desc->going = 0;
@@ -697,14 +699,14 @@ static void http_close(void *d)
 	pthread_cond_signal (&desc->dont_wait_signal);
 	pthread_join (desc->buffer_thread, NULL);
     }
-    
+
     /* free resources */
     if (desc->host)  free (desc->host);
     if (desc->path)  free (desc->path);
     if (desc->metadata) free (desc->metadata);
     if (desc->sock)  close (desc->sock);
     if (desc->buffer)  free (desc->buffer);
-    
+
     free (desc);
 } /* end of http_close */
 
@@ -715,7 +717,7 @@ static void *http_open(const char *uri, reader_status_type status, void *data)
     http_desc_t *desc;
     char redirect[10240];
     int tries = 0;
- 
+
     /* Alloc descripor and init members. */
     desc = malloc (sizeof (http_desc_t));
     desc->going = 0;
@@ -728,8 +730,10 @@ static void *http_open(const char *uri, reader_status_type status, void *data)
     desc->buffer_pos = 0;
     desc->begin = 0;
     desc->len = 0;
-    desc->direction = 0; 
+    desc->direction = 0;
     desc->metadata = NULL;
+    desc->host = NULL;
+    desc->path = NULL;
     desc->status = status;
     desc->data = data;
     pthread_mutex_init (&desc->buffer_lock, NULL);
@@ -746,7 +750,7 @@ static void *http_open(const char *uri, reader_status_type status, void *data)
     /* Connect */
     if (desc->status) {
 	    desc->status(desc->data, "Connecting");
-    }	    
+    }
     while (tries++ < 5) {
 	redirect[0] = 0;
     	if (reconnect (desc, redirect)) {
@@ -777,19 +781,19 @@ static float http_can_handle(const char *uri)
 
 /* ******************************************************************* */
 /* init plugin                                                         */
-static int http_init()
+static int http_init(void)
 {
     http_buffer_size = prefs_get_int (ap_prefs, "http", "buffer_size", DEFAULT_HTTP_BUFFER_SIZE);
 
     /* Be sure http.buffer_size is available in config */
     prefs_set_int (ap_prefs, "http", "buffer_size", http_buffer_size);
-    
+
     return 1;
 }
 
 /* ******************************************************************* */
 /* shutdown plugin                                                     */
-static void http_shutdown()
+static void http_shutdown(void)
 {
     return;
 }
@@ -803,11 +807,11 @@ static size_t http_metadata (void *d, size_t size, void *data)
 	size_t msize;
 
 	pthread_mutex_lock (&desc->meta_lock);
-	
+
 	if (!desc->metadata) {
 		pthread_mutex_unlock (&desc->meta_lock);
 		return 0;
-	}	
+	}
 	msize = strlen(desc->metadata);
 	if (msize > size)
 		msize = size;
@@ -826,16 +830,18 @@ static size_t http_read (void *ptr, size_t size, void *d)
     int tocopy;		    /* how much bytes we got? */
 
     /* Init temp mutex */
-    pthread_mutex_init (&mut, NULL);   
+    pthread_mutex_init (&mut, NULL);
 
     /* check for reopen */
-    if (desc->begin > desc->pos || desc->begin + desc->len + 3*HTTP_BLOCK_SIZE < desc->pos)
+    if (desc->begin > desc->pos 
+	    || desc->begin + desc->len + 3*HTTP_BLOCK_SIZE < desc->pos
+	    || (!desc->going && desc->pos < desc->size))
 	reconnect (desc, NULL);
- 
+
     /* wait while the buffer will has entire block */
     while (1) {
-	int readed;
-	
+	size_t readed;
+
 	/* check for error */
 	if (desc->error) {
 	    return 0;
@@ -844,7 +850,7 @@ static size_t http_read (void *ptr, size_t size, void *d)
 	/* We will work with buffer... we have to lock it! */
 	pthread_mutex_lock (&desc->buffer_lock);
 	readed = desc->begin + desc->len - desc->pos;
-	
+
 	/* done? */
 	if (readed > 0 && readed >= size) {
 	    tocopy = size;
@@ -862,14 +868,14 @@ static size_t http_read (void *ptr, size_t size, void *d)
 	    tocopy = 0;
 	    break;
 	}
-	
+
 	/* break waiting */
 	desc->dont_wait = 1;
 	pthread_cond_signal (&desc->dont_wait_signal);
-	
+
 	/* Allow buffer_thread to use buffer */
         pthread_mutex_unlock (&desc->buffer_lock);
-    
+
 	/* Wait for next portion of data */
 	if (!desc->new_datablock) {
 	    pthread_mutex_lock (&mut);
@@ -888,7 +894,7 @@ static size_t http_read (void *ptr, size_t size, void *d)
 	/* let them know about new state, heh */
 	status_notify (desc);
     }
-    
+
     /* Allow buffer_thread to use buffer. */
     pthread_mutex_unlock (&desc->buffer_lock);
 
@@ -901,16 +907,16 @@ static size_t http_read (void *ptr, size_t size, void *d)
 static int http_seek (void *d, long offset, int whence)
 {
     http_desc_t *desc = (http_desc_t*)d;
-  
+
     if (!desc->seekable)  return -1;
-    
+
     if (whence == SEEK_SET)
 	desc->pos = offset;
     else if (whence == SEEK_END)
 	desc->pos = desc->size + offset;
-    else 
+    else
 	desc->pos += offset;
- 
+
     return 0;
 }
 
@@ -926,14 +932,14 @@ static long http_tell (void *d)
 
 /* ******************************************************************* */
 /* directory test                                                      */
-static float http_can_expand (const char *uri)
+static float http_can_expand (const char * UNUSED (uri))
 {
     return 0.0;
 }
 
 /* ******************************************************************* */
 /* expand directory                                                    */
-static char **http_expand (const char *uri)
+static char **http_expand (const char * UNUSED (uri))
 {
     return NULL;
 }
@@ -950,10 +956,10 @@ static int http_seekable (void *d)
 static long http_length (void *d)
 {
     http_desc_t *desc = (http_desc_t*)d;
-    
+
     if (!desc->seekable)
 	return -1;
-    
+
     return desc->size;
 }
 
@@ -964,7 +970,7 @@ static int http_eof (void *d)
 
     if (!desc->seekable)
 	return 0;
-    
+
     return !(desc->pos < desc->size);
 }
 
